@@ -20,7 +20,10 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
