@@ -20,7 +20,10 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -55,7 +58,7 @@ cim_init_logger(const char * entity)
 
 	cl_log_set_entity(entity);
 	cl_log_enable_stderr(debug_level?TRUE:FALSE);
-	cl_log_set_facility(HA_LOG_FACILITY);
+	cl_log_set_facility(LOG_DAEMON);
         return HA_OK;
 }
 
