@@ -1,3 +1,4 @@
+/* $Id: mlock.c,v 1.13 2005/07/29 03:15:42 sunjd Exp $ */
 /*
  *
  * multi-clients NFS lock test code
@@ -20,7 +21,6 @@
  */
 
 
-#include <lha_internal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -1033,7 +1033,7 @@ main (int argc, char**  argv)
 	gi.testwarn = 0;
 	gi.testfail = 0;
 	
-	strlcpy(gi.filename, filename, sizeof(gi.filename));
+	strcpy(gi.filename, filename);
 	
 	init_comm(servername);
 	
