@@ -1,3 +1,4 @@
+/* $Id: crm_resource.c,v 1.45 2006/08/04 09:50:02 andrew Exp $ */
 
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
@@ -17,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 
@@ -49,11 +50,11 @@
 #ifdef HAVE_GETOPT_H
 #  include <getopt.h>
 #endif
+#include <crm/dmalloc_wrapper.h>
 void usage(const char *cmd, int exit_status);
 
 gboolean do_force = FALSE;
 gboolean BE_QUIET = FALSE;
-const char *attr_set_type = XML_TAG_ATTR_SETS;
 char *host_id = NULL;
 const char *rsc_id = NULL;
 const char *host_uname = NULL;
@@ -63,18 +64,12 @@ const char *prop_value = NULL;
 const char *rsc_type = NULL;
 const char *prop_id = NULL;
 const char *prop_set = NULL;
-char *migrate_lifetime = NULL;
 char rsc_cmd = 'L';
 char *our_pid = NULL;
 IPC_Channel *crmd_channel = NULL;
 char *xml_file = NULL;
-int cib_options = cib_sync_call;
 
-#define OPTARGS	"V?LRQxDCPp:WMUr:H:v:t:p:g:d:i:s:G:S:fX:lmu:F"
-#define CMD_ERR(fmt, args...) do {		\
-	crm_warn(fmt, ##args);			\
-	fprintf(stderr, fmt, ##args);		\
-    } while(0)
+#define OPTARGS	"V?LRQxDCPp:WMUr:H:v:t:p:g:d:i:s:G:S:fX:l"
 
 static int
 do_find_resource(const char *rsc, pe_working_set_t *data_set)
@@ -90,7 +85,7 @@ do_find_resource(const char *rsc, pe_working_set_t *data_set)
 		   crm_debug_3("resource %s is running on: %s",
 			       rsc, node->details->uname);
 		   if(BE_QUIET) {
-			   fprintf(stdout, "%s\n", node->details->uname);
+			   fprintf(stdout, "%s\n ", node->details->uname);
 		   } else {
 			   fprintf(stdout, "resource %s is running on: %s\n",
 				   rsc, node->details->uname);
@@ -134,8 +129,7 @@ do_find_resource_list(pe_working_set_t *data_set, gboolean raw)
 			print_raw_rsc(rsc, 0);
 			continue;
 			
-		} else if(is_set(rsc->flags, pe_rsc_orphan)
-			  && rsc->fns->active(rsc, TRUE) == FALSE) {
+		} else if(rsc->orphan && rsc->fns->active(rsc, TRUE) == FALSE) {
 			continue;
 		}
 		rsc->fns->print(
@@ -187,13 +181,13 @@ dump_resource_attr(
 		current = the_rsc->running_on->data;
 
 	} else if(g_list_length(the_rsc->running_on) > 1) {
-		CMD_ERR("%s is active on more than one node,"
+		fprintf(stderr, "%s is active on more than one node,"
 			" returning the default value for %s\n",
-			the_rsc->id, crm_str(value));
+			the_rsc->id, value);
 	} 
 	
 	unpack_instance_attributes(
-		the_rsc->xml, attr_set_type, current?current->details->attrs:NULL,
+		the_rsc->xml, XML_TAG_ATTR_SETS, current?current->details->attrs:NULL,
 		the_rsc->parameters, NULL, data_set->now);
 
 	if(the_rsc->parameters != NULL) {
@@ -214,7 +208,8 @@ set_resource_attr(const char *rsc_id, const char *attr_set, const char *attr_id,
 {
 	int rc = cib_ok;
 	int matches = 0;
-	
+	int cib_options = cib_sync_call;
+
 	char *local_attr_id = NULL;
 	char *local_attr_set = NULL;
 	
@@ -225,40 +220,40 @@ set_resource_attr(const char *rsc_id, const char *attr_set, const char *attr_id,
 
 	resource_t *rsc = pe_find_resource(data_set->resources, rsc_id);
 
+	if(do_force) {
+		crm_debug("Forcing...");
+		cib_options |= cib_scope_local|cib_quorum_override;
+	}
+			
 	if(rsc == NULL) {
 		return cib_NOTEXISTS;
 	}
 
-	/* filter by set and type */
-	matches = find_xml_children(
-		&set_children, rsc->xml, 
-		attr_set_type, XML_ATTR_ID, attr_set, FALSE);
-	crm_log_xml_debug(set_children, "search by set:");
-
-	crm_debug("%d objects matching tag=%s id=%s",
-		  matches, attr_set_type, attr_set?attr_set:"<any>");
-	
-	if(matches == 0) {
-		/* nothing more to search */
-		crm_debug("No objects matching tag=%s id=%s",
-			  attr_set_type, attr_set?attr_set:"<any>");
-		
-	} else if(attr_id == NULL) {
+	/* filter by set name */
+	if(attr_set != NULL) {
 		matches = find_xml_children(
-			&nv_children, set_children,
-			XML_CIB_TAG_NVPAIR, XML_NVPAIR_ATTR_NAME, attr_name, FALSE);
+			&set_children, rsc->xml, 
+			XML_TAG_ATTR_SETS, XML_ATTR_ID, attr_set);
+		crm_log_xml_debug(set_children, "search by set:");
+	}
+
+	matches = 0;
+	if(attr_id == NULL) {
+		matches = find_xml_children(
+			&nv_children, set_children?set_children:rsc->xml,
+			XML_CIB_TAG_NVPAIR, XML_NVPAIR_ATTR_NAME, attr_name);
 		crm_log_xml_debug(nv_children, "search by name:");
 
-	} else {
+	} else if(attr_id != NULL) {
 		matches = find_xml_children(
-			&nv_children, set_children,
-			XML_CIB_TAG_NVPAIR, XML_ATTR_ID, attr_id, FALSE);
+			&nv_children, set_children?set_children:rsc->xml,
+			XML_CIB_TAG_NVPAIR, XML_ATTR_ID, attr_id);
 		crm_log_xml_debug(nv_children, "search by id:");
 	}
 	
 	
 	if(matches > 1) {
-		CMD_ERR("Multiple attributes match name=%s for the resource %s:\n",
+		fprintf(stderr, "Multiple attributes match name=%s for the resource %s:\n",
 			attr_name, rsc->id);
 
 		if(set_children == NULL) {
@@ -266,14 +261,14 @@ set_resource_attr(const char *rsc_id, const char *attr_set, const char *attr_id,
 			set_children = NULL;
 			find_xml_children(
 				&set_children, rsc->xml, 
-				attr_set_type, NULL, NULL, FALSE);
+				XML_TAG_ATTR_SETS, NULL, NULL);
 			xml_child_iter(
 				set_children, set,
 				free_xml(nv_children);
 				nv_children = NULL;
 				find_xml_children(
 					&nv_children, set,
-					XML_CIB_TAG_NVPAIR, XML_NVPAIR_ATTR_NAME, attr_name, FALSE);
+					XML_CIB_TAG_NVPAIR, XML_NVPAIR_ATTR_NAME, attr_name);
 				xml_child_iter(
 					nv_children, child,
 					fprintf(stderr,"  Set: %s,\tValue: %s,\tID: %s\n",
@@ -292,26 +287,22 @@ set_resource_attr(const char *rsc_id, const char *attr_set, const char *attr_id,
 		}
 		
 		if(BE_QUIET == FALSE) {
-			CMD_ERR("\nThe following text can be suppressed with the -Q option:\n");
+			fprintf(stderr, "\nThe following text can be suppressed with the -Q option:\n");
 			if(attr_set == NULL) {
-				CMD_ERR("  * To choose an existing entry to change, please supply one of the set names above using the -s option.\n");
+				fprintf(stderr, "  * To choose an existing entry to change, please supply one of the set names above using the -s option.\n");
 			} else {
-				CMD_ERR("  * To choose an existing entry to change, please supply one of the IDs above using the -i option.\n");			
+				fprintf(stderr, "  * To choose an existing entry to change, please supply one of the IDs above using the -i option.\n");			
 			}
-			CMD_ERR("  * To create a new value with a default ID, please supply a different set name using the -s option.\n");
+			fprintf(stderr, "  * To create a new value with a default ID, please supply a different set name using the -s option.\n");
 			
-			CMD_ERR("You can also use --query-xml to display the complete resource definition.\n");
+			fprintf(stderr, "You can also use --query-xml to display the complete resource definition.\n");
 		}
 		
 		return cib_unknown;
 		
 	} else if(matches == 0) {
 		if(attr_set == NULL) {
-			if(safe_str_eq(attr_set_type, XML_TAG_META_SETS)) {
-				local_attr_set = crm_concat(rsc->id, "meta-options", '-');
-			} else {
-				local_attr_set = crm_strdup(rsc->id);
-			}
+			local_attr_set = crm_strdup(rsc->id);
 			attr_set = local_attr_set;
 		}
 		if(attr_id == NULL) {
@@ -322,7 +313,7 @@ set_resource_attr(const char *rsc_id, const char *attr_set, const char *attr_id,
 		xml_top = create_xml_node(NULL, crm_element_name(rsc->xml));
 		crm_xml_add(xml_top, XML_ATTR_ID, rsc->id);
 		
-		xml_obj = create_xml_node(xml_top, attr_set_type);
+		xml_obj = create_xml_node(xml_top, XML_TAG_ATTR_SETS);
 		crm_xml_add(xml_obj, XML_ATTR_ID, attr_set);
 		
 		xml_obj = create_xml_node(xml_obj, XML_TAG_ATTRS);
@@ -357,30 +348,29 @@ delete_resource_attr(
 	const char *rsc_id, const char *attr_set, const char *attr_id,
 	const char *attr_name, cib_t *cib, pe_working_set_t *data_set)
 {
-	crm_data_t *xml_obj = NULL;
-	crm_data_t *xml_match = NULL;
-
 	int rc = cib_ok;
-	char *local_attr_id = NULL;
+	int cib_options = cib_sync_call;
+	crm_data_t *xml_obj = NULL;
 	resource_t *rsc = pe_find_resource(data_set->resources, rsc_id);
+	char *local_attr_id = NULL;
+	char *local_attr_set = NULL;
 
+	if(do_force) {
+		crm_debug("Forcing...");
+		cib_options |= cib_scope_local|cib_quorum_override;
+	}
+			
 	if(rsc == NULL) {
 		return cib_NOTEXISTS;
 	}
 
- 	rc = find_attr_details(
-	    rsc->xml, NULL, attr_set, attr_id, attr_name, &xml_match, TRUE);
+	if(attr_set == NULL) {
+		local_attr_set = crm_strdup(rsc->id);
+		attr_set = local_attr_set;
+	}
 
-	if(rc == cib_NOTEXISTS) {
-	    return cib_ok;
-	}
-	
-	if(rc != cib_ok) {
-	    return rc;
-	}
-	
 	if(attr_id == NULL) {
-		local_attr_id = crm_element_value_copy(xml_match, XML_ATTR_ID);
+		local_attr_id = crm_concat(attr_set, attr_name, '-');
 		attr_id = local_attr_id;
 	}
 
@@ -393,9 +383,9 @@ delete_resource_attr(
 	rc = cib->cmds->delete(cib, XML_CIB_TAG_RESOURCES, xml_obj, NULL,
 			       cib_options);
 
-	free_xml(xml_obj);
-	free_xml(xml_match);
 	crm_free(local_attr_id);
+	crm_free(local_attr_set);
+	free_xml(xml_obj);
 	return rc;
 }
 
@@ -438,7 +428,6 @@ crmd_msg_callback(IPC_Channel * server, void *private_data)
 	       && server->ops->is_message_pending(server) == TRUE) {
 		if(new_input != NULL) {
 			delete_ha_msg_input(new_input);
-			new_input = NULL;
 		}
 		
 		if (server->ops->recv(server, &msg) != IPC_OK) {
@@ -460,9 +449,16 @@ crmd_msg_callback(IPC_Channel * server, void *private_data)
 			    new_input->msg, crm_system_name, our_pid,
 			    XML_ATTR_RESPONSE) == FALSE) {
 			crm_info("Message was not a CRM response. Discarding.");
+			continue;
 		}
-		delete_ha_msg_input(new_input);
-		new_input = NULL;		
+
+/* 		result = cl_get_string(new_input->msg, XML_ATTR_RESULT); */
+/* 		if(result == NULL || strcasecmp(result, "ok") == 0) { */
+/* 			result = "pass"; */
+/* 		} else { */
+/* 			result = "fail"; */
+/* 		} */
+		
 	}
 
 	if (server->ch_status == IPC_DISCONNECT) {
@@ -474,98 +470,42 @@ crmd_msg_callback(IPC_Channel * server, void *private_data)
 }
 
 static int
-send_lrm_rsc_op(IPC_Channel *crmd_channel, const char *op,
-		const char *host_uname, const char *rsc_id,
-		gboolean only_failed, pe_working_set_t *data_set)
+delete_lrm_rsc(
+	IPC_Channel *crmd_channel, const char *host_uname,
+	const char *rsc_id, const char *rsc_long_id)
 {
-	char *key = NULL;
 	int rc = cib_send_failed;
 	HA_Message *cmd = NULL;
-	crm_data_t *xml_rsc = NULL;
-	const char *value = NULL;
-	HA_Message *params = NULL;
 	crm_data_t *msg_data = NULL;
-	resource_t *rsc = pe_find_resource(data_set->resources, rsc_id);
-
-	if(rsc == NULL) {
-		CMD_ERR("Resource %s not found\n", rsc_id);
-		return cib_NOTEXISTS;
-
-	} else if(rsc->variant != pe_native) {
-		CMD_ERR("We can only process primitive resources, not %s\n", rsc_id);
-		return cib_invalid_argument;
-
-	} else if(host_uname == NULL) {
-		CMD_ERR("Please supply a hostname with -H\n");
-		return cib_invalid_argument;
-	}
+	crm_data_t *rsc = NULL;
+	HA_Message *params = NULL;
+	char *key = crm_concat(crm_system_name, our_pid, '-');
 	
-	key = crm_concat("0:0:crm-resource", our_pid, '-');
+	CRM_DEV_ASSERT(rsc_id != NULL);
 	
 	msg_data = create_xml_node(NULL, XML_GRAPH_TAG_RSC_OP);
 	crm_xml_add(msg_data, XML_ATTR_TRANSITION_KEY, key);
 	
-	xml_rsc = create_xml_node(msg_data, XML_CIB_TAG_RESOURCE);
-	crm_xml_add(xml_rsc, XML_ATTR_ID, rsc->id);
-	crm_xml_add(xml_rsc, XML_ATTR_ID_LONG, rsc->long_name);
-
-	value = crm_element_value(rsc->xml, XML_ATTR_TYPE);
-	crm_xml_add(xml_rsc, XML_ATTR_TYPE, value);
-	if(value == NULL) {
-		CMD_ERR("%s has no type!  Aborting...\n", rsc_id);
-		return cib_NOTEXISTS;
-	}
-
-	value = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
-	crm_xml_add(xml_rsc, XML_AGENT_ATTR_CLASS, value);
-	if(value == NULL) {
-		CMD_ERR("%s has no class!  Aborting...\n", rsc_id);
-		return cib_NOTEXISTS;
-	}
-
-	value = crm_element_value(rsc->xml, XML_AGENT_ATTR_PROVIDER);
-	crm_xml_add(xml_rsc, XML_AGENT_ATTR_PROVIDER, value);
+	rsc = create_xml_node(msg_data, XML_CIB_TAG_RESOURCE);
+	crm_xml_add(rsc, XML_ATTR_ID, rsc_id);
+	crm_xml_add(rsc, XML_ATTR_ID_LONG, rsc_long_id);
 
 	params = create_xml_node(msg_data, XML_TAG_ATTRS);
 	crm_xml_add(params, XML_ATTR_CRM_VERSION, CRM_FEATURE_SET);
-	crm_xml_add(params, CRM_META"_"XML_LRM_ATTR_INTERVAL, "60000"); /* 1 minute */
 	
-	cmd = create_request(op, msg_data, host_uname,
+	cmd = create_request(CRM_OP_LRM_DELETE, msg_data, host_uname,
 			     CRM_SYSTEM_CRMD, crm_system_name, our_pid);
 
-/* 	crm_log_xml_warn(cmd, "send_lrm_rsc_op"); */	
 	free_xml(msg_data);
 	crm_free(key);
 
 	if(send_ipc_message(crmd_channel, cmd)) {
-	    rc = 0;
-	    sleep(1); /* dont exit striaght away, give the crmd time
-		       * to process our request
-		       */
-	} else {
-	    CMD_ERR("Could not send %s op to the crmd", op);
+		rc = 0;
 	}
-	
 	crm_msg_del(cmd);
 	return rc;
 }
 
-
-static int
-delete_lrm_rsc(IPC_Channel *crmd_channel, const char *host_uname,
-	       const char *rsc_id, pe_working_set_t *data_set)
-{
-	return send_lrm_rsc_op(crmd_channel, CRM_OP_LRM_DELETE, host_uname, rsc_id, TRUE, data_set); 
-}
-
-static int
-fail_lrm_rsc(IPC_Channel *crmd_channel, const char *host_uname,
-	     const char *rsc_id, pe_working_set_t *data_set)
-{
-    crm_warn("Failing: %s", rsc_id);
-    return send_lrm_rsc_op(crmd_channel, CRM_OP_LRM_FAIL, host_uname, rsc_id, FALSE, data_set); 
-}
-
 static int
 refresh_lrm(IPC_Channel *crmd_channel, const char *host_uname)  
 {
@@ -588,7 +528,6 @@ migrate_resource(
 	const char *existing_node, const char *preferred_node,
 	cib_t *	cib_conn) 
 {
-	char *later_s = NULL;
 	enum cib_errors rc = cib_ok;
 	char *id = NULL;
 	crm_data_t *cib = NULL;
@@ -596,8 +535,7 @@ migrate_resource(
 	crm_data_t *expr = NULL;
 	crm_data_t *constraints = NULL;
 	crm_data_t *fragment = NULL;
-	crm_data_t *lifetime = NULL;
-	
+
 	crm_data_t *can_run = NULL;
 	crm_data_t *dont_run = NULL;
 
@@ -617,46 +555,15 @@ migrate_resource(
 	crm_xml_add(dont_run, XML_ATTR_ID, id);
 	crm_free(id);
 
-	if(migrate_lifetime) {
-		char *life = crm_strdup(migrate_lifetime);
-		char *life_mutable = life;
-		
-		ha_time_t *now = NULL;
-		ha_time_t *later = NULL;
-		ha_time_t *duration = parse_time_duration(&life_mutable);
-		
-		if(duration == NULL) {
-			CMD_ERR("Invalid duration specified: %s\n",
-				migrate_lifetime);
-			CMD_ERR("Please refer to"
-				" http://en.wikipedia.org/wiki/ISO_8601#Duration"
-				" for examples of valid durations\n");
-			crm_free(life);
-			return cib_invalid_argument;
-		}
-		now = new_ha_date(TRUE);
-		later = add_time(now, duration);
-		log_date(LOG_INFO, "now     ", now, ha_log_date|ha_log_time);
-		log_date(LOG_INFO, "later   ", later, ha_log_date|ha_log_time);
-		log_date(LOG_INFO, "duration", duration, ha_log_date|ha_log_time|ha_log_local);
-		later_s = date_to_string(later, ha_log_date|ha_log_time);
-		printf("Migration will take effect until: %s\n", later_s);
-
-		free_ha_date(duration);
-		free_ha_date(later);
-		free_ha_date(now);
-		crm_free(life);
-	}
-	
 	if(existing_node == NULL) {
 		crm_log_xml_notice(can_run, "Deleting");
 		rc = cib_conn->cmds->delete(cib_conn, XML_CIB_TAG_CONSTRAINTS,
-					    dont_run, NULL, cib_options);
+					    dont_run, NULL, cib_sync_call);
 		if(rc == cib_NOTEXISTS) {
 			rc = cib_ok;
 
 		} else if(rc != cib_ok) {
-			goto bail;
+			return rc;
 		}
 
 	} else {
@@ -666,34 +573,16 @@ migrate_resource(
 				" with a score of -INFINITY for resource %s"
 				" on %s.\n",
 				ID(dont_run), rsc_id, existing_node);
-			CMD_ERR("\tThis will prevent %s from running"
+			fprintf(stderr, "\tThis will prevent %s from running"
 				" on %s until the constraint is removed using"
 				" the 'crm_resource -U' command or manually"
 				" with cibadmin\n", rsc_id, existing_node);
-			CMD_ERR("\tThis will be the case even if %s is"
+			fprintf(stderr, "\tThis will be the case even if %s is"
 				" the last node in the cluster\n", existing_node);
-			CMD_ERR("\tThis messgae can be disabled with -Q\n");
+			fprintf(stderr, "\tThis messgae can be disabled with -Q\n");
 		}
 		
 		crm_xml_add(dont_run, "rsc", rsc_id);
-
-		if(later_s) {
-			lifetime = create_xml_node(dont_run, "lifetime");
-
-			rule = create_xml_node(lifetime, XML_TAG_RULE);
-			id = crm_concat("cli-standby-lifetime", rsc_id, '-');
-			crm_xml_add(rule, XML_ATTR_ID, id);
-			crm_free(id);
-
-			expr = create_xml_node(rule, "date_expression");
-			id = crm_concat("cli-standby-lifetime-end",rsc_id,'-');
-			crm_xml_add(expr, XML_ATTR_ID, id);
-			crm_free(id);			
-
-			crm_xml_add(expr, "operation", "lt");
-			crm_xml_add(expr, "end", later_s);
-		}
-		
 		rule = create_xml_node(dont_run, XML_TAG_RULE);
 		expr = create_xml_node(rule, XML_TAG_EXPRESSION);
 		id = crm_concat("cli-standby-rule", rsc_id, '-');
@@ -710,41 +599,23 @@ migrate_resource(
 		crm_xml_add(expr, XML_EXPR_ATTR_OPERATION, "eq");
 		crm_xml_add(expr, XML_EXPR_ATTR_VALUE, existing_node);
 		crm_xml_add(expr, XML_EXPR_ATTR_TYPE, "string");
-		
+
 		add_node_copy(constraints, dont_run);
 	}
 	
 	if(preferred_node == NULL) {
 		crm_log_xml_notice(can_run, "Deleting");
 		rc = cib_conn->cmds->delete(cib_conn, XML_CIB_TAG_CONSTRAINTS,
-					    can_run, NULL, cib_options);
+					    can_run, NULL, cib_sync_call);
 		if(rc == cib_NOTEXISTS) {
 			rc = cib_ok;
 
 		} else if(rc != cib_ok) {
-			goto bail;
+			return rc;
 		}
 
 	} else {
 		crm_xml_add(can_run, "rsc", rsc_id);
-
-		if(later_s) {
-			lifetime = create_xml_node(can_run, "lifetime");
-
-			rule = create_xml_node(lifetime, XML_TAG_RULE);
-			id = crm_concat("cli-prefer-lifetime", rsc_id, '-');
-			crm_xml_add(rule, XML_ATTR_ID, id);
-			crm_free(id);
-
-			expr = create_xml_node(rule, "date_expression");
-			id = crm_concat("cli-prefer-lifetime-end", rsc_id, '-');
-			crm_xml_add(expr, XML_ATTR_ID, id);
-			crm_free(id);			
-
-			crm_xml_add(expr, "operation", "lt");
-			crm_xml_add(expr, "end", later_s);
-		}
-
 		rule = create_xml_node(can_run, XML_TAG_RULE);
 		expr = create_xml_node(rule, XML_TAG_EXPRESSION);
 		id = crm_concat("cli-prefer-rule", rsc_id, '-');
@@ -761,25 +632,22 @@ migrate_resource(
 		crm_xml_add(expr, XML_EXPR_ATTR_OPERATION, "eq");
 		crm_xml_add(expr, XML_EXPR_ATTR_VALUE, preferred_node);
 		crm_xml_add(expr, XML_EXPR_ATTR_TYPE, "string");
-		
+
 		add_node_copy(constraints, can_run);
 	}
 
 	if(preferred_node != NULL || existing_node != NULL) {
 		crm_log_xml_notice(fragment, "CLI Update");
 		rc = cib_conn->cmds->update(cib_conn, XML_CIB_TAG_CONSTRAINTS,
-					    fragment, NULL, cib_options);
+					    fragment, NULL, cib_sync_call);
 	}
-
-  bail:
+	
 	free_xml(fragment);
 	free_xml(dont_run);
 	free_xml(can_run);
-	crm_free(later_s);
 	return rc;
 }
 
-
 int
 main(int argc, char **argv)
 {
@@ -811,10 +679,7 @@ main(int argc, char **argv)
 		{"un-migrate", 0, 0, 'U'},
 		{"resource",   1, 0, 'r'},
 		{"host-uname", 1, 0, 'H'},
-		{"lifetime",   1, 0, 'u'},
-		{"fail",       0, 0, 'F'},
 		{"force",      0, 0, 'f'},
-		{"meta",       0, 0, 'm'},
 
 		{"set-parameter",   1, 0, 'p'},
 		{"get-parameter",   1, 0, 'g'},
@@ -831,13 +696,15 @@ main(int argc, char **argv)
 #endif
 
 	crm_system_name = basename(argv[0]);
+	cl_log_set_entity(crm_system_name);
+	cl_log_set_facility(LOG_USER);
+	set_crm_log_level(LOG_ERR);
+	cl_log_enable_stderr(TRUE);
 	
 	if(argc < 2) {
 		usage(crm_system_name, LSB_EXIT_EINVAL);
 	}
 	
-	crm_log_init(crm_system_name, LOG_WARNING, FALSE, FALSE, argc, argv);
-
 	while (1) {
 #ifdef HAVE_GETOPT_H
 		flag = getopt_long(argc, argv, OPTARGS,
@@ -862,16 +729,12 @@ main(int argc, char **argv)
 			case 'Q':
 				BE_QUIET = TRUE;
 				break;
-			case 'm':
-				attr_set_type = XML_TAG_META_SETS;
-				break;
-				
+
 			case 'L':
 			case 'l':
 			case 'R':
 			case 'x':
 			case 'D':
-			case 'F':
 			case 'C':
 			case 'P':
 			case 'W':
@@ -880,10 +743,6 @@ main(int argc, char **argv)
 				rsc_cmd = flag;
 				break;
 				
-			case 'u':
-				migrate_lifetime = crm_strdup(optarg);
-				break;
-				
 			case 'p':
 				crm_debug_2("Option %c => %s", flag, optarg);
 				prop_name = optarg;
@@ -946,18 +805,18 @@ main(int argc, char **argv)
 				break;
 				
 			default:
-				CMD_ERR("Argument code 0%o (%c) is not (?yet?) supported\n", flag, flag);
+				fprintf(stderr, "Argument code 0%o (%c) is not (?yet?) supported\n", flag, flag);
 				++argerr;
 				break;
 		}
 	}
 
 	if (optind < argc) {
-		CMD_ERR("non-option ARGV-elements: ");
+		fprintf(stderr, "non-option ARGV-elements: ");
 		while (optind < argc) {
-			CMD_ERR("%s ", argv[optind++]);
+			fprintf(stderr, "%s ", argv[optind++]);
 		}
-		CMD_ERR("\n");
+		fprintf(stderr, "\n");
 	}
 
 	if (optind > argc) {
@@ -974,25 +833,11 @@ main(int argc, char **argv)
 		our_pid[10] = '\0';
 	}
 
-	if(do_force) {
-		crm_debug("Forcing...");
-		cib_options |= cib_scope_local|cib_quorum_override;
-	}
-
-	if(rsc_cmd == 'L'
-	   || rsc_cmd == 'W'
-	   || rsc_cmd == 'D'
-	   || rsc_cmd == 'x'
-	   || rsc_cmd == 'M'
-	   || rsc_cmd == 'U'
-	   || rsc_cmd == 'C' 
-	   || rsc_cmd == 'F' 
-	   || rsc_cmd == 'p'
-	   || rsc_cmd == 'd'
-	   || rsc_cmd == 'g'
-	   || rsc_cmd == 'G'
-	   || rsc_cmd == 'S'
-	   || rsc_cmd == 'l') {
+	if(rsc_cmd == 'L' || rsc_cmd == 'W' || rsc_cmd == 'D' || rsc_cmd == 'x'
+	   || rsc_cmd == 'M' || rsc_cmd == 'U' || rsc_cmd == 'C' 
+	   || rsc_cmd == 'p' || rsc_cmd == 'd' || rsc_cmd == 'g'
+	   || rsc_cmd == 'G' || rsc_cmd == 'S' || rsc_cmd == 'l') {
+		
 		resource_t *rsc = NULL;
 		if(xml_file != NULL) {
 			FILE *xml_strm = fopen(xml_file, "r");
@@ -1001,16 +846,13 @@ main(int argc, char **argv)
 			} else {
 				cib_xml_copy = file2xml(xml_strm, FALSE);
 			}
-			if(xml_strm != NULL) {
-				fclose(xml_strm);
-			}
 
 		} else {
 			cib_conn = cib_new();
 			rc = cib_conn->cmds->signon(
 				cib_conn, crm_system_name, cib_command_synchronous);
 			if(rc != cib_ok) {
-				CMD_ERR("Error signing on to the CIB service: %s\n",
+				fprintf(stderr, "Error signing on to the CIB service: %s\n",
 					cib_error2string(rc));
 				return rc;
 			}
@@ -1030,18 +872,16 @@ main(int argc, char **argv)
 		} else {
 			rc = cib_NOTEXISTS;
 		}
+		
 	}
-
-	if(rsc_cmd == 'R'
-	   || rsc_cmd == 'C'
-	   || rsc_cmd == 'F'
-	   || rsc_cmd == 'P') {
+	if(rsc_cmd == 'R' || rsc_cmd == 'C' || rsc_cmd == 'P') {
 		GCHSource *src = NULL;
 		src = init_client_ipc_comms(CRM_SYSTEM_CRMD, crmd_msg_callback,
 				      NULL, &crmd_channel);
 
 		if(src == NULL) {
-			CMD_ERR("Error signing on to the CRMd service\n");
+			fprintf(stderr,
+				"Error signing on to the CRMd service\n");
 			return 1;
 		}
 		
@@ -1051,7 +891,6 @@ main(int argc, char **argv)
 		set_IPC_Channel_dnotify(src, resource_ipc_connection_destroy);
 	}
 
-	crm_warn("here i am - 3");
 	if(rsc_cmd == 'L') {
 		rc = cib_ok;
 		do_find_resource_list(&data_set, FALSE);
@@ -1061,69 +900,63 @@ main(int argc, char **argv)
 		do_find_resource_list(&data_set, TRUE);
 		
 	} else if(rsc_cmd == 'C') {
-		rc = delete_lrm_rsc(crmd_channel, host_uname, rsc_id, &data_set);
+		resource_t *rsc = pe_find_resource(data_set.resources, rsc_id);
+
+		delete_lrm_rsc(crmd_channel, host_uname,
+			       rsc?rsc->id:rsc_id, rsc?rsc->long_name:NULL);
 		
-	} else if(rsc_cmd == 'F') {
-		rc = fail_lrm_rsc(crmd_channel, host_uname, rsc_id, &data_set);
+		sleep(5);
+		refresh_lrm(crmd_channel, host_uname);
+
+		if(rsc != NULL) {
+			char *now_s = NULL;
+			time_t now = time(NULL);
+
+			/* force the TE to start a transition */
+			sleep(5); /* wait for the refresh */
+			now_s = crm_itoa(now);
+			update_attr(cib_conn, cib_sync_call,
+				    NULL, NULL, NULL, NULL, "last-lrm-refresh", now_s);
+			crm_free(now_s);
+		}
 		
 	} else if(rc == cib_NOTEXISTS) {
-		CMD_ERR("Resource %s not found: %s\n",
+		fprintf(stderr, "Resource %s not found: %s\n",
 			crm_str(rsc_id), cib_error2string(rc));
 		
 	} else if(rsc_cmd == 'W') {
-		if(rsc_id == NULL) {
-			CMD_ERR("Must supply a resource id with -r\n");
-			return cib_NOTEXISTS;
-		} 
+		CRM_DEV_ASSERT(rsc_id != NULL);
 		rc = do_find_resource(rsc_id, &data_set);
 		
 	} else if(rsc_cmd == 'x') {
-		if(rsc_id == NULL) {
-			CMD_ERR("Must supply a resource id with -r\n");
-			return cib_NOTEXISTS;
-		} 
+		CRM_DEV_ASSERT(rsc_id != NULL);
 		rc = dump_resource(rsc_id, &data_set);
 
 	} else if(rsc_cmd == 'U') {
-		if(rsc_id == NULL) {
-			CMD_ERR("Must supply a resource id with -r\n");
-			return cib_NOTEXISTS;
-		} 
 		rc = migrate_resource(rsc_id, NULL, NULL, cib_conn);
 
 	} else if(rsc_cmd == 'M') {
-		node_t *dest = NULL;
-		node_t *current = NULL;
 		const char *current_uname = NULL;
 		resource_t *rsc = pe_find_resource(data_set.resources, rsc_id);
 		if(rsc != NULL && rsc->running_on != NULL) {
-			current = rsc->running_on->data;
+			node_t *current = rsc->running_on->data;
 			if(current != NULL) {
 				current_uname = current->details->uname;
 			}
 		}
-
-		if(host_uname != NULL) {
-			dest = pe_find_node(data_set.nodes, host_uname);
-		}
 		
 		if(rsc == NULL) {
-			CMD_ERR("Resource %s not migrated:"
+			fprintf(stderr, "Resource %s not migrated:"
 				" not found\n", rsc_id);
 
-		} else if(rsc->variant == pe_native
-			  && g_list_length(rsc->running_on) > 1) {
-			CMD_ERR("Resource %s not migrated:"
+		} else if(g_list_length(rsc->running_on) > 1) {
+			fprintf(stderr, "Resource %s not migrated:"
 				" active on multiple nodes\n", rsc_id);
 			
-		} else if(host_uname != NULL && dest == NULL) {
-			CMD_ERR("Error performing operation: "
-				"%s is not a known node\n", host_uname);
-
 		} else if(host_uname != NULL
 			  && safe_str_eq(current_uname, host_uname)) {
-			CMD_ERR("Error performing operation: "
-				"%s is already active on %s\n",
+			fprintf(stderr, "Error performing operation: "
+				"%s is already active on %s",
 				rsc_id, host_uname);
 
 		} else if(current_uname != NULL
@@ -1137,32 +970,23 @@ main(int argc, char **argv)
 				rsc_id, NULL, host_uname, cib_conn);
 
 		} else {
-			CMD_ERR("Resource %s not migrated: "
+			fprintf(stderr, "Resource %s not migrated: "
 				"not-active and no prefered location"
 				" specified.\n", rsc_id);
 		}
 		
 	} else if(rsc_cmd == 'G') {
-		if(rsc_id == NULL) {
-			CMD_ERR("Must supply a resource id with -r\n");
-			return cib_NOTEXISTS;
-		} 
+		CRM_DEV_ASSERT(rsc_id != NULL);
 		rc = dump_resource_prop(rsc_id, prop_name, &data_set);
 
 	} else if(rsc_cmd == 'S') {
 		crm_data_t *msg_data = NULL;
-		if(prop_value == NULL || strlen(prop_value) == 0) {
-			CMD_ERR("You need to supply a value with the -v option\n");
+		if(prop_value == NULL) {
+			fprintf(stderr, "You need to supply a value with the -v option\n");
 			return CIBRES_MISSING_FIELD;
-
-		} else if(cib_conn == NULL) {
-			return cib_connection;
 		}
 
-		if(rsc_id == NULL) {
-			CMD_ERR("Must supply a resource id with -r\n");
-			return cib_NOTEXISTS;
-		} 
+		CRM_DEV_ASSERT(rsc_id != NULL);
 		CRM_DEV_ASSERT(rsc_type != NULL);
 		CRM_DEV_ASSERT(prop_name != NULL);
 		CRM_DEV_ASSERT(prop_value != NULL);
@@ -1172,33 +996,24 @@ main(int argc, char **argv)
 		crm_xml_add(msg_data, prop_name, prop_value);
 		
 		rc = cib_conn->cmds->modify(cib_conn, XML_CIB_TAG_RESOURCES,
-					    msg_data, NULL, cib_options);
+					    msg_data, NULL, cib_sync_call);
 		free_xml(msg_data);
 
 	} else if(rsc_cmd == 'g') {
-		if(rsc_id == NULL) {
-			CMD_ERR("Must supply a resource id with -r\n");
-			return cib_NOTEXISTS;
-		} 
+		CRM_DEV_ASSERT(rsc_id != NULL);
 		rc = dump_resource_attr(rsc_id, prop_name, &data_set);
 
 	} else if(rsc_cmd == 'p') {
-		if(rsc_id == NULL) {
-			CMD_ERR("Must supply a resource id with -r\n");
-			return cib_NOTEXISTS;
-		} 
-		if(prop_value == NULL || strlen(prop_value) == 0) {
-			CMD_ERR("You need to supply a value with the -v option\n");
+		CRM_DEV_ASSERT(rsc_id != NULL);
+		if(prop_value == NULL) {
+			fprintf(stderr, "You need to supply a value with the -v option\n");
 			return CIBRES_MISSING_FIELD;
 		}
 		rc = set_resource_attr(rsc_id, prop_set, prop_id, prop_name,
 				       prop_value, cib_conn, &data_set);
 
 	} else if(rsc_cmd == 'd') {
-		if(rsc_id == NULL) {
-			CMD_ERR("Must supply a resource id with -r\n");
-			return cib_NOTEXISTS;
-		} 
+		CRM_DEV_ASSERT(rsc_id != NULL);
 		rc = delete_resource_attr(rsc_id, prop_id, prop_set, prop_name,
 					  cib_conn, &data_set);
 
@@ -1215,28 +1030,26 @@ main(int argc, char **argv)
 
 	} else if(rsc_cmd == 'D') {
 		crm_data_t *msg_data = NULL;
+		int cib_options = cib_sync_call;
 		
-		if(rsc_id == NULL) {
-			CMD_ERR("Must supply a resource id with -r\n");
-			return cib_NOTEXISTS;
-		} 
+		CRM_CHECK(rsc_id != NULL, return cib_NOTEXISTS);
 		if(rsc_type == NULL) {
-			CMD_ERR("You need to specify a resource type with -t");
+			fprintf(stderr, "You need to specify a resource type with -t");
 			return cib_NOTEXISTS;
-
-		} else if(cib_conn == NULL) {
-			return cib_connection;
 		}
 
+		if(do_force) {
+			cib_options |= cib_scope_local|cib_quorum_override;
+		}
 		msg_data = create_xml_node(NULL, rsc_type);
 		crm_xml_add(msg_data, XML_ATTR_ID, rsc_id);
-
+		
 		rc = cib_conn->cmds->delete(cib_conn, XML_CIB_TAG_RESOURCES,
 					    msg_data, NULL, cib_options);
 		free_xml(msg_data);
 
 	} else {
-		CMD_ERR("Unknown command: %c\n", rsc_cmd);
+		fprintf(stderr, "Unknown command: %c\n", rsc_cmd);
 	}
 
 	if(cib_conn != NULL) {
@@ -1244,12 +1057,12 @@ main(int argc, char **argv)
 		cib_conn->cmds->signoff(cib_conn);
 	}
 	if(rc == cib_no_quorum) {
-		CMD_ERR("Error performing operation: %s\n",
+		fprintf(stderr, "Error performing operation: %s\n",
 			cib_error2string(rc));
-		CMD_ERR("Try using -f\n");
+		fprintf(stderr, "Try using -f\n");
 
 	} else if(rc != cib_ok) {
-		CMD_ERR("Error performing operation: %s\n",
+		fprintf(stderr, "Error performing operation: %s\n",
 			cib_error2string(rc));
 	}
 	
@@ -1283,7 +1096,7 @@ usage(const char *cmd, int exit_status)
 		" creating a rule for the current location and a score of -INFINITY\n"
 		"\t\tNOTE: This will prevent the resource from running on this"
 		" node until the constraint is removed with -U\n"
-		"\t\t\t  Requires: -r, Optional: -H, -f, --lifetime\n", "migrate", 'M');
+		"\t\t\t  Requires: -r, Optional: -H, -f\n", "migrate", 'M');
 	fprintf(stream, "\t--%s (-%c)\t: Remove all constraints created by -M\n"
 		"\t\t\t  Requires: -r\n", "un-migrate", 'U');
 	fprintf(stream, "\t--%s (-%c)\t: Delete a resource from the CIB\n"
@@ -1296,13 +1109,19 @@ usage(const char *cmd, int exit_status)
 		"\t\t\t  Optional: -H\n", "refresh", 'R');
 	fprintf(stream, "\t--%s (-%c) <string>\t: "
 		"Set the named parameter for a resource\n"
-		"\t\t\t  Requires: -r, -v.  Optional: -i, -s, --meta\n", "set-parameter", 'p');
+		"\t\t\t  Requires: -r, -v.  Optional: -i, -s\n", "set-parameter", 'p');
 	fprintf(stream, "\t--%s (-%c) <string>\t: "
 		"Get the named parameter for a resource\n"
-		"\t\t\t  Requires: -r.  Optional: -i, -s, --meta\n", "get-parameter", 'g');
+		"\t\t\t  Requires: -r.  Optional: -i, -s\n", "get-parameter", 'g');
 	fprintf(stream, "\t--%s (-%c) <string>: "
 		"Delete the named parameter for a resource\n"
-		"\t\t\t  Requires: -r.  Optional: -i, --meta\n", "delete-parameter", 'd');
+		"\t\t\t  Requires: -r.  Optional: -i\n", "delete-parameter", 'd');
+	fprintf(stream, "\t--%s (-%c) <string>\t: "
+		"Get the named property (eg. class, type, is_managed) a resource\n"
+		"\t\t\t  Requires: -r\n", "get-property", 'G');
+	fprintf(stream, "\t--%s (-%c) <string>\t: "
+		"Set the named property (not parameter) for a resource\n"
+		"\t\t\t  Requires: -r, -t, -v", "set-property", 'S');
 	fprintf(stream, "\nOptions\n");
 	fprintf(stream, "\t--%s (-%c) <string>\t: Resource ID\n", "resource", 'r');
 	fprintf(stream, "\t--%s (-%c) <string>\t: "
@@ -1313,18 +1132,14 @@ usage(const char *cmd, int exit_status)
 		"Property value\n", "property-value", 'v');
 	fprintf(stream, "\t--%s (-%c) <string>\t: "
 		"Host name\n", "host-uname", 'H');
-	fprintf(stream, "\t--%s\t: Modify a resource's configuration option rather than one which is passed to the resource agent script."
-		"\n\t\tFor use with -p, -g, -d\n", "meta");
-	fprintf(stream, "\t--%s (-%c) <string>\t: "
-		"Lifespan of migration constraints\n", "lifetime", 'u');
 	fprintf(stream, "\t--%s (-%c)\t: "
 		"Force the resource to move by creating a rule for the"
 		" current location and a score of -INFINITY\n"
 		"\t\tThis should be used if the resource's stickiness and"
-		" constraint scores total more than INFINITY (Currently 100,000)\n"
+		" constraint scores total more than INFINITY (Currently 10,000)\n"
 		"\t\tNOTE: This will prevent the resource from running on this"
-		" node until the constraint is removed with -U or the --lifetime duration expires\n",
-		"force", 'f');
+		" node until the constraint is removed with -U\n",
+		"force-relocation", 'f');
 	fprintf(stream, "\t-%c <string>\t: (Advanced Use Only) ID of the instance_attributes object to change\n", 's');
 	fprintf(stream, "\t-%c <string>\t: (Advanced Use Only) ID of the nvpair object to change/delete\n", 'i');
 	fflush(stream);
