@@ -50,37 +50,64 @@ extern gboolean check_join_state(
 /* #define MAX_EMPTY_CALLBACKS 20 */
 /* int empty_callbacks = 0; */
 
+#define trigger_fsa(source) crm_debug_3("Triggering FSA: %s", __FUNCTION__); \
+	G_main_set_trigger(source);
+
 gboolean
-crmd_ha_msg_dispatch(IPC_Channel *channel, gpointer user_data)
+crmd_ha_msg_dispatch(ll_cluster_t *cluster_conn, gpointer user_data)
 {
-	int lpc = 0;
-	ll_cluster_t *hb_cluster = (ll_cluster_t*)user_data;
+	IPC_Channel *channel = NULL;
+	gboolean stay_connected = TRUE;
 
-	while(lpc < 2 && hb_cluster->llc_ops->msgready(hb_cluster)) {
-		if(channel->ch_status != IPC_CONNECT) {
-			/* there really is no point continuing */
-			break;
-		}
- 		lpc++; 
-		/* invoke the callbacks but dont block */
-		hb_cluster->llc_ops->rcvmsg(hb_cluster, 0);
+	crm_debug_3("Invoked");
+
+	if(cluster_conn != NULL) {
+		channel = cluster_conn->llc_ops->ipcchan(cluster_conn);
 	}
-
-	crm_debug_3("%d HA messages dispatched", lpc);
-	G_main_set_trigger(fsa_source);
 	
-	if (channel && (channel->ch_status != IPC_CONNECT)) {
-		crm_crit("Lost connection to heartbeat service.");
-		return FALSE;
+	CRM_CHECK(cluster_conn != NULL, ;);
+	CRM_CHECK(channel != NULL, ;);
+	
+	if(channel != NULL && IPC_ISRCONN(channel)) {
+		if(cluster_conn->llc_ops->msgready(cluster_conn) == 0) {
+			crm_debug_2("no message ready yet");
+		}
+		/* invoke the callbacks but dont block */
+		cluster_conn->llc_ops->rcvmsg(cluster_conn, 0);
+	}
+	
+	if (channel == NULL || channel->ch_status != IPC_CONNECT) {
+		if(is_set(fsa_input_register, R_HA_DISCONNECTED) == FALSE) {
+			crm_crit("Lost connection to heartbeat service.");
+		} else {
+			crm_info("Lost connection to heartbeat service.");
+		}
+		trigger_fsa(fsa_source);
+		stay_connected = FALSE;
 	}
     
-	return TRUE;
+	return stay_connected;
 }
 
+void
+crmd_ha_connection_destroy(gpointer user_data)
+{
+	crm_debug_3("Invoked");
+	if(is_set(fsa_input_register, R_HA_DISCONNECTED)) {
+		/* we signed out, so this is expected */
+		crm_info("Heartbeat disconnection complete");
+		return;
+	}
+
+	crm_crit("Lost connection to heartbeat service!");
+	register_fsa_input(C_HA_DISCONNECT, I_ERROR, NULL);	
+	trigger_fsa(fsa_source);
+}
 
 void
-crmd_ha_msg_callback(const HA_Message * msg, void* private_data)
+crmd_ha_msg_callback(HA_Message * msg, void* private_data)
 {
+	int level = LOG_DEBUG;
 	ha_msg_input_t *new_input = NULL;
 	oc_node_t *from_node = NULL;
 	
@@ -93,6 +120,8 @@ crmd_ha_msg_callback(const HA_Message * msg, void* private_data)
 
 	CRM_DEV_ASSERT(from != NULL);
 
+	crm_debug_2("HA[inbound]: %s from %s", op, from);
+
 	if(fsa_membership_copy == NULL) {
 		crm_debug("Ignoring HA messages until we are"
 			  " connected to the CCM (%s op from %s)", op, from);
@@ -104,7 +133,6 @@ crmd_ha_msg_callback(const HA_Message * msg, void* private_data)
 	from_node = g_hash_table_lookup(fsa_membership_copy->members, from);
 
 	if(from_node == NULL) {
-		int level = LOG_DEBUG;
 		if(safe_str_eq(op, CRM_OP_VOTE)) {
 			level = LOG_WARNING;
 
@@ -114,72 +142,59 @@ crmd_ha_msg_callback(const HA_Message * msg, void* private_data)
 		} else if(safe_str_eq(sys_from, CRM_SYSTEM_DC)) {
 			level = LOG_WARNING;
 		}
-		do_crm_log(level, __FILE__, __FUNCTION__, 
+		crm_log_maybe(level, 
 			   "Ignoring HA message (op=%s) from %s: not in our"
 			   " membership list (size=%d)", op, from,
 			   g_hash_table_size(fsa_membership_copy->members));
 		
 		crm_log_message_adv(LOG_MSG, "HA[inbound]: CCM Discard", msg);
 
-	} else if(AM_I_DC
-	   && safe_str_eq(sys_from, CRM_SYSTEM_DC)
-	   && safe_str_neq(from, fsa_our_uname)) {
-		crm_err("Another DC detected: %s (op=%s)", from, op);
-		crm_log_message_adv(
-			LOG_WARNING, "HA[inbound]: Duplicate DC", msg);
-		new_input = new_ha_msg_input(msg);
+	} else if(safe_str_eq(sys_to, CRM_SYSTEM_DC) && AM_I_DC == FALSE) {
+		crm_debug_2("Ignoring message for the DC [F_SEQ=%s]", seq);
+		return;
 
-		/* make sure the election happens NOW */
-		register_fsa_error_adv(C_FSA_INTERNAL, I_ELECTION, NULL,
-				       new_input, __FUNCTION__);
-		
+	} else if(safe_str_eq(sys_from, CRM_SYSTEM_DC)) {
+		if(AM_I_DC && safe_str_neq(from, fsa_our_uname)) {
+			crm_err("Another DC detected: %s (op=%s)", from, op);
+			/* make sure the election happens NOW */
+			level = LOG_WARNING;
+			if(fsa_state != S_ELECTION) {
+				new_input = new_ha_msg_input(msg);
+				register_fsa_error_adv(
+					C_FSA_INTERNAL, I_ELECTION, NULL,
+					new_input, __FUNCTION__);
+			}
+			
 #if 0
 		/* still thinking about this one...
 		 * could create a timing issue if we dont notice the
 		 * election before a new DC is elected.
 		 */
-	} else if(fsa_our_dc != NULL
-		  && safe_str_eq(sys_from, CRM_SYSTEM_DC)
-		  && safe_str_neq(from, fsa_our_dc)) {
-		crm_warn("Ignoring message from wrong DC: %s vs. %s ",
-			 from, fsa_our_dc);
-		crm_log_message_adv(LOG_WARNING, "HA[inbound]: wrong DC", msg);
+		} else if(fsa_our_dc != NULL && safe_str_neq(from,fsa_our_dc)){
+			crm_warn("Ignoring message from wrong DC: %s vs. %s ",
+				 from, fsa_our_dc);
+			return;
 #endif
-	} else if(safe_str_eq(sys_to, CRM_SYSTEM_DC) && AM_I_DC == FALSE) {
-		crm_debug_2("Ignoring message for the DC [F_SEQ=%s]", seq);
-		crm_log_message_adv(LOG_DEBUG_4, "HA[inbound]: ignore", msg);
-		return;
+		} else {
+			crm_debug_2("Processing DC message from %s [F_SEQ=%s]",
+				    from, seq);
+		}
+	}
 
-	} else if(safe_str_eq(from, fsa_our_uname)
-		  && safe_str_eq(op, CRM_OP_VOTE)) {
-		crm_log_message_adv(LOG_DEBUG_4, "HA[inbound]", msg);
-		crm_debug_2("Ignoring our own vote [F_SEQ=%s]: own vote", seq);
-		return;
-		
-	} else if(AM_I_DC && safe_str_eq(op, CRM_OP_HBEAT)) {
-		crm_debug_2("Ignoring our own heartbeat [F_SEQ=%s]", seq);
-		crm_log_message_adv(LOG_DEBUG_4, "HA[inbound]: own heartbeat", msg);
-		return;
-
-	} else {
-		crm_debug_3("Processing message");
+	if(new_input == NULL) {
 		crm_log_message_adv(LOG_MSG, "HA[inbound]", msg);
 		new_input = new_ha_msg_input(msg);
-		register_fsa_input(C_HA_MESSAGE, I_ROUTER, new_input);
+		route_message(C_HA_MESSAGE, new_input);
 	}
 
-	
-#if 0
-	if(ha_msg_value(msg, XML_ATTR_REFERENCE) == NULL) {
-		ha_msg_add(new_input->msg, XML_ATTR_REFERENCE, seq);
-	}
-#endif
-
 	delete_ha_msg_input(new_input);
+	trigger_fsa(fsa_source);
 
 	return;
 }
 
+
+
 /*
  * Apparently returning TRUE means "stay connected, keep doing stuff".
  * Returning FALSE means "we're all done, close the connection"
@@ -188,90 +203,93 @@ gboolean
 crmd_ipc_msg_callback(IPC_Channel *client, gpointer user_data)
 {
 	int lpc = 0;
-	IPC_Message *msg = NULL;
+	HA_Message *msg = NULL;
 	ha_msg_input_t *new_input = NULL;
 	crmd_client_t *curr_client = (crmd_client_t*)user_data;
 	gboolean stay_connected = TRUE;
 	
-	crm_debug_2("Processing IPC message from %s",
+	crm_debug_2("Invoked: %s",
 		   curr_client->table_key);
 
-	while(lpc == 0 && client->ops->is_message_pending(client)) {
-		if (client->ch_status == IPC_DISCONNECT) {
-			/* The message which was pending for us is that
-			 * the IPC status is now IPC_DISCONNECT */
+	while(IPC_ISRCONN(client)) {
+		if(client->ops->is_message_pending(client) == 0) {
 			break;
 		}
-		if (client->ops->recv(client, &msg) != IPC_OK) {
-			perror("Receive failure:");
-			crm_err("[%s] [receive failure]",
-				curr_client->table_key);
-			stay_connected = FALSE;
-			break;
-			
-		} else if (msg == NULL) {
-			crm_err("[%s] [no message this time]",
+
+		msg = msgfromIPC_noauth(client);
+		if (msg == NULL) {
+			crm_info("%s: no message this time",
 				curr_client->table_key);
 			continue;
 		}
 
 		lpc++;
-		new_input = new_ipc_msg_input(msg);
-		msg->msg_done(msg);
+		new_input = new_ha_msg_input(msg);
+		crm_msg_del(msg);
 		
 		crm_debug_2("Processing msg from %s", curr_client->table_key);
-		crm_log_message_adv(LOG_MSG, "CRMd[inbound]", new_input->msg);
+		crm_log_message_adv(LOG_DEBUG_2, "CRMd[inbound]", new_input->msg);
 		if(crmd_authorize_message(new_input, curr_client)) {
-			register_fsa_input(C_IPC_MESSAGE, I_ROUTER, new_input);
+			route_message(C_IPC_MESSAGE, new_input);
 		}
 		delete_ha_msg_input(new_input);
 		
 		msg = NULL;
 		new_input = NULL;
+
+		if(client->ch_status != IPC_CONNECT) {
+			break;
+		}
 	}
 	
 	crm_debug_2("Processed %d messages", lpc);
     
-	if (client->ch_status == IPC_DISCONNECT) {
+	if (client->ch_status != IPC_CONNECT) {
 		stay_connected = FALSE;
 		process_client_disconnect(curr_client);
 	}
 
-	G_main_set_trigger(fsa_source);
+	trigger_fsa(fsa_source);
 	return stay_connected;
 }
 
 
 
+extern GCHSource *lrm_source;
+
 gboolean
-lrm_dispatch(IPC_Channel*src_not_used, gpointer user_data)
+lrm_dispatch(IPC_Channel *src_not_used, gpointer user_data)
 {
-	int num_msgs = 0;
+	/* ?? src == lrm_channel ?? */
 	ll_lrm_t *lrm = (ll_lrm_t*)user_data;
-	crm_debug_3("received callback");
-	num_msgs = lrm->lrm_ops->rcvmsg(lrm, FALSE);
-	if(num_msgs < 1) {
-		crm_err("lrm->lrm_ops->rcvmsg() failed, connection lost?");
-		clear_bit_inplace(fsa_input_register, R_LRM_CONNECTED);
-		register_fsa_input(C_FSA_INTERNAL, I_ERROR, NULL);
+	IPC_Channel *lrm_channel = lrm->lrm_ops->ipcchan(lrm);
+
+	crm_debug_3("Invoked");
+	lrm->lrm_ops->rcvmsg(lrm, FALSE);
+
+	if(lrm_channel->ch_status != IPC_CONNECT) {
+		if(is_set(fsa_input_register, R_LRM_CONNECTED)) {
+			crm_crit("LRM Connection failed");
+			register_fsa_input(C_FSA_INTERNAL, I_ERROR, NULL);
+			clear_bit_inplace(fsa_input_register, R_LRM_CONNECTED);
+			
+		} else {
+			crm_info("LRM Connection disconnected");
+		}
+
+		lrm_source = NULL;
 		return FALSE;
 	}
 	return TRUE;
 }
 
+extern gboolean process_lrm_event(lrm_op_t *op);
+
 void
 lrm_op_callback(lrm_op_t* op)
 {
-	CRM_DEV_ASSERT(op != NULL);
-	if(crm_assert_failed) {
-		return;
-	}
-	
-	crm_debug("received callback: %s/%s (%s)",
-		  op->op_type, op->rsc_id, op_status2text(op->op_status));
-
-	/* Make sure the LRM events are received in order */
-	register_fsa_input_later(C_LRM_OP_CALLBACK, I_LRM_EVENT, op);
+	CRM_CHECK(op != NULL, return);
+	process_lrm_event(op);
 }
 
 void
@@ -279,86 +297,95 @@ crmd_ha_status_callback(
 	const char *node, const char * status,	void* private_data)
 {
 	crm_data_t *update = NULL;
-
-	crm_debug_3("received callback");
 	crm_notice("Status update: Node %s now has status [%s]",node,status);
 
-	if(safe_str_neq(status, DEADSTATUS)) {
-		crm_debug_3("nstatus callback was not for a dead node");
-		return;
+	if(safe_str_eq(status, DEADSTATUS)) {
+		/* this node is taost */
+		update = create_node_state(
+			node, status, XML_BOOLEAN_NO, OFFLINESTATUS,
+			CRMD_STATE_INACTIVE, NULL, TRUE, __FUNCTION__);
+		crm_xml_add(update, XML_CIB_ATTR_REPLACE, XML_TAG_TRANSIENT_NODEATTRS);
+
+	} else if(safe_str_eq(status, ACTIVESTATUS)) {
+		update = create_node_state(
+			node, status, NULL, NULL, NULL, NULL, FALSE, __FUNCTION__);
+	}
+		
+	if(update != NULL) {
+		/* this change should not be broadcast */
+		fsa_cib_anon_update(
+			XML_CIB_TAG_STATUS, update,
+			cib_inhibit_bcast|cib_scope_local|cib_quorum_override);
+		trigger_fsa(fsa_source);
+		free_xml(update);
 	}
-
-	/* this node is taost */
-	update = create_node_state(
-		node, node, status, NULL, NULL, NULL, NULL, __FUNCTION__);
-	
-	crm_xml_add(update, XML_CIB_ATTR_CLEAR_SHUTDOWN, XML_BOOLEAN_TRUE);
-
-	/* this change should not be broadcast */
-	update_local_cib(create_cib_fragment(update, NULL));
-	G_main_set_trigger(fsa_source);
-	free_xml(update);
 }
 
-
 void
 crmd_client_status_callback(const char * node, const char * client,
 		 const char * status, void * private)
 {
-	const char    *join = NULL;
-	const char   *extra = NULL;
-	crm_data_t *  update = NULL;
-
-	crm_debug_3("received callback");
+	const char *join = NULL;
+	crm_data_t *update = NULL;
+	gboolean clear_shutdown = FALSE;
+	
+	crm_debug_3("Invoked");
 	if(safe_str_neq(client, CRM_SYSTEM_CRMD)) {
 		return;
 	}
 
 	if(safe_str_eq(status, JOINSTATUS)){
 		status = ONLINESTATUS;
-		extra  = XML_CIB_ATTR_CLEAR_SHUTDOWN;
+ 		clear_shutdown = TRUE;
 
 	} else if(safe_str_eq(status, LEAVESTATUS)){
 		status = OFFLINESTATUS;
 		join   = CRMD_STATE_INACTIVE;
-		extra  = XML_CIB_ATTR_CLEAR_SHUTDOWN;
+/* 		clear_shutdown = TRUE; */
 	}
 	
 	set_bit_inplace(fsa_input_register, R_PEER_DATA);
 	g_hash_table_replace(
 		crmd_peer_state, crm_strdup(node), crm_strdup(status));
 
-	if(fsa_state == S_STARTING || fsa_state == S_STOPPING) {
+	if(is_set(fsa_input_register, R_CIB_CONNECTED) == FALSE) {
+		return;
+	} else if(fsa_state == S_STOPPING) {
 		return;
 	}
 
 	crm_notice("Status update: Client %s/%s now has status [%s]",
 		   node, client, status);
 
-	if(safe_str_eq(node, fsa_our_dc)
-	   && safe_str_eq(status, OFFLINESTATUS)) {
+	if(safe_str_eq(status, ONLINESTATUS)) {
+		/* remove the cached value in case it changed */
+		crm_info("Uncaching UUID for %s", node);
+		unget_uuid(node);
+	}
+	
+	if(safe_str_eq(node, fsa_our_dc) && safe_str_eq(status, OFFLINESTATUS)){
 		/* did our DC leave us */
 		crm_info("Got client status callback - our DC is dead");
 		register_fsa_input(C_CRMD_STATUS_CALLBACK, I_ELECTION, NULL);
 		
 	} else {
-		crm_data_t *fragment = NULL;
 		crm_debug_3("Got client status callback");
 		update = create_node_state(
-			node, node, NULL, NULL, status, join, NULL,
-			__FUNCTION__);
-		
-		crm_xml_add(update, extra, XML_BOOLEAN_TRUE);
-		fragment = create_cib_fragment(update, NULL);
+			node, NULL, NULL, status, join,
+			NULL, clear_shutdown, __FUNCTION__);
 
+		if(clear_shutdown) {
+			crm_xml_add(update, XML_CIB_ATTR_REPLACE,
+				    XML_TAG_TRANSIENT_NODEATTRS);
+		}
+		
 		/* it is safe to keep these updates on the local node
 		 * each node updates their own CIB
 		 */
-		fsa_cib_conn->cmds->modify(
-			fsa_cib_conn, XML_CIB_TAG_STATUS, fragment, NULL,
+		fsa_cib_anon_update(
+			XML_CIB_TAG_STATUS, update,
 			cib_inhibit_bcast|cib_scope_local|cib_quorum_override);
 
-		free_xml(fragment);
 		free_xml(update);
 
 		if(AM_I_DC && safe_str_eq(status, OFFLINESTATUS)) {
@@ -372,22 +399,39 @@ crmd_client_status_callback(const char * node, const char * client,
 		}
 	}
 	
-	G_main_set_trigger(fsa_source);
+	trigger_fsa(fsa_source);
 }
 
-void
-crmd_ha_connection_destroy(gpointer user_data)
+static void
+crmd_ipc_connection_destroy(gpointer user_data)
 {
-	crm_crit("Heartbeat has left us");
-	/* this is always an error */
-	/* feed this back into the FSA */
-	register_fsa_input(C_HA_DISCONNECT, I_ERROR, NULL);
-}
+	crmd_client_t *client = user_data;
 
+	if(client == NULL) {
+		crm_debug_4("No client to delete");
+		return;
+	}
+	
+	if(client->client_source != NULL) {
+		crm_debug_4("Deleting %s (%p) from mainloop",
+			    client->uuid, client->client_source);
+		G_main_del_IPC_Channel(client->client_source); 
+		client->client_source = NULL;
+	}
+
+	crm_debug_3("Freeing %s client", client->uuid);
+	crm_free(client->table_key);
+	crm_free(client->sub_sys);
+	crm_free(client->uuid);
+	crm_free(client);
+
+	return;
+}
 
 gboolean
 crmd_client_connect(IPC_Channel *client_channel, gpointer user_data)
 {
+	crm_debug_3("Invoked");
 	if (client_channel == NULL) {
 		crm_err("Channel was NULL");
 
@@ -415,7 +459,7 @@ crmd_client_connect(IPC_Channel *client_channel, gpointer user_data)
 			G_main_add_IPC_Channel(
 				G_PRIORITY_LOW, client_channel,
 				FALSE,  crmd_ipc_msg_callback,
-				blank_client, default_ipc_connection_destroy);
+				blank_client, crmd_ipc_connection_destroy);
 	}
     
 	return TRUE;
@@ -428,14 +472,20 @@ gboolean ccm_dispatch(int fd, gpointer user_data)
 	oc_ev_t *ccm_token = (oc_ev_t*)user_data;
 	gboolean was_error = FALSE;
 	
-	crm_debug_3("received callback");	
+	crm_debug_3("Invoked");
 	rc = oc_ev_handle_event(ccm_token);
+
 	if(rc != 0) {
-		crm_err("CCM connection appears to have failed: rc=%d.", rc);
-		register_fsa_input(C_CCM_CALLBACK, I_ERROR, NULL);
+		if(is_set(fsa_input_register, R_CCM_DISCONNECTED) == FALSE) {
+			/* we signed out, so this is expected */
+			register_fsa_input(C_CCM_CALLBACK, I_ERROR, NULL);
+			crm_err("CCM connection appears to have failed: rc=%d.",
+				rc);
+		}
 		was_error = TRUE;
 	}
-	G_main_set_trigger(fsa_source);
+
+	trigger_fsa(fsa_source);
 	return !was_error;
 }
 
@@ -453,7 +503,7 @@ crmd_ccm_msg_callback(
 	gboolean update_quorum = FALSE;
 	gboolean trigger_transition = FALSE;
 
-	crm_debug_3("received callback");
+	crm_debug_3("Invoked");
 
 	if(data != NULL) {
 		instance = membership->m_instance;
@@ -488,6 +538,8 @@ crmd_ccm_msg_callback(
 		case OC_EV_MS_EVICTED:
 			update_quorum = TRUE;
 			register_fsa_input(C_FSA_INTERNAL, I_STOP, NULL);
+			crm_err("Shutting down after CCM event: %s",
+				ccm_event_name(event));
 			break;
 		default:
 			crm_err("Unknown CCM event: %d", event);
@@ -558,7 +610,10 @@ crmd_ccm_msg_callback(
 void
 crmd_cib_connection_destroy(gpointer user_data)
 {
-	if(is_set(fsa_input_register, R_SHUTDOWN)) {
+	crm_debug_3("Invoked");
+	trigger_fsa(fsa_source);
+
+	if(is_set(fsa_input_register, R_CIB_CONNECTED) == FALSE) {
 		crm_info("Connection to the CIB terminated...");
 		return;
 	}
@@ -582,13 +637,15 @@ crm_fsa_trigger(gpointer user_data)
 	if(fsa_diff_max_ms > 0) {
 		fsa_start = time_longclock();
 	}
+	crm_debug_2("Invoked (queue len: %d)", g_list_length(fsa_message_queue));
 	s_crmd_fsa(C_FSA_INTERNAL);
+	crm_debug_2("Exited  (queue len: %d)", g_list_length(fsa_message_queue));
 	if(fsa_diff_max_ms > 0) {
 		fsa_stop = time_longclock();
 		fsa_diff = sub_longclock(fsa_stop, fsa_start);
 		fsa_diff_ms = longclockto_ms(fsa_diff);
 		if(fsa_diff_ms > fsa_diff_max_ms) {
-			crm_warn("FSA took %dms to complete", fsa_diff_ms);
+			crm_err("FSA took %dms to complete", fsa_diff_ms);
 
 		} else if(fsa_diff_ms > fsa_diff_warn_ms) {
 			crm_warn("FSA took %dms to complete", fsa_diff_ms);
