@@ -1,4 +1,4 @@
-/* $Id: fsa_defines.h,v 1.39 2005/05/25 12:51:02 andrew Exp $ */
+/* $Id: fsa_defines.h,v 1.45 2006/02/20 12:10:41 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -331,6 +331,7 @@ enum crmd_fsa_input {
 #define	A_EXIT_1		0x0000000080000000ULL
 
 #define	A_SHUTDOWN_REQ		0x0000000100000000ULL
+#define	A_ELECTION_CHECK	0x0000000200000000ULL
 
 /* -- CCM actions -- */
 #define	A_CCM_CONNECT		0x0000001000000000ULL
@@ -402,8 +403,6 @@ enum crmd_fsa_input {
 #define	O_TE_RESTART		(A_TE_START|A_TE_STOP)
 #define	O_CIB_RESTART		(A_CIB_START|A_CIB_STOP)
 
-#define O_DC_TICKLE O_DC_TIMER_RESTART
-
 /*======================================
  *
  * "register" contents
@@ -419,8 +418,7 @@ enum crmd_fsa_input {
 #define	R_STAYDOWN	0x00000008ULL /* Should we restart? */
 
 #define R_JOIN_OK	0x00000010ULL /* Have we completed the join process */
-#define	R_HAVE_RES	0x00000040ULL /* Do we have any resources running
-					 locally */
+#define	R_READ_CONFIG	0x00000040ULL
 #define	R_INVOKE_PE	0x00000080ULL /* Does the PE needed to be invoked at
 					 the next appropriate point? */
 
@@ -434,7 +432,6 @@ enum crmd_fsa_input {
 #define	R_PE_REQUIRED	0x00002000ULL /* Is the Policy Engine required? */
 #define	R_TE_REQUIRED	0x00004000ULL /* Is the Transition Engine required? */
 
-
 #define	R_CIB_DONE	0x00010000ULL /* Have we calculated the CIB? */
 #define R_HAVE_CIB	0x00020000ULL /* Do we have an up-to-date CIB */
 #define R_CIB_ASKED	0x00040000ULL /* Have we asked for an up-to-date CIB */
@@ -442,6 +439,9 @@ enum crmd_fsa_input {
 #define R_CCM_DATA	0x00100000ULL /* Have we got CCM data yet */
 #define R_PEER_DATA	0x00200000ULL /* Have we got T_CL_STATUS data yet */
 
+#define R_HA_DISCONNECTED  0x00400000ULL /* did we sign out of our own accord */
+#define R_CCM_DISCONNECTED 0x00800000ULL /* did we sign out of our own accord */
+
 #define	R_REQ_PEND	0x01000000ULL /* Are there Requests waiting for
 					 processing? */
 #define	R_PE_PEND	0x02000000ULL /* Has the PE been invoked and we're
@@ -452,7 +452,6 @@ enum crmd_fsa_input {
 					 response? if so perhaps we shouldnt
 					 stop yet */
 
-
 #define R_IN_TRANSITION	0x10000000ULL /*  */
 #define R_SENT_RSC_STOP 0x20000000ULL /* Have we sent a stop action to all
 				       * resources in preparation for
