@@ -1,4 +1,4 @@
-/* $Id: main.c,v 1.28 2005/07/19 19:12:30 andrew Exp $ */
+/* $Id: main.c,v 1.32 2006/02/27 09:55:57 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -47,14 +47,16 @@
 
 GMainLoop*  mainloop = NULL;
 const char* crm_system_name = SYS_NAME;
-extern cib_t *te_cib_conn;
+cib_t *te_cib_conn = NULL;
+extern GTRIGSource *transition_trigger;
+extern crm_action_timer_t *transition_timer;
 
 void usage(const char* cmd, int exit_status);
 int init_start(void);
 gboolean tengine_shutdown(int nsig, gpointer unused);
 extern void te_update_confirm(const char *event, HA_Message *msg);
 extern void te_update_diff(const char *event, HA_Message *msg);
-
+extern crm_graph_functions_t te_graph_fns;
 
 int
 main(int argc, char ** argv)
@@ -62,11 +64,14 @@ main(int argc, char ** argv)
 	gboolean allow_cores = TRUE;
 	int argerr = 0;
 	int flag;
-
+	
 	crm_log_init(crm_system_name);
 	G_main_add_SignalHandler(
 		G_PRIORITY_HIGH, SIGTERM, tengine_shutdown, NULL, NULL);
 
+	transition_trigger = G_main_add_TriggerHandler(
+		G_PRIORITY_LOW, te_graph_trigger, NULL, NULL);
+
 	crm_debug_3("Begining option processing");
 
 	while ((flag = getopt(argc, argv, OPTARGS)) != EOF) {
@@ -153,7 +158,7 @@ init_start(void)
 	}
 
 	if(init_ok && ST_OK != stonithd_set_stonith_ops_callback(
-		   tengine_stonith_callback, NULL)) {
+		   tengine_stonith_callback)) {
 		crm_err("Could not set stonith callback");
 		stonithd_signoff();
 /*  		init_ok = FALSE; */
@@ -172,12 +177,32 @@ init_start(void)
 		}
 	}
 
+	if(init_ok) {
+                cl_uuid_t new_uuid;
+                char uuid_str[UU_UNPARSE_SIZEOF];
+                
+                cl_uuid_generate(&new_uuid);
+                cl_uuid_unparse(&new_uuid, uuid_str);
+                te_uuid = crm_strdup(uuid_str);
+                crm_info("Registering TE UUID: %s", te_uuid);
+		set_graph_functions(&te_graph_fns);
+
+		/* create a blank one */
+		transition_graph = unpack_graph(NULL);
+		transition_graph->complete = TRUE;
+		transition_graph->completion_action = tg_restart;
+
+		crm_malloc0(transition_timer, sizeof(crm_action_timer_t));
+		transition_timer->source_id = 0;
+		transition_timer->reason    = timeout_abort;
+		transition_timer->action    = NULL;
+	}
+	
 	if(init_ok) {
 		/* Create the mainloop and run it... */
 		crm_info("Starting %s", crm_system_name);
 
 		mainloop = g_main_new(FALSE);
-		initialize_graph();
 
 		g_main_run(mainloop);
 		return_to_orig_privs();
@@ -189,6 +214,7 @@ init_start(void)
 			 crm_system_name);
 	}
 
+	
 	if(init_ok) {
 		return 0;
 	}
@@ -214,22 +240,11 @@ usage(const char* cmd, int exit_status)
 	exit(exit_status);
 }
 
+gboolean shuttingdown;
 gboolean
 tengine_shutdown(int nsig, gpointer unused)
-{
-#if 0
-	static int shuttingdown = 0;
-  
-	if (!shuttingdown) {
-		shuttingdown = 1;
-	}
-	if (mainloop != NULL && g_main_is_running(mainloop)) {
-		g_main_quit(mainloop);
-	}else{
-		exit(LSB_EXIT_OK);
-	}
+{  
+	shuttingdown = TRUE;
+	abort_transition(INFINITY, tg_shutdown, "Shutdown", NULL);
 	return TRUE;
-#else
-	return FALSE;
-#endif
 }
