@@ -30,16 +30,10 @@ Additional Audits, Revised Start action, Default Configuration:
 import os,sys,CTS,CTSaudits,CTStests, warnings
 from CTS import *
 from CM_hb import HeartbeatCM
+from xml.dom.minidom import *
 from CTSaudits import ClusterAudit
 from CTStests import *
 from CIB import *
-try:
-    from xml.dom.minidom import *
-except ImportError:
-    sys.__stdout__.write("Python module xml.dom.minidom not found\n")
-    sys.__stdout__.write("Please install python-xml or similar before continuing\n")
-    sys.__stdout__.flush()
-    sys.exit(1)
 
 #######################################################################
 #
@@ -57,7 +51,7 @@ class LinuxHAv2(HeartbeatCM):
     def __init__(self, Environment, randseed=None):
         HeartbeatCM.__init__(self, Environment, randseed=randseed)
 
-        self.clear_cache = 0
+	self.clear_cache = 0
         self.cib_installed = 0
         self.config = None
         self.cluster_monitor = 0
@@ -69,22 +63,22 @@ class LinuxHAv2(HeartbeatCM):
             "StableTime"     : 30,
             "StartCmd"       : "/etc/init.d/heartbeat start > /dev/null 2>&1",
             "StopCmd"        : "/etc/init.d/heartbeat stop  > /dev/null 2>&1",
-            "ElectionCmd"    : "/usr/local/sbin/crmadmin -E %s",
-            "StatusCmd"      : "/usr/local/sbin/crmadmin -S %s 2>/dev/null",
-            "EpocheCmd"      : "/usr/local/sbin/ccm_tool -e",
-            "QuorumCmd"      : "/usr/local/sbin/ccm_tool -q",
-            "CibQuery"       : "/usr/local/sbin/cibadmin -Ql",
-            "ParitionCmd"    : "/usr/local/sbin/ccm_tool -p",
-            "IsRscRunning"   : "/usr/local/lib/heartbeat/lrmadmin -E %s monitor 0 0 EVERYTIME 2>/dev/null|grep return",
-            "ExecuteRscOp"   : "/usr/local/lib/heartbeat/lrmadmin -n %s -E %s %s 0 %d EVERYTIME 2>/dev/null",
-            "CIBfile"        : "%s:/usr/local/var/lib/heartbeat/crm/cib.xml",
+            "ElectionCmd"    : "/usr/sbin/crmadmin -E %s",
+            "StatusCmd"      : "/usr/sbin/crmadmin -S %s 2>/dev/null",
+            "EpocheCmd"      : "/usr/sbin/ccm_tool -e",
+            "QuorumCmd"      : "/usr/sbin/ccm_tool -q",
+            "CibQuery"       : "/usr/sbin/cibadmin -Ql",
+            "ParitionCmd"    : "/usr/sbin/ccm_tool -p",
+            "IsRscRunning"   : "/usr/lib64/heartbeat/lrmadmin -E %s monitor 0 0 EVERYTIME 2>/dev/null|grep return",
+            "ExecuteRscOp"   : "/usr/lib64/heartbeat/lrmadmin -E %s %s 0 0 EVERYTIME 2>/dev/null",
+            "CIBfile"        : "%s:/var/lib/heartbeat/crm/cib.xml",
             "TmpDir"         : "/tmp",
-            "BreakCommCmd2"  : "/usr/local/share/heartbeat/TestHeartbeatComm break-communication %s>/dev/null 2>&1",
+            "BreakCommCmd2"  : "/usr/lib64/heartbeat/TestHeartbeatComm break-communication %s>/dev/null 2>&1",
             "IsIPAddrRscRunning"   : "",
 
-            "StandbyCmd"   : "/usr/local/sbin/crm_standby -U %s -v %s 2>/dev/null",
-            "UUIDQueryCmd"   : "/usr/local/sbin/crmadmin -N",
-            "StandbyQueryCmd"    : "/usr/local/sbin/crm_standby -GQ -U %s 2>/dev/null",
+            "StandbyCmd"   : "/usr/sbin/crm_standby -U %s -v %s 2>/dev/null",
+            "UUIDQueryCmd"   : "/usr/sbin/crmadmin -N",
+            "StandbyQueryCmd"    : "/usr/sbin/crm_standby -GQ -U %s 2>/dev/null",
 
             # Patterns to look for in the log files for various occasions...
             "Pat:DC_IDLE"      : "crmd.*State transition.*-> S_IDLE",
@@ -92,7 +86,7 @@ class LinuxHAv2(HeartbeatCM):
             # This wont work if we have multiple partitions
             # Use: "Pat:They_started" : "%s crmd:.*State transition.*-> S_NOT_DC",
             "Pat:They_started" : "Updating node state to member for %s",
-            "Pat:We_started"   : "%s crmd:.* State transition.*-> S_IDLE",
+            "Pat:We_started"   : "crmd.*%s: State transition.*-> S_IDLE",
             "Pat:We_stopped"   : "heartbeat.*%s.*Heartbeat shutdown complete",
             "Pat:Logd_stopped" : "%s logd:.*Exiting write process",
             "Pat:They_stopped" : "%s crmd:.*LOST:.* %s ",
@@ -116,6 +110,7 @@ class LinuxHAv2(HeartbeatCM):
                 r", exiting\.",
                 r"WARN.*Ignoring HA message.*vote.*not in our membership list",
                 r"pengine.*Attempting recovery of resource",
+                r"pengine.*Handling failed ",
                 r"tengine.*is taking more than 2x its timeout",
                 r"Confirm not received from",
                 r"Welcome reply not received from",
@@ -125,16 +120,15 @@ class LinuxHAv2(HeartbeatCM):
                 r"Search terminated:",
                 r"No need to invoke the TE",
                 r":global_timer_callback",
+                r"Updating failcount for ",
                 r"Faking parameter digest creation",
                 r"Parameters to .* action changed:",
-                r"Parameters to .* changed",
             ),
         })
         del self["Standby"]
         if self.Env["DoBSC"]:
             del self["Pat:They_stopped"]
             del self["Pat:Logd_stopped"]
-            self.Env["use_logd"] = 0
 
         self.check_transitions = 0
         self.check_elections = 0
@@ -148,22 +142,15 @@ class LinuxHAv2(HeartbeatCM):
         '''Return list of errors which are known and very noisey should be ignored'''
         if 1:
             return [ 
-                "crmadmin:",
-                "ERROR: Message hist queue is filling up"
+                "crmadmin:"
                 ]
         return []
 
     def install_config(self, node):
-        if not self.ns.WaitForNodeToComeUp(node):
-            self.log("Node %s is not up." % node)
-            return None
-
         if not self.CIBsync.has_key(node) and self.Env["ClobberCIB"] == 1:
             self.CIBsync[node] = 1
-            self.rsh.remote_py(node, "os", "system", "rm -f /usr/local/var/lib/heartbeat/crm/cib.xml")
-            self.rsh.remote_py(node, "os", "system", "rm -f /usr/local/var/lib/heartbeat/crm/cib.xml.sig")
-            self.rsh.remote_py(node, "os", "system", "rm -f /usr/local/var/lib/heartbeat/crm/cib.xml.last")
-            self.rsh.remote_py(node, "os", "system", "rm -f /usr/local/var/lib/heartbeat/crm/cib.xml.sig.last")
+            self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml")
+            self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml.sig")
 
             # Only install the CIB on the first node, all the other ones will pick it up from there
             if self.cib_installed == 1:
@@ -178,7 +165,7 @@ class LinuxHAv2(HeartbeatCM):
                 os.system("rm -f "+cib_file)
                 self.debug("Creating new CIB for " + node + " in: " + cib_file)
                 os.system("echo \'" + self.default_cts_cib + "\' > " + cib_file)
-                if 0!=self.rsh.echo_cp(None, cib_file, node, "/usr/local/var/lib/heartbeat/crm/cib.xml"):
+                if 0!=self.rsh.echo_cp(None, cib_file, node, "/var/lib/heartbeat/crm/cib.xml"):
                     raise ValueError("Can not create CIB on %s "%node)
 
                 os.system("rm -f "+cib_file)
@@ -187,7 +174,7 @@ class LinuxHAv2(HeartbeatCM):
                 if 0!=self.rsh.cp(self.Env["CIBfilename"], "root@" + (self["CIBfile"]%node)):
                     raise ValueError("Can not scp file to %s "%node)
         
-            self.rsh.remote_py(node, "os", "system", "chown hacluster /usr/local/var/lib/heartbeat/crm/cib.xml")
+            self.rsh.remote_py(node, "os", "system", "chown hacluster /var/lib/heartbeat/crm/cib.xml")
 
     def prepare(self):
         '''Finish the Initialization process. Prepare to test...'''
@@ -199,6 +186,11 @@ class LinuxHAv2(HeartbeatCM):
     def test_node_CM(self, node):
         '''Report the status of the cluster manager on a given node'''
 
+        if not self.ns.WaitForNodeToComeUp(node):
+             self.log("WaitForNodeToComeUp failed, node %s set to down" % node)
+             self.ShouldBeStatus[node] == self["down"]
+             return 3
+
         watchpats = [ ]
         watchpats.append("Current ping state: (S_IDLE|S_NOT_DC)")
         watchpats.append(self["Pat:They_started"]%node)
@@ -269,6 +261,10 @@ class LinuxHAv2(HeartbeatCM):
         idle_watch = CTS.LogWatcher(self["LogFileName"], watchpats, timeout)
         idle_watch.setwatch()
 
+        if not self.ns.WaitForAllNodesToComeUp(self.Env["nodes"]):
+            self.log("WaitForAllNodesToComeUp failed.")
+            return None
+
         any_up = 0
         for node in self.Env["nodes"]:
             # have each node dump its current state
@@ -276,21 +272,19 @@ class LinuxHAv2(HeartbeatCM):
                 self.rsh.readaline(node, (self["StatusCmd"] %node) )
                 any_up = 1
 
-        if any_up == 0:
-            self.debug("Cluster is inactive") 
+        if any_up == 0 or idle_watch.look():
             return 1
 
-        ret = idle_watch.look()
-        if ret:
-            self.debug(ret) 
-            return 1
-
-        self.log("Warn: Cluster Master not IDLE after %ds" % timeout) 
+        self.log("Warn: Cluster Master not IDLE") 
         return None
 
     def is_node_dc(self, node, status_line=None):
         rc = 0
 
+        if not self.ns.WaitForNodeToComeUp(node):
+           self.log("WaitForNodeToComeUp %s failed." % node) 
+           return rc
+
         if not status_line: 
             status_line = self.rsh.readaline(node, self["StatusCmd"]%node)
 
@@ -312,34 +306,12 @@ class LinuxHAv2(HeartbeatCM):
             
         return rc
 
-    def active_resources(self, node):
-        (rc, output) = self.rsh.remote_py(
-            node, "os", "system", """/usr/local/sbin/crm_mon -1 | grep "Started %s" """ % node)
-
-        resources = []
-        for line in output:
-            fields = line.split()
-            resources.append(fields[0])
-        return resources
-
-    def ResourceOp(self, resource, op, node, interval=0, app="lrmadmin"):
-        '''
-        Execute an operation on a resource
-        '''
-        self.rsh.readaline(node, self["ExecuteRscOp"] 
-                           % (app, resource, op, interval))
-        return self.rsh.lastrc
-
-    def ResourceLocation(self, rid):
-        ResourceNodes = []
-        for node in self.Env["nodes"]:
-            if self.ShouldBeStatus[node] == self["up"]:
-                if self.ResourceOp(rid, "monitor", node) == 0:
-                    ResourceNodes.append(node)
-        return ResourceNodes
-
     def isolate_node(self, node, allowlist):
         '''isolate the communication between the nodes'''
+        if not self.ns.WaitForNodeToComeUp(node):
+           self.log("WaitForNodeToComeUp %s failed." % node)
+           return None
+ 
         rc = self.rsh(node, self["BreakCommCmd2"]%allowlist)
         if rc == 0:
             return 1
@@ -365,7 +337,7 @@ class LinuxHAv2(HeartbeatCM):
                 os.system("cp "+self.Env["CIBfilename"]+" "+cib_file)
         else:            
             if 0 != self.rsh.echo_cp(
-                self.Env["nodes"][0], "/usr/local/var/lib/heartbeat/crm/cib.xml", None, cib_file):
+                self.Env["nodes"][0], "/var/lib/heartbeat/crm/cib.xml", None, cib_file):
                 raise ValueError("Can not copy file to %s, maybe permission denied"%cib_file)
 
         self.config = parse(cib_file)
@@ -455,6 +427,10 @@ class LinuxHAv2(HeartbeatCM):
     def find_partitions(self):
         ccm_partitions = []
 
+        if not self.ns.WaitForAllNodesToComeUp(self.Env["nodes"]):
+            self.log("WaitForAllNodesToComeUp failed.")
+            return None
+
         for node in self.Env["nodes"]:
             self.debug("Retrieving partition details for %s" %node)
             if self.ShouldBeStatus[node] == self["up"]:
@@ -464,12 +440,13 @@ class LinuxHAv2(HeartbeatCM):
                     self.log("no partition details for %s" %node)
                 elif len(partition) > 2:
                     partition = partition[:-1]
+                    self.debug("partition details for %s: %s" %(node, partition))
                     found=0
                     for a_partition in ccm_partitions:
                         if partition == a_partition:
                             found = 1
                     if found == 0:
-                        self.debug("Adding partition from %s: %s" %(node, partition))
+                        self.debug("Adding partition")
                         ccm_partitions.append(partition)
                 else:
                     self.log("bad partition details for %s" %node)
@@ -484,6 +461,10 @@ class LinuxHAv2(HeartbeatCM):
         if not node_list:
             node_list = self.Env["nodes"]
 
+        if not self.ns.WaitForAllNodesToComeUp(node_list):
+            self.log("WaitForAllNodesToComeUp failed in HasQuorum.")
+            return 0
+
         for node in node_list:
             if self.ShouldBeStatus[node] == self["up"]:
                 quorum = self.rsh.readaline(node, self["QuorumCmd"])
@@ -496,112 +477,30 @@ class LinuxHAv2(HeartbeatCM):
 
         return 0
     def Components(self):    
-        complist = []
-        common_ignore = [
-                    "Pending action:",
-                    "ERROR: crm_log_message_adv:",
-                    "ERROR: MSG: No message to dump",
-                    "pending LRM operations at shutdown",
-                    "Lost connection to the CIB service",
-                    "Connection to the CIB terminated...",
-                    "Sending message to CIB service FAILED",
-                    "crmd: .*Action A_RECOVER .* not supported",
-                    "ERROR: stonithd_op_result_ready: not signed on",
-                    "send_ipc_message: IPC Channel to .* is not connected",
-                    "unconfirmed_actions: Waiting on .* unconfirmed actions",
-                    "cib_native_msgready: Message pending on command channel",
-                    "crmd:.*do_exit: Performing A_EXIT_1 - forcefully exiting the CRMd",
-                    "verify_stopped: Resource .* was active at shutdown.  You may ignore this error if it is unmanaged.",
-            ]
-
-        stonith_ignore = [
-            "ERROR: stonithd_signon: ",
-            "update_failcount: Updating failcount for child_DoFencing",
-            "ERROR: te_connect_stonith: Sign-in failed: triggered a retry",
-            ]
-
-        stonith_ignore.extend(common_ignore)
-
-        complist.append(Process("ccm", 0, [
-                    "State transition S_IDLE",
-                    "CCM connection appears to have failed",
-                    "crmd: .*Action A_RECOVER .* not supported",
-                    "crmd: .*Input I_TERMINATE from do_recover",
-                    "Exiting to recover from CCM connection failure",
-                    "crmd:.*do_exit: Could not recover from internal error",
-                    "crmd: .*I_ERROR.*(ccm_dispatch|crmd_cib_connection_destroy)",
-#                    "WARN: determine_online_status: Node .* is unclean",
-#                    "Scheduling Node .* for STONITH",
-#                    "Executing .* fencing operation",
-#                    "tengine_stonith_callback: .*result=0",
-                    "State transition S_.* -> S_INTEGRATION.*input=I_NODE_JOIN",
-                    "State transition S_STARTING -> S_PENDING",
-                    ], [], common_ignore, 1, self))
-
-        complist.append(Process("cib", 0, [
-                    "State transition S_IDLE",
-                    "Lost connection to the CIB service",
-                    "Connection to the CIB terminated...",
-                    "crmd: .*Input I_TERMINATE from do_recover",
-                    "crmd: .*I_ERROR.*crmd_cib_connection_destroy",
-                    "crmd:.*do_exit: Could not recover from internal error",
-                    ], [], common_ignore, 1, self))
-
-        complist.append(Process("lrmd", 0, [
-                    "State transition S_IDLE",
-                    "LRM Connection failed",
-                    "crmd: .*I_ERROR.*lrm_dispatch",
-                    "State transition S_STARTING -> S_PENDING",
-                    ".*crmd .*exited with return code 2.",
-                    "crmd: .*Input I_TERMINATE from do_recover",
-                    "crmd:.*do_exit: Could not recover from internal error",
-                    ], [], common_ignore, 1, self))
-
-        complist.append(Process("crmd", 0, [
-#                    "WARN: determine_online_status: Node .* is unclean",
-#                    "Scheduling Node .* for STONITH",
-#                    "Executing .* fencing operation",
-#                    "tengine_stonith_callback: .*result=0",
-                    "State transition S_IDLE",
-                    "State transition S_STARTING -> S_PENDING",
-                    ], [
-                    "tengine: .*ERROR: subsystem_msg_dispatch: The server .* has left us: Shutting down...NOW",
-                    "pengine: .*ERROR: subsystem_msg_dispatch: The server .* has left us: Shutting down...NOW",
-                    ], common_ignore, 1, self))
-
-        complist.append(Process("pengine", 1, [
-                    "State transition S_IDLE",
-                    ".*crmd .*exited with return code 2.",
-                    "crmd: .*Input I_TERMINATE from do_recover",
-                    "crmd:.*do_exit: Could not recover from internal error",
-                    ], [], common_ignore, 1, self))
-
-        complist.append(Process("tengine", 1, [
-                    "State transition S_IDLE",
-                    ".*crmd .*exited with return code 2.",
-                    "crmd: .*Input I_TERMINATE from do_recover",
-                    "crmd:.*do_exit: Could not recover from internal error",
-                    ], [], common_ignore, 1, self))
-
+        complist = [Process("lrmd",self),Process("crmd",self)]
         if self.Env["DoFencing"] == 1 :
-            complist.append(Process("stonithd", 0, [], [
-                        "tengine_stonith_connection_destroy: Fencing daemon has left us",
-                        "Attempting connection to fencing daemon",
-                        "te_connect_stonith: Connected",
-                        ], stonith_ignore, 0, self))
-#            complist.append(Process("heartbeat", 0, [], [], [], None, self))
+            complist.append(Process("stonithd",self))
+            complist.append(Process("heartbeat",self))
         return complist
-
+    
     def NodeUUID(self, node):
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s failed in NodeUUID." % node)
+            return ""
+
         lines = self.rsh.readlines(node, self["UUIDQueryCmd"])
         for line in lines:
-            self.debug("UUIDLine:"+ line)
+            self.debug("UUIDLine:"+ line) 
             m = re.search(r'%s.+\((.+)\)' % node, line)
-            if m:
+            if m:   
                 return m.group(1)
-        return ""
-
+        return ""       
+                            
     def StandbyStatus(self, node):
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s failed in StandbyStatus." % node)
+            return ""
+
         out=self.rsh.readaline(node, self["StandbyQueryCmd"]%node)
         if not out:
             return "off"
@@ -612,6 +511,10 @@ class LinuxHAv2(HeartbeatCM):
     # status == "on" : Enter Standby mode
     # status == "off": Enter Active mode
     def SetStandbyMode(self, node, status):
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s failed in SetStandbyMode." % node)
+            return True
+
         current_status = self.StandbyStatus(node)
         cmd = self["StandbyCmd"] % (node, status)
         ret = self.rsh(node, cmd)
@@ -657,32 +560,57 @@ class HAResource(Resource):
             if op.getAttribute('name') == "start" and op.getAttribute('prereq') == "nothing":
                 self.needs_quorum = 0
 
+        if self.needs_quorum == 0:
+            self.CM.debug("Resource %s does not need quorum" % self.rid)
+#        else:
+#            self.CM.debug("Resource %s DOES need quorum" % self.rid)
+
     def IsRunningOn(self, nodename):
         '''
         This member function returns true if our resource is running
         on the given node in the cluster.
         We call the status operation for the resource script.
         '''
-        rc = self.CM.ResourceOp(self.rid, "monitor", nodename)
-        return (rc == 0)
+        if not self.CM.ns.WaitForNodeToComeUp(nodename):
+            self.CM.log("WaitForNodeToComeUp %s failed in IsRunningOn." % node)
+            return 0
+ 
+        out=self.CM.rsh.readaline(nodename, self.CM["IsRscRunning"]%self.rid)
+        return re.search("0",out)
         
     def RunningNodes(self):
-        return self.CM.ResourceLocation(self.rid)
+        ResourceNodes = []
+        for node in self.CM.Env["nodes"]:
+            if self.CM.ShouldBeStatus[node] == self.CM["up"]:
+                if self.IsRunningOn(node):
+                    ResourceNodes.append(node)
+        return ResourceNodes
+
+    def _ResourceOperation(self, operation, nodename):
+        '''
+        Execute an operation on the resource
+        '''
+        if not self.CM.ns.WaitForNodeToComeUp(nodename):
+            self.CM.log("WaitForNodeToComeUp %s failed in _ResourceOperation." % node)
+            return 0
+
+        self.CM.rsh.readaline(nodename, self.CM["ExecuteRscOp"]%(self.rid,operation))
+        return self.CM.rsh.lastrc == 0
 
     def Start(self, nodename):
         '''
         This member function starts or activates the resource.
         '''
-        return self.CM.ResourceOp(self.rid, "start", nodename)
+        return self._ResourceOperation("start", nodename)
 
     def Stop(self, nodename):
         '''
         This member function stops or deactivates the resource.
         '''
-        return self.CM.ResourceOp(self.rid, "stop", nodename)
+        return self._ResourceOperation("stop", nodename)
 
     def IsWorkingCorrectly(self, nodename):
-        return self.IsRunningOn(nodename)
+        return self._ResourceOperation("monitor", nodename)
 
 
 #######################################################################
