@@ -195,7 +195,7 @@ class RemoteExec:
            If the call fail, lastrc == 1 and return the reason (string)
         '''
         encode_args = binascii.b2a_base64(pickle.dumps(args))
-        encode_cmd = string.join(["/usr/local/share/heartbeat/cts/CTSproxy.py",module,func,encode_args])
+        encode_cmd = string.join(["/usr/lib64/heartbeat/cts/CTSproxy.py",module,func,encode_args])
 
         #print "%s: %s.%s %s" % (node, module, func, repr(args))
         result = self.readlines(node, encode_cmd)
@@ -361,7 +361,7 @@ class NodeStatus:
         self.rsh = RemoteExec()
 
     def IsNodeBooted(self, node):
-        '''Return TRUE if the given node is booted (responds to pings)'''
+        '''Return TRUE if the given node is booted (responds to pings'''
         return os.system("/bin/ping -nq -c1 -w1 %s >/dev/null 2>&1" % node) == 0
 
     def IsSshdUp(self, node):
@@ -426,12 +426,11 @@ class ClusterManager(UserDict):
     def __init__(self, Environment, randseed=None):
         self.Env = Environment
         self.__InitialConditions()
-        self.clear_cache = 0
-        self.TestLoggingLevel=0
+	self.clear_cache = 0
         self.data = {
             "up"             : "up",        # Status meaning up
             "down"           : "down",  # Status meaning down
-            "StonithCmd"     : "/usr/local/sbin/stonith -t baytech -p '10.10.10.100 admin admin' %s",
+            "StonithCmd"     : "/usr/sbin/stonith -t baytech -p '10.10.10.100 admin admin' %s",
             "DeadTime"       : 30,        # Max time to detect dead node...
             "StartTime"      : 90,        # Max time to start up
     #
@@ -502,20 +501,19 @@ class ClusterManager(UserDict):
         return None
 
     def clear_all_caches(self):
-        if self.clear_cache:
-            for node in self.Env["nodes"]:
-                if self.ShouldBeStatus[node] == self["down"]:
-                    self.debug("Removing cache file on: "+node)
-                    self.rsh.remote_py(node, "os", "system", 
-                                       "rm -f /usr/local/var/lib/heartbeat/hostcache")
-                else:
-                    self.debug("NOT Removing cache file on: "+node)
+	if self.clear_cache:
+	    for node in self.Env["nodes"]:
+		if self.ShouldBeStatus[node] == self["down"]:
+		    self.debug("Removing cache file on: "+node)
+		    self.rsh.remote_py(node, "os", "system", 
+				       "rm -f /var/lib/heartbeat/hostcache")
+		else:
+		    self.debug("NOT Removing cache file on: "+node)
 
     def StartaCM(self, node):
 
         '''Start up the cluster manager on a given node'''
         self.debug("Starting %s on node %s" %(self["Name"], node))
-        ret = 1
 
         if not self.ShouldBeStatus.has_key(node):
             self.ShouldBeStatus[node] = self["down"]
@@ -523,6 +521,10 @@ class ClusterManager(UserDict):
         if self.ShouldBeStatus[node] != self["down"]:
             return 1
 
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s failed" % node)
+            return 0
+
         patterns = []
         # Technically we should always be able to notice ourselves starting
         if self.upcount() == 0:
@@ -537,6 +539,10 @@ class ClusterManager(UserDict):
 
         self.install_config(node)
 
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("Second WaitForNodeToComeUp %s failed" % node)
+            return 0
+
         self.ShouldBeStatus[node] = "any"
         if self.StataCM(node) and self.cluster_stable(self["DeadTime"]):
             self.log ("%s was already started" %(node))
@@ -546,7 +552,7 @@ class ClusterManager(UserDict):
         if self.clear_cache:
             self.debug("Removing cache file on: "+node)
             self.rsh.remote_py(node, "os", "system", 
-                               "rm -f /usr/local/var/lib/heartbeat/hostcache")
+                               "rm -f /var/lib/heartbeat/hostcache")
 
         if self.rsh(node, self["StartCmd"]) != 0:
             self.log ("Warn: Start command failed on node %s" %(node))
@@ -576,11 +582,15 @@ class ClusterManager(UserDict):
 
         self.debug("Starting %s on node %s" %(self["Name"], node))
 
+        if not self.ns.IsNodeBooted(node):
+            self.log("node %s not booted, StartaCMnoBlock failed." % node)
+            return 0
+
         # Clear out the host cache so autojoin can be exercised
         if self.clear_cache:
             self.debug("Removing cache file on: "+node)
             self.rsh.remote_py(node, "os", "system", 
-                               "rm -f /usr/local/var/lib/heartbeat/hostcache")
+                               "rm -f /var/lib/heartbeat/hostcache")
 
         self.rsh.noBlock(node, self["StartCmd"])
         self.ShouldBeStatus[node]=self["up"]
@@ -595,6 +605,10 @@ class ClusterManager(UserDict):
         if self.ShouldBeStatus[node] != self["up"]:
             return 1
 
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s filed" % node)
+            return 0
+
         if self.rsh(node, self["StopCmd"]) == 0:
             self.ShouldBeStatus[node]=self["down"]
             self.cluster_stable(self["DeadTime"])
@@ -610,6 +624,10 @@ class ClusterManager(UserDict):
 
         self.debug("Stopping %s on node %s" %(self["Name"], node))
 
+        if not self.ns.IsNodeBooted(node):
+            self.log("node %s not booted, StopaCNnoBlock failed." % node)
+            return 0
+
         self.rsh.noBlock(node, self["StopCmd"])
         self.ShouldBeStatus[node]=self["down"]
         return 1
@@ -626,6 +644,10 @@ class ClusterManager(UserDict):
         '''Force the cluster manager on a given node to reread its config
            This may be a no-op on certain cluster managers.
         '''
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s filed" % node)
+            return 0
+
         rc=self.rsh(node, self["RereadCmd"])
         if rc == 0:
             return 1
@@ -722,7 +744,12 @@ class ClusterManager(UserDict):
         return result
 
     def isolate_node(self, node):
+
         '''isolate the communication between the nodes'''
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s filed in isolate_node" % node) 
+            return None
+
         rc = self.rsh(node, self["BreakCommCmd"])
         if rc == 0:
             return 1
@@ -731,7 +758,12 @@ class ClusterManager(UserDict):
         return None
  
     def unisolate_node(self, node):
+
         '''fix the communication between the nodes'''
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s filed in unisolate_node" % node)
+            return None
+
         rc = self.rsh(node, self["FixCommCmd"])
         if rc == 0:
             return 1
@@ -741,6 +773,10 @@ class ClusterManager(UserDict):
         
     def reducecomm_node(self,node):
         '''reduce the communication between the nodes'''
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s filed in reducecomm_node" % node)
+            return None
+
         rc = self.rsh(node, self["ReduceCommCmd"]%(self.Env["XmitLoss"],self.Env["RecvLoss"]))
         if rc == 0:
             return 1
@@ -751,6 +787,11 @@ class ClusterManager(UserDict):
     def savecomm_node(self,node):
         '''save current the communication between the nodes'''
         rc = 0
+
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s filed in savecomm_node" % node)
+            return None
+
         if float(self.Env["XmitLoss"])!=0 or float(self.Env["RecvLoss"])!=0 :
             rc = self.rsh(node, self["SaveFileCmd"]);
         if rc == 0:
@@ -762,6 +803,10 @@ class ClusterManager(UserDict):
     def restorecomm_node(self,node):
         '''restore the saved communication between the nodes'''
         rc = 0
+        if not self.ns.WaitForNodeToComeUp(node):
+            self.log("WaitForNodeToComeUp %s filed in restoreomm_node" % node)
+            return None
+
         if float(self.Env["XmitLoss"])!=0 or float(self.Env["RecvLoss"])!=0 :
             rc = self.rsh(node, self["RestoreCommCmd"]);
         if rc == 0:
@@ -814,21 +859,7 @@ class ClusterManager(UserDict):
         raise ValueError("Abstract Class member (Components)")
     
     
-    def RestartClusterLogging(self):
-        self.log("WARN: Restarting logging on cluster nodes")
-        for node in self.Env["nodes"]:
-            cmd=self.Env["logrestartcmd"]
-            if self.rsh.noBlock(node, cmd) != 0:
-                self.log ("ERROR: Cannot restart logging on %s [%s failed]" % (node, cmd))
-
     def TestLogging(self):
-        self.TestLoggingLevel=self.TestLoggingLevel+1
-        ret=1
-        if self.TestLoggingLevel > 3:
-            self.log("ERROR: Unable to fix remote logging. Stopping tests.")
-            self.TestLoggingLevel=self.TestLoggingLevel-1
-            return None
-           
         patterns= []
         prefix="Test message from "
         for node in self.Env["nodes"]:
@@ -842,16 +873,11 @@ class ClusterManager(UserDict):
                 self.log ("ERROR: Cannot execute remote command [%s] on %s" % (cmd, node))
         watch_result = watch.lookforall()
         if watch.unmatched:
-            self.log("ERROR: Remote logging is not working.")
+            self.log("ERROR: Remote logging is not working.  Please fix before continuing.")
             for regex in watch.unmatched:
                 self.log ("ERROR: Test message [%s] not found in logs." % (regex))
-            self.RestartClusterLogging()
-            time.sleep(30*self.TestLoggingLevel)
-            ret=self.TestLogging()
-            if ret:
-              self.log("NOTE: Cluster logging now working.")
-        self.TestLoggingLevel=self.TestLoggingLevel-1
-        return ret
+            return None
+        return 1
 
     def CheckDf(self):
         dfcmd="df -k /var/log | tail -1 | tr -s ' ' | cut -d' ' -f2"
@@ -870,7 +896,7 @@ class ClusterManager(UserDict):
                     if idfout == 0:
                         self.log("CRIT: Completely out of log disk space on %s" % node)
                         result=None
-                    elif idfout <= 1000:
+                    elif idfout <= 500:
                         self.log("WARN: Low on log disk space (%d Mbytes) on %s" % (idfout, node))
         return result
 
@@ -939,19 +965,18 @@ class Component:
         None
         
 class Process(Component):
-    def __init__(self, name, dc_only, pats, dc_pats, badnews_ignore, triggersreboot, cm):
+    def __init__(self, name, cm):
         self.name = str(name)
-        self.dc_only = dc_only
-        self.pats = pats
-        self.dc_pats = dc_pats
         self.CM = cm
-        self.badnews_ignore = badnews_ignore
-	self.triggersreboot = triggersreboot
         self.KillCmd = "killall -9 " + self.name
         
     def kill(self, node):
+        if not self.CM.ns.WaitForNodeToComeUp(node):
+            self.CM.log("WaitForNodeToComeUp %s filed in kill" % node)
+            return None
+
         if self.CM.rsh(node, self.KillCmd) != 0:
-            self.CM.log ("ERROR: Kill %s failed on node %s" %(self.name,node))
+            self.log ("Warn: Kill %s failed on node %s" %(name,node))
             return None
         return 1
 
