@@ -35,7 +35,6 @@ from socket import gethostbyname_ex
 tests = None
 cm = None
 old_handler = None
-DefaultFacility = "daemon"
 
 def sig_handler(signum, frame) :
     if cm != None:
@@ -50,18 +49,17 @@ class ResetMechanism:
         raise ValueError("Abstract class member (reset)")
 
 class Stonith(ResetMechanism):
-    def __init__(self, sttype="external/ssh", pName=None, pValue=None
-        ,        path="/usr/local/sbin/stonith"):
+    def __init__(self, sttype="external/ssh", parm=None
+        ,        path="/usr/sbin/stonith"):
         self.pathname=path
-        self.configName=pName
-        self.configValue=pValue
+        self.configstring=parm
         self.stonithtype=sttype
 
     def reset(self, node):
-        if self.configValue == None :
+        if self.configstring == None :
             config=node
         else:
-            config=self.configValue
+            config=self.configstring
         cmdstring = "%s -t '%s' -p '%s' '%s' 2>/dev/null" % (self.pathname
         ,        self.stonithtype, config, node)
         return (os.system(cmdstring) == 0)
@@ -71,15 +69,15 @@ class Stonithd(ResetMechanism):
         self.sttype = sttype
         self.nodes = nodes
 
-        self.query_cmd_pat   = '/usr/local/lib/heartbeat/stonithdtest/apitest 0 %s 60000 0'
-        self.reset_cmd_pat   = '/usr/local/lib/heartbeat/stonithdtest/apitest 1 %s 60000 0'
-        self.poweron_cmd_pat = '/usr/local/lib/heartbeat/stonithdtest/apitest 2 %s 60000 0'
-        self.poweroff_cmd_pat= '/usr/local/lib/heartbeat/stonithdtest/apitest 3 %s 60000 0'
+        self.query_cmd_pat   = '/usr/lib64/heartbeat/stonithdtest/apitest 0 %s 20000 0'
+        self.reset_cmd_pat   = '/usr/lib64/heartbeat/stonithdtest/apitest 1 %s 20000 0'
+        self.poweron_cmd_pat = '/usr/lib64/heartbeat/stonithdtest/apitest 2 %s 20000 0'
+        self.poweroff_cmd_pat= '/usr/lib64/heartbeat/stonithdtest/apitest 3 %s 20000 0'
 
-        self.lrmd_add_pat    = '/usr/local/lib/heartbeat/lrmadmin -A %s stonith ' + sttype + ' NULL hostlist=%s livedangerously=yes'
-        self.lrmd_start_pat  = '/usr/local/lib/heartbeat/lrmadmin -E %s start 0 0 EVERYTIME'
-        self.lrmd_stop_pat   = '/usr/local/lib/heartbeat/lrmadmin -E %s stop 0 0 EVERYTIME'
-        self.lrmd_del_pat    = '/usr/local/lib/heartbeat/lrmadmin -D %s'
+        self.lrmd_add_pat    = '/usr/lib64/heartbeat/lrmadmin -A %s stonith ' + sttype + ' NULL hostlist=%s'
+        self.lrmd_start_pat  = '/usr/lib64/heartbeat/lrmadmin -E %s start 0 0 EVERYTIME'
+        self.lrmd_stop_pat   = '/usr/lib64/heartbeat/lrmadmin -E %s stop 0 0 EVERYTIME'
+        self.lrmd_del_pat    = '/usr/lib64/heartbeat/lrmadmin -D %s'
 
         self.rsc_id          = 'my_stonithd_id'
 
@@ -148,7 +146,7 @@ class Stonithd(ResetMechanism):
         elif action == "POWERON":
             command = self.poweron_cmd_pat % target_node
         else:
-            self.fail_reason = "unknown operation type %s" % action
+            self.fail_reason = "unknown opration type %s" % action
             return 0
 
         lines = self._remote_readlines(init_node, command)
@@ -193,24 +191,25 @@ class Logger:
 class SysLog(Logger):
     # http://docs.python.org/lib/module-syslog.html
     defaultsource="CTS"
+    defaultfacility= syslog.LOG_LOCAL7
     map = {
-            "kernel":   syslog.LOG_KERN,
-            "user":     syslog.LOG_USER,
-            "mail":     syslog.LOG_MAIL,
-            "daemon":   syslog.LOG_DAEMON,
-            "auth":     syslog.LOG_AUTH,
-            "lpr":      syslog.LOG_LPR,
-            "news":     syslog.LOG_NEWS,
-            "uucp":     syslog.LOG_UUCP,
-            "cron":     syslog.LOG_CRON,
-            "local0":   syslog.LOG_LOCAL0,
-            "local1":   syslog.LOG_LOCAL1,
-            "local2":   syslog.LOG_LOCAL2,
-            "local3":   syslog.LOG_LOCAL3,
-            "local4":   syslog.LOG_LOCAL4,
-            "local5":   syslog.LOG_LOCAL5,
-            "local6":   syslog.LOG_LOCAL6,
-            "local7":   syslog.LOG_LOCAL7,
+            "kernel":	syslog.LOG_KERN,
+            "user":	syslog.LOG_USER,
+            "mail":	syslog.LOG_MAIL,
+            "daemon":	syslog.LOG_MAIL,
+            "auth":	syslog.LOG_AUTH,
+            "lpr":	syslog.LOG_LPR,
+            "news":	syslog.LOG_NEWS,
+            "uucp":	syslog.LOG_UUCP,
+            "cron":	syslog.LOG_CRON,
+            "local0":	syslog.LOG_LOCAL0,
+            "local1":	syslog.LOG_LOCAL1,
+            "local2":	syslog.LOG_LOCAL2,
+            "local3":	syslog.LOG_LOCAL3,
+            "local4":	syslog.LOG_LOCAL4,
+            "local5":	syslog.LOG_LOCAL5,
+            "local6":	syslog.LOG_LOCAL6,
+            "local7":	syslog.LOG_LOCAL7,
     }
     def __init__(self, labinfo):
 
@@ -221,11 +220,10 @@ class SysLog(Logger):
 
         if labinfo.has_key("SyslogFacility"):
             self.facility=labinfo["SyslogFacility"]
+            if SysLog.map.has_key(self.facility):
+              self.facility=SysLog.map[self.facility]
         else:
-            self.facility=DefaultFacility
-
-        if SysLog.map.has_key(self.facility):
-            self.facility=SysLog.map[self.facility]
+            self.facility=SysLog.defaultfacility
 
         syslog.openlog(self.source, 0, self.facility)
 
@@ -346,12 +344,12 @@ class CtsLab(UserDict):
             self["reset"] = Stonith()
         if not self.has_key("CMclass"):
             self["CMclass"] = HeartbeatCM
+        if not self.has_key("HAdir"):
+            self["HAdir"] = "/etc/ha.d"
         if not self.has_key("LogFileName"):
-            self["LogFileName"] = "/usr/local/var/log/ha-log"
-        if not self.has_key("logrestartcmd"):
-            self["logrestartcmd"] = "/etc/init.d/syslog restart"
+            self["LogFileName"] = "/var/log/ha-log"
         if not self.has_key("logfacility"):
-            LogFacility = DefaultFacility
+            LogFacility = "local7"
 
         #
         #  Now set up our random number generator...
@@ -378,7 +376,7 @@ class CtsLab(UserDict):
         "Log using each of the supplied logging methods"
         for logfcn in self._logfunctions:
             if logfcn.name() != "StdErrLog":
-                logfcn("debug: %s" % string.strip(args))
+                logfcn(string.strip(args))
 
     def __setitem__(self, key, value):
         '''Since this function gets called whenever we modify the
@@ -396,11 +394,8 @@ class CtsLab(UserDict):
                 # I don't think I need the IP address, etc. but this validates
                 # the node name against /etc/hosts and/or DNS, so it's a
                 # GoodThing(tm).
-                try:
-                    self.Nodes[node] = gethostbyname_ex(node)
-                except:
-                    print node+" not found in DNS... aborting"
-                    raise
+                self.Nodes[node] = gethostbyname_ex(node)
+            
         #
         #        Reset Mechanism
         #
@@ -480,16 +475,12 @@ def usage(arg):
     +  " --facility syslog-facility" \
     +  " --choose testcase-name" \
     +  " --test-ip-base ip" \
-    +  " --oprofile \"whitespace separated list of nodes to oprofile\"" \
     +  " (-2 |"\
     +  " -v2 |"\
     +  " --crm |"\
     +  " --classic)"\
-    +  " (--populate-resources | -r)" \
     +  " --resource-can-stop" \
     +  " --stonith (1 | 0 | yes | no)" \
-    +  " --stonith-type type" \
-    +  " --stonith-args name=value" \
     +  " --standby (1 | 0 | yes | no)" \
     +  " --fencing (1 | 0 | yes | no)" \
     +  " --suppress_cib_writes (1 | 0 | yes | no)" \
@@ -509,8 +500,8 @@ if __name__ == '__main__':
     from CTS import Scenario, InitClusterManager, PingFest, PacketLoss, BasicSanityCheck
     import CM_hb
 
-    HAdir = "/usr/local/etc/ha.d"
-    LogFile = "/usr/local/var/log/ha-log-"+DefaultFacility
+    HAdir = "/etc/ha.d"
+    LogFile = "/var/log/ha-log-local7"
     DoStonith = 1
     DoStandby = 1
     DoFencing = 1
@@ -532,11 +523,6 @@ if __name__ == '__main__':
     DoBSC = 0
     ListTests = 0
     HaveSeed = 0
-    oprofile = None
-    StonithType = "external/ssh"
-    StonithParams = None
-    StonithParams = "hostlist=dynamic".split('=')
-
     #
     # The values of the rest of the parameters are now properly derived from
     # the configuration files.
@@ -567,7 +553,7 @@ if __name__ == '__main__':
        elif args[i] == "-l" or args[i] == "--limit-nodes":
            skipthis=1
            LimitNodes = int(args[i+1])
-       elif args[i] == "-r" or args[i] == "--populate-resources":
+       elif args[i] == "-r":
            CIBResource = 1
        elif args[i] == "-L" or args[i] == "--logfile":
            skipthis=1
@@ -575,9 +561,6 @@ if __name__ == '__main__':
        elif args[i] == "--test-ip-base":
            skipthis=1
            IPBase = args[i+1]
-       elif args[i] == "--oprofile":
-           skipthis=1
-           oprofile = args[i+1].split(' ')
        elif args[i] == "--trunc":
            TruncateLog=1
        elif args[i] == "-v2":
@@ -592,12 +575,6 @@ if __name__ == '__main__':
                DoStonith=0
            else:
                usage(args[i+1])
-       elif args[i] == "--stonith-type":
-           StonithType = args[i+1]
-           skipthis=1
-       elif args[i] == "--stonith-args":
-           StonithParams = args[i+1].split('=')
-           skipthis=1
        elif args[i] == "--suppress-cib-writes":
            skipthis=1
            if args[i+1] == "1" or args[i+1] == "yes":
@@ -607,7 +584,7 @@ if __name__ == '__main__':
            else:
                usage(args[i+1])
        elif args[i] == "--bsc":
-           DoBSC=1
+	   DoBSC=1
        elif args[i] == "--standby":
            skipthis=1
            if args[i+1] == "1" or args[i+1] == "yes":
@@ -665,10 +642,7 @@ if __name__ == '__main__':
            HaveSeed = 1
        else:
            NumIter=int(args[i])
-     
-    if not oprofile:
-        oprofile = []
-
+          
     #
     # This reading of HBconfig here is ugly, and I suppose ought to
     # be done by the Cluster manager.  This would probably mean moving the
@@ -681,14 +655,13 @@ if __name__ == '__main__':
     if DoBSC:
         NumIter = 2
         Version = 2
-        while len(node_list) > 1:
-            node_list.pop(len(node_list)-1)
+        LimitNodes = 1
 
     if LogFacility == None:
         if config.Parameters.has_key("logfacility"): 
             LogFacility = config.Parameters["logfacility"][0]
         else:
-            LogFacility = DefaultFacility
+            LogFacility = "local7"
 
     if LimitNodes > 0:
         if len(node_list) > LimitNodes:
@@ -697,9 +670,6 @@ if __name__ == '__main__':
             while len(node_list) > LimitNodes:
                 node_list.pop(len(node_list)-1)
 
-    if StonithParams[0] == "hostlist":
-        StonithParams[1] = string.join(node_list, " ")
-
 #    alt_list = []
 #    for node in node_list:
 #        alt_list.append(string.lower(node))
@@ -724,7 +694,6 @@ if __name__ == '__main__':
     Environment["DoBSC"]    = 0
     Environment["use_logd"] = 0
     Environment["logfacility"] = LogFacility
-    Environment["oprofile"] = oprofile
 
     if config.Parameters.has_key("use_logd"):
         Environment["use_logd"] = 1
@@ -736,7 +705,7 @@ if __name__ == '__main__':
     if HaveSeed:
         Environment["RandSeed"] = Seed 
 
-    Environment["reset"] = Stonith(sttype=StonithType, pName=StonithParams[0], pValue=StonithParams[1])
+    Environment["reset"] = Stonith(sttype="external/ssh", parm=string.join(node_list, " "))
 
     if DoBSC:
         Environment["DoBSC"]       = 1
@@ -768,28 +737,26 @@ if __name__ == '__main__':
     cm.log(">>>>>>>>>>>>>>>> BEGINNING " + repr(NumIter) + " TESTS ")
     cm.log("HA configuration directory: " + Environment["HAdir"])
     cm.log("System log files: " + Environment["LogFileName"])
-    cm.log("Enable Stonith:   %d" % Environment["DoStonith"])
-    cm.log("Enable Fencing:   %d" % Environment["DoFencing"])
-    cm.log("Enable Standby:   %d" % Environment["DoStandby"])
-    cm.log("Enable Resources: %d" % Environment["CIBResource"])
+    cm.log("Enable Stonith: " + ("%d" % Environment["DoStonith"]))
+    cm.log("Enable Fencing: " + ("%d" % Environment["DoFencing"]))
+    cm.log("Enable Standby: " + ("%d" % Environment["DoStandby"]))
     if Environment.has_key("SuppressMonitoring") \
     and Environment["SuppressMonitoring"]:
         cm.log("Resource Monitoring is disabled")
-    cm.ns.WaitForAllNodesToComeUp(config.Parameters["node"])
     cm.log("Cluster nodes: ")
     for node in config.Parameters["node"]:
-        (rc, lines) = cm.rsh.remote_py(node, "os", "system", 
-                                     "/usr/local/sbin/crm_uuid")
+	(rc, lines) = cm.rsh.remote_py(node, "os", "system", 
+				     "/usr/sbin/crm_uuid")
         if not lines:
             cm.log("    * %s: __undefined_uuid__" % node)
         else:
-            out=lines[0]
+	    out=lines[0]
             out = out[:-1]
             cm.log("    * %s: %s" % (node, out))
-
+        
     Audits = AuditList(cm)
     Tests = []
-
+    
     if Environment["DoBSC"]:
         test = BSC_AddResource(cm)
         Tests.append(test)
