@@ -29,15 +29,15 @@ Add RecourceRecover testcase Zhao Kai <zhaokai@cn.ibm.com>
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #
-#        SPECIAL NOTE:
+#	SPECIAL NOTE:
 #
-#        Tests may NOT implement any cluster-manager-specific code in them.
-#        EXTEND the ClusterManager object to provide the base capabilities
-#        the test needs if you need to do something that the current CM classes
-#        do not.  Otherwise you screw up the whole point of the object structure
-#        in CTS.
+#	Tests may NOT implement any cluster-manager-specific code in them.
+#	EXTEND the ClusterManager object to provide the base capabilities
+#	the test needs if you need to do something that the current CM classes
+#	do not.  Otherwise you screw up the whole point of the object structure
+#	in CTS.
 #
-#                Thank you.
+#		Thank you.
 #
 
 import CTS
@@ -46,6 +46,7 @@ import CTSaudits
 import time, os, re, types, string, tempfile, sys
 from CTSaudits import *
 from stat import *
+from xml.dom.minidom import *
 
 #        List of all class objects for tests which we ought to
 #        consider running.
@@ -93,24 +94,27 @@ class RandomTests:
             errcount=0
             BadNewsDebug=0
             #BadNews.debug=1
-            ignorelist = []                
-            ignorelist.append(" CTS: ")
-            ignorelist.append("BadNews:")
-            ignorelist.extend(self.CM.errorstoignore())
-
-            if test:
-                ignorelist.extend(test.errorstoignore())
-
             while errcount < 1000:
                 if BadNewsDebug: print "Looking for BadNews"
                 match=BadNews.look(0)
                 if match:
                    if BadNewsDebug: print "BadNews found: "+match
                    add_err = 1
+                   ignorelist = []
+                   if test:
+                       ignorelist=test.errorstoignore()
+                   ignorelist.append(" CTS: ")
+                   ignorelist.append("BadNews:")
                    for ignore in ignorelist:
-                       if add_err == 1 and re.search(ignore, match):
-                           if BadNewsDebug: print "Ignoring based on pattern: ("+ignore+")"
+                       if re.search(ignore, match):
+                           if BadNewsDebug: print "Ignoring:"+match+" (pattern: "+ignore+")"
                            add_err = 0
+                   if add_err == 1:
+                       ignorelist=self.CM.errorstoignore()
+                       for ignore in ignorelist:
+                           if re.search(ignore, match):
+                               if BadNewsDebug: print "Ignoring:"+match+" (pattern: "+ignore+")"
+                               add_err = 0
                    if add_err == 1:
                        self.CM.log("BadNews: " + match)
                        self.incr("BadNews")
@@ -149,22 +153,10 @@ random for the selected number of iterations.
         ,        timeout=0)
         BadNews.setwatch()
 
-        self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"])
-
-        for node in self.CM.Env["nodes"]:
-            if node in self.CM.Env["oprofile"]:
-                self.CM.log("Enabling oprofile on %s" % node) 
-                self.CM.rsh.remote_py(node, "os", "system", "opcontrol --init")
-                self.CM.rsh.remote_py(node, "os", "system", "opcontrol --start")
-
         if not self.Scenario.SetUp(self.CM):
             return None
 
-        for node in self.CM.Env["nodes"]:
-            if node in self.CM.Env["oprofile"]:
-                self.CM.rsh.remote_py(
-                    node, "os", "system", "opcontrol --save=cts.setup")
-
+        self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"])
         testcount=1
         time.sleep(30)
 
@@ -188,20 +180,6 @@ random for the selected number of iterations.
             test.starttime=starttime
             ret=test(nodechoice)
 
-            for node in self.CM.Env["nodes"]:
-                if node in self.CM.Env["oprofile"]:
-                    self.CM.rsh.remote_py(
-                        node, "os", "system", 
-                        "opcontrol --save=cts.%d" % (testcount-1))
-
-            if not self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"]):
-              if os.path.isfile("./RecoverFromDeadNode"):
-                self.CM.log("Calling ./RecoverFromDeadNode in an attempt to get things going again.")
-                os.system("./RecoverFromDeadNode")
-              if not self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"]):
-                self.CM.log("One or more nodes will not come up - exiting")
-                break
-
             if ret:
                 self.incr("success")
             else:
@@ -211,6 +189,7 @@ random for the selected number of iterations.
                 self.CM.statall()
                 # Make sure logging is working and we have enough disk space...
                 if not self.CM.Env["DoBSC"]:
+                    self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"])
                     if not self.CM.TestLogging():
                         sys.exit(1)
                     if not self.CM.CheckDf():
@@ -233,11 +212,6 @@ random for the selected number of iterations.
 
         self.Scenario.TearDown(self.CM)
         
-        for node in self.CM.Env["nodes"]:
-            if node in self.CM.Env["oprofile"]:
-                self.CM.log("Disabling oprofile on %s" % node) 
-                self.CM.rsh.remote_py(node, "os", "system", "opcontrol --shutdown")
-
         self.audit(BadNews, None)
 
         for test in self.Tests:
@@ -367,25 +341,22 @@ class StopTest(CTSTest):
         self.CM.StopaCM(node)
         watch_result = watch.lookforall()
 
-        failreason=None
         UnmatchedList = "||"
         if watch.unmatched:
             for regex in watch.unmatched:
-                self.CM.log ("ERROR: Shutdown pattern not found: %s" % (regex))
+                self.CM.log ("Warn: Shutdown pattern not found: %s" % (regex))
                 UnmatchedList +=  regex + "||";
-                failreason="Missing shutdown pattern"
 
         self.CM.cluster_stable(self.CM["DeadTime"])
 
+        # because syslog looses so many messages we can only really fail 
+        #  the stop if _none_ of the CCM peers notice the node leave
         if not watch.unmatched or self.CM.upcount() == 0:
             return self.success()
         elif len(watch.unmatched) >= self.CM.upcount():
             return self.failure("no match against (%s)" % UnmatchedList)
 
-        if failreason == None:
-            return self.success()
-        else:
-            return self.failure(failreason)
+        return self.success()
 #
 # We don't register StopTest because it's better when called by
 # another test...
@@ -516,7 +487,6 @@ class StonithTest(CTSTest):
         self.usstart      = self.CM["Pat:We_started"]
         self.themstart    = self.CM["Pat:They_started"]
         self.timeout      = timeout
-        self.ssherror     = False
     
     def _reset(self, node):
         StonithWorked=False
@@ -570,14 +540,10 @@ class StonithTest(CTSTest):
 
         #        Reset (stonith) the node
 
-        self.CM.debug("Resetting: "+node)
         StonithWorked = self._reset(node)
 
         if not StonithWorked:
             return self.failure("Stonith didn't work")
-        if self.ssherror == True:
-            self.CM.log("NOTE: Stonith command reported success but node %s did not restart (atd, reboot or ssh error)" % node)
-            return self.success()
 
         upwatch.setwatch()
 
@@ -617,11 +583,6 @@ class StonithTest(CTSTest):
         # wait for the cluster to stabilize
         self.CM.cluster_stable()
 
-        if node in self.CM.Env["oprofile"]:
-            self.CM.log("Enabling oprofile on %s" % node) 
-            self.CM.rsh.remote_py(node, "os", "system", "opcontrol --init")
-            self.CM.rsh.remote_py(node, "os", "system", "opcontrol --start")
-
         # return case processing
         if rc == 0:
             return self.failure("Node %s did not restart" %node)
@@ -659,35 +620,14 @@ class StonithdTest(StonithTest):
             return self.skipped()
 
         StonithWorked = False
-        SshNotWork = 0
         for tries in range(1,5):
-            # For some unknown reason, every now and then the ssh plugin just
-            # can't kill the target_node - everything works fine with stonithd
-            # and the plugin, but atd, reboot or ssh (or maybe something else)
-            # doesn't do its job and target_node remains alive.  So look for
-            # the indicative messages and bubble-up the error via ssherror
-            watchpats = []
-            watchpats.append("Initiating ssh-reset")
-            watchpats.append("CRIT: still able to ping")
-                
-            watch = CTS.LogWatcher(self.CM["LogFileName"], watchpats
-            ,     timeout=self.CM["DeadTime"]+60)
-            watch.setwatch()
-        
             fail_reasons = []
             if self.CM.Env.ResetNode2(self.init_node, target_node, fail_reasons):
                 StonithWorked = True
                 break
-            if watch.lookforall():
-                SshNotWork = SshNotWork + 1
-                continue
             for reason in fail_reasons:
                 self.CM.log(reason)
 
-        if StonithWorked == False and SshNotWork == tries:
-            StonithWorked = True
-            self.ssherror = True
-
         return StonithWorked
 
     def setup(self, target_node):
@@ -844,7 +784,6 @@ class SimulStart(CTSTest):
 #        Register SimulStart as a good test to run
 AllTestClasses.append(SimulStart)
 
-###################################################################
 class SimulStop(CTSTest):
 ###################################################################
     '''Stop all the nodes ~ simultaneously'''
@@ -881,7 +820,6 @@ class SimulStop(CTSTest):
 #     Register SimulStop as a good test to run
 AllTestClasses.append(SimulStop)
 
-###################################################################
 class StopOnebyOne(CTSTest):
 ###################################################################
     '''Stop all the nodes in order'''
@@ -921,10 +859,9 @@ class StopOnebyOne(CTSTest):
 #     Register StopOnebyOne as a good test to run
 AllTestClasses.append(StopOnebyOne)
 
-###################################################################
 class RestartOnebyOne(CTSTest):
 ###################################################################
-    '''Restart all the nodes in order'''
+    '''Stop all the nodes in order'''
     def __init__(self, cm):
         CTSTest.__init__(self,cm)
         self.name="RestartOnebyOne"
@@ -960,54 +897,6 @@ class RestartOnebyOne(CTSTest):
 #     Register StopOnebyOne as a good test to run
 AllTestClasses.append(RestartOnebyOne)
 
-###################################################################
-class PartialStart(CTSTest):
-###################################################################
-    '''Start a node - but tell it to stop before it finishes starting up'''
-    def __init__(self, cm):
-        CTSTest.__init__(self,cm)
-        self.name="PartialStart"
-        self.startall = SimulStartLite(cm)
-        self.stopall = SimulStopLite(cm)
-
-    def __call__(self, node):
-        '''Perform the 'PartialStart' test. '''
-        self.incr("calls")
-
-        ret = self.stopall(None)
-        if not ret:
-            return self.failure("Setup failed")
-
-#	FIXME!  This should use the CM class to get the pattern
-#		then it would be applicable in general
-        watchpats = []
-        watchpats.append("Starting crmd")
-        watch = CTS.LogWatcher(self.CM["LogFileName"], watchpats,
-                               timeout=self.CM["DeadTime"]+10)
-        watch.setwatch()
-
-        self.CM.StartaCMnoBlock(node)
-        ret = watch.lookforall()
-        if not ret:
-            self.CM.log("Patterns not found: " + repr(watch.unmatched))
-            return self.failure("Setup of %s failed" % node) 
-
-        ret = self.stopall(None)
-        if not ret:
-            return self.failure("%s did not stop in time" % node)
-
-        return self.success()
-
-    def is_applicable(self):
-        '''Partial is always applicable'''
-        if self.CM["Name"] == "linux-ha-v2":
-          return 1
-        else:
-          return 0
-
-#     Register StopOnebyOne as a good test to run
-AllTestClasses.append(PartialStart)
-
 ###################################################################
 class StandbyTest(CTSTest):
 ###################################################################
@@ -1220,7 +1109,7 @@ class Fastdetection(CTSTest):
             if stopwatch.look():
                 Stoptime = os.times()[4]
 #        This test is CM-specific - FIXME!!
-                self.CM.rsh(node, "killall -9 /usr/local/lib/heartbeat/ccm /usr/local/lib/heartbeat/ipfail >/dev/null 2>&1; true")
+                self.CM.rsh(node, "killall -9 /usr/lib64/heartbeat/ccm /usr/lib64/heartbeat/ipfail >/dev/null 2>&1; true")
                 Detectiontime = Stoptime-Starttime
                 detectms = int(Detectiontime*1000+0.5)
                 self.CM.log("...failure detection time: %d ms" % detectms)
@@ -1236,7 +1125,7 @@ class Fastdetection(CTSTest):
                 return self.success()
             else:
 #        This test is CM-specific - FIXME!!
-                self.CM.rsh(node, "killall -9 /usr/local/lib/heartbeat/ccm /usr/local/lib/heartbeat/ipfail >/dev/null 2>&1; true")
+                self.CM.rsh(node, "killall -9 /usr/lib64/heartbeat/ccm /usr/lib64/heartbeat/ipfail >/dev/null 2>&1; true")
                 self.CM.ShouldBeStatus[node] = self.CM["down"]
                 ret=self.start(node)
                 return self.failure("Didn't find the log message")
@@ -1664,12 +1553,8 @@ class ResourceRecover(CTSTest):
         self.startall = SimulStartLite(cm)
         self.max=30
         self.rid=None
-        self.action = "fail"
 
-        # make sure the interval is greater than 0 so failcount is updated
-        self.interval = 60000
-
-    def __call__(self, node):
+    def __call__(self, dummy):
         '''Perform the 'ResourceRecover' test. '''
         self.incr("calls")
         
@@ -1677,49 +1562,47 @@ class ResourceRecover(CTSTest):
         if not ret:
             return self.failure("Setup failed")
 
-        resourcelist = self.CM.active_resources(node)
+        resourcelist = self.CM.Resources()
         # if there are no resourcelist, return directly
         if len(resourcelist)==0:
-            self.CM.log("No active resources on %s" % node)
+            self.CM.log("Can't find any resource on cluster,exit this test")
             return self.skipped()
 
-        self.rid = self.CM.Env.RandomGen.choice(resourcelist)
-        self.CM.debug("Shooting %s..." % self.rid)
+        resource=self.CM.Env.RandomGen.choice(resourcelist)
+        resourcetype = str(resource.Type())
+        self.rid = str(resource.rid)
+        label = resourcetype+"::"+str(resource.rid)
+        nodes=resource.RunningNodes()
+        if len(nodes) == 0:
+            return self.skipped()
+        self.CM.log("...resource %s on %s" % (label, nodes[0]))
 
-        pats = []
-        pats.append("crmd.* Performing op=%s_stop_0" % self.rid)
-        pats.append("crmd.* Performing op=%s_start_0" % self.rid)
-        pats.append("crmd.* LRM operation %s_start_0.*complete" % self.rid)
-        pats.append("Updating failcount for %s on .* after .* %s"
-                    % (self.rid, self.action))
 
-        watch = CTS.LogWatcher(self.CM["LogFileName"], pats, timeout=60)
+        pat=("""crmd.* Performing op start on %s""" % (resource.rid))
+        watch = CTS.LogWatcher(self.CM["LogFileName"], [pat], timeout=240)
         watch.setwatch()
         
-        # fail a resource by calling an action it doesn't support
-        self.CM.rsh.remote_py(node, "os", "system",
-                              "/usr/local/sbin/crm_resource -F -r %s -H %s &>/dev/null" % (self.rid, node))
+        # stop a resource
+        resource.Stop(nodes[0])
 
-        watch.lookforall()
+        # wait for the resource to be started
+        if not watch.look():
+            self.CM.log("WARN: No match for pattern: %s" %repr(pat))
 
-        self.CM.cluster_stable()
-        recovernode=self.CM.ResourceLocation(self.rid)
+        # should be ample time for the LRM + RA to complete
+        time.sleep(10)
 
+        recovernode=resource.RunningNodes()
         if len(recovernode)==1:
             self.CM.debug("Recovered: %s is running on %s" 
-                          %(self.rid, recovernode[0]))
-            if not watch.unmatched: 
-                return self.success()
-            else:
-                return self.failure("Patterns not found: %s" 
-                                    % repr(watch.unmatched))
-
+                          %(label, recovernode[0]))
+            self.CM.cluster_stable()
+            return self.success()
         elif len(recovernode)==0:
-            return self.failure("%s was not recovered and is inactive" 
-                                % self.rid)
+            return self.failure("%s was not recovered and is inactive" %(label))
         else:
             return self.failure("%s is now active on more than one node: %s"
-                                %(self.rid, str(recovernode)))
+                                %(label, str(recovernode)))
 
     def is_applicable(self):
         '''ResourceRecover is applicable only when there are resources running
@@ -1733,13 +1616,13 @@ class ResourceRecover(CTSTest):
                 return 1
         return 0
     
+#
+#        FIXME!!  This test has hard-coded cluster-manager-specific things in it!!
+#        
     def errorstoignore(self):
         '''Return list of errors which should be ignored'''
-        return [ """Updating failcount for %s""" % self.rid,
-                 """Unknown operation: fail""",
-                 """ERROR: sending stonithRA op to stonithd failed.""",
-                 """ERROR: process_lrm_event: LRM operation %s_%s_%d""" % (self.rid, self.action, self.interval),
-                 """ERROR: process_graph_event: Action %s_%s_%d initiated outside of a transition""" % (self.rid, self.action, self.interval),
+        return [ """ERROR: .* Action .*%s_monitor_.* on .* failed .*target: 0 vs. rc: 7.*: Error""" % self.rid,
+                 """Updating failcount for """
                  ]
 
 AllTestClasses.append(ResourceRecover)
@@ -1750,98 +1633,51 @@ class ComponentFail(CTSTest):
     def __init__(self, cm):
         CTSTest.__init__(self,cm)
         self.name="ComponentFail"
+        self.start = StartTest(cm)
         self.startall = SimulStartLite(cm)
+        self.stopall = SimulStopLite(cm)
         self.complist = cm.Components()
-        self.patterns = []
-        self.okerrpatterns = []
-
+        self.theystart = cm["Pat:They_started"]
+        
     def __call__(self, node):
         '''Perform the 'ComponentFail' test. '''
         self.incr("calls")
-        self.patterns = []
-        self.okerrpatterns = []
-
+        
         # start all nodes
+        if not self.CM.cluster_stable():
+            self.stopall(None)        
+            
         ret = self.startall(None)
         if not ret:
             return self.failure("Setup failed")
 
-        if not self.CM.cluster_stable(self.CM["StableTime"]):
-            return self.failure("Setup failed - unstable")
-
-        node_is_dc = self.CM.is_node_dc(node, None)
-
         # select a component to kill
-        chosen = self.CM.Env.RandomGen.choice(self.complist)
-        while chosen.dc_only == 1 and node_is_dc == 0:
-            chosen = self.CM.Env.RandomGen.choice(self.complist)
-
-        self.CM.log("...component %s (dc=%d,boot=%d)" % (chosen.name, node_is_dc,chosen.triggersreboot))
-        self.incr(chosen.name)
+        component = self.CM.Env.RandomGen.choice(self.complist)
+        self.CM.log("choose %s to kill"%component.name)
+        patterns = []
+        patterns.append("%s heartbeat.*Respawning.*%s" %(node, component.name))
+        patterns.append(self.theystart%node)
         
-        self.patterns.extend(chosen.pats)
-
-        # Make sure the node goes down and then comes back up if it should reboot...
-        if chosen.triggersreboot:
-          for other in self.CM.Env["nodes"]:
-              if other != node:
-                  self.patterns.append(self.CM["Pat:They_stopped"] %(other, node))
-          self.patterns.append(self.CM["Pat:They_started"] % node)
-
-        # In an ideal world, this next stuff should be in the "chosen" object as a member function
-        if chosen.dc_only:
-          if chosen.triggersreboot:
-            # Sometimes these will be in the log, and sometimes they won't...
-            self.okerrpatterns.append("%s crmd:.*Process %s:.* exited" %(node, chosen.name))
-            self.okerrpatterns.append("%s crmd:.*I_ERROR.*crmdManagedChildDied" %node)
-            self.okerrpatterns.append("%s crmd:.*The %s subsystem terminated unexpectedly" %(node, chosen.name))
-            self.okerrpatterns.append("ERROR: Client .* exited with return code")
-          else:
-            self.patterns.append("%s crmd:.*Process %s:.* exited" %(node, chosen.name))
-            self.patterns.append("%s crmd:.*I_ERROR.*crmdManagedChildDied" %node)
-            self.patterns.append("%s crmd:.*The %s subsystem terminated unexpectedly" %(node, chosen.name))
-        else:
-          if chosen.triggersreboot:
-            # Sometimes this won't be in the log...
-            self.okerrpatterns.append("%s heartbeat.*%s.*killed by signal 9" %(node, chosen.name))
-            self.okerrpatterns.append("%s heartbeat.*Respawning client " % (node))
-            self.okerrpatterns.append("ERROR: Client .* exited with return code")
-          else:
-            self.patterns.append("%s heartbeat.*%s.*killed by signal 9" %(node, chosen.name))
-            self.patterns.append("%s heartbeat.*Respawning client.*%s" %(node, chosen.name))
-        if node_is_dc:
-          self.patterns.extend(chosen.dc_pats)
-
-        # supply a copy so self.patterns doesnt end up empty
-        tmpPats = []
-        tmpPats.extend(self.patterns)
-        self.patterns.extend(chosen.badnews_ignore)
-
         # set the watch for stable
         watch = CTS.LogWatcher(
-            self.CM["LogFileName"], tmpPats, 
-            self.CM["DeadTime"] + self.CM["StableTime"] + self.CM["StartTime"])
+            self.CM["LogFileName"], patterns, self.CM["DeadTime"]+10)
         watch.setwatch()
         
         # kill the component
-        chosen.kill(node)
-
-        # check to see Heartbeat noticed
-        matched = watch.lookforall()
-        if not matched:
-            self.CM.log("Patterns not found: " + repr(watch.unmatched))
-            self.CM.cluster_stable(self.CM["StartTime"])
-            return self.failure("Didn't find all expected patterns")
+        component.kill(node)
         
-        self.CM.debug("Found: "+ repr(matched))
+        # check to see Heartbeat noticed
+        match = watch.look()
+        if match:
+            self.CM.log("Found match: %s"%(match))
 
-        # now watch it recover...
-        for attempt in (1, 2, 3, 4, 5):
-            self.CM.debug("Waiting for the cluster to recover...")
-            if self.CM.cluster_stable(self.CM["StartTime"]):
+            # now watch it recover...
+            if self.CM.cluster_stable():
                 return self.success()
-
-        return self.failure("Cluster did not become stable")
+            else:
+                self.failure("Cluster not stable")
+        else:
+            return self.failure("Heartbeat didnt notice %s die" %component)
 
     def is_applicable(self):
         if self.CM["Name"] == "linux-ha-v2":
@@ -1850,12 +1686,9 @@ class ComponentFail(CTSTest):
     
     def errorstoignore(self):
         '''Return list of errors which should be ignored'''
-	# Note that okerrpatterns refers to the last time we ran this test
-	# The good news is that this works fine for us...
-        self.okerrpatterns.extend(self.patterns)
-        return self.okerrpatterns
+        return ["""heartbeat.*killed by signal 9""", """heartbeat.*Respawning"""]
     
-AllTestClasses.append(ComponentFail)
+#AllTestClasses.append(ComponentFail)
 
 ####################################################################
 class Split_brainTest2(CTSTest):
@@ -2200,6 +2033,10 @@ class NearQuorumPointTest(CTSTest):
         self.CM.debug("start nodes:" + repr(startset))
         self.CM.debug("stop nodes:" + repr(stopset))
 
+        if not self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"]):
+            self.log("WaitForAllNodesToComeUp filed")
+            return self.skipped()
+ 
         #add search patterns
         watchpats = [ ]
         for node in stopset:
@@ -2213,9 +2050,6 @@ class NearQuorumPointTest(CTSTest):
         if len(watchpats) == 0:
             return self.skipped()
 
-        if len(startset) != 0:
-            watchpats.append(self.CM["Pat:DC_IDLE"])
-
         watch = CTS.LogWatcher(self.CM["LogFileName"], watchpats
         ,     timeout=self.CM["DeadTime"]+10)
         
@@ -2279,13 +2113,13 @@ class BSC_AddResource(CTSTest):
         CTSTest.__init__(self, cm)
         self.name="AddResource"
         self.resource_offset = 0
-        self.cib_cmd="""/usr/local/sbin/cibadmin -C -o %s -X '%s' """
+        self.cib_cmd="""/usr/sbin/cibadmin -C -o %s -X '%s' """
 
     def __call__(self, node):
         self.resource_offset =         self.resource_offset  + 1
 
         r_id = "bsc-rsc-%s-%d" % (node, self.resource_offset)
-        start_pat = "crmd.*%s_start_0.*complete"
+        start_pat = "crmd.*start_0 on %s complete"
 
         patterns = []
         patterns.append(start_pat % r_id)
@@ -2372,6 +2206,10 @@ class SimulStopLite(CTSTest):
         '''Perform the 'SimulStopLite' setup work. '''
         self.incr("calls")
 
+        if not self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"]):
+            self.log("WaitForAllNodesToComeUp filed")
+            return self.skipped()
+
         self.CM.debug("Setup: " + self.name)
 
         #     We ignore the "node" parameter...
@@ -2415,7 +2253,7 @@ class SimulStopLite(CTSTest):
                     + repr(watch.unmatched))
 
         self.CM.clear_all_caches()
-        return self.failure("Missing log message: "+repr(watch.unmatched))
+        return self.success()
 
     def is_applicable(self):
         '''SimulStopLite is a setup test and never applicable'''
@@ -2434,6 +2272,10 @@ class SimulStartLite(CTSTest):
         self.incr("calls")
         self.CM.debug("Setup: " + self.name)
 
+        if not self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"]):
+            self.log("WaitForAllNodesToComeUp filed")
+            return self.skipped()
+
         #        We ignore the "node" parameter...
         watchpats = [ ]
 
@@ -2481,9 +2323,10 @@ class SimulStartLite(CTSTest):
             return self.failure("Unstable cluster nodes exist: " 
                                 + repr(unstable))
 
-        self.CM.log("ERROR: All nodes started but CTS didnt detect: " 
+        self.CM.log("Warn: All nodes started but CTS didnt detect: " 
                     + repr(watch.unmatched))
-        return self.failure() 
+
+        return self.success() 
 
 
     def is_applicable(self):
@@ -2501,6 +2344,8 @@ class LoggingTest(CTSTest):
         '''Perform the 'Logging' test. '''
         self.incr("calls")
         
+        self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"])
+
         # Make sure logging is working and we have enough disk space...
         if not self.CM.TestLogging():
             sys.exit(1)
