@@ -1,4 +1,23 @@
-/* $Id: hb_resource.h,v 1.3.2.1 2004/04/20 05:05:28 alan Exp $ */
+/* $Id: hb_resource.h,v 1.8 2005/07/29 06:55:37 sunjd Exp $ */
+/*
+ * Copyright (C) 2001-2002 Luis Claudio R. Goncalves
+ *                              <lclaudio@conectiva.com.br>
+ * Copyright (C) 1999-2002 Alan Robertson <alanr@unix.sh>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 #ifndef _HB_RESOURCE_H
 #define _HB_RESOURCE_H
 #include <heartbeat.h>
@@ -14,7 +33,7 @@ enum hb_rsc_state {
 				/* BOTHSTARTING now equiv to STARTING (?) */
 	HB_R_RSCRCVD,		/* Resource Message received */
 	HB_R_STABLE,		/* Local resources acquired, too... */
-	HB_R_SHUTDOWN,		/* We're in shutdown... */
+	HB_R_SHUTDOWN		/* We're in shutdown... */
 };
 
 /*
