@@ -1,4 +1,4 @@
-/* $Id: cl_malloc.h,v 1.4 2005/05/19 15:50:46 alan Exp $ */
+/* $Id: cl_malloc.h,v 1.6 2006/07/07 20:36:31 andrew Exp $ */
 /*
  * ha_malloc.h: malloc utilities for the Linux-HA heartbeat program
  *
@@ -34,9 +34,30 @@ typedef struct cl_mem_stats_s {
 						/* arena value */
 }cl_mem_stats_t;
 
+/* This allows the code to track _all_ allocated memory, who allocated
+ * it and from where in the code it was allocated. It is a very memory
+ * expensive debugging tool and NOT! meant for production: */
+#undef HA_MALLOC_TRACK
+
 void*		cl_malloc(size_t size);
 void*		cl_calloc(size_t nmemb, size_t size);
 void*		cl_realloc(void* oldval, size_t newsize);
+#ifdef HA_MALLOC_TRACK
+void*		cl_malloc_track(size_t size, 
+		const char *file, const char *function, const int line);
+void*		cl_calloc_track(size_t nmemb, size_t size,
+		const char *file, const char *function, const int line);
+void*		cl_realloc_track(void* oldval, size_t newsize,
+		const char *file, const char *function, const int line);
+#ifndef	HA_MALLOC_ORIGINAL
+#define		cl_malloc(s) cl_malloc_track(s, \
+		__FILE__, __PRETTY_FUNCTION__, __LINE__)
+#define		cl_calloc(n,s) cl_calloc_track(n,s, \
+		__FILE__, __PRETTY_FUNCTION__, __LINE__)
+#define		cl_realloc(o,s) cl_realloc_track(o,s, \
+		__FILE__, __PRETTY_FUNCTION__, __LINE__)
+#endif
+#endif
 char*		cl_strdup(const char *s);
 void		cl_free(void *ptr);
 int		cl_is_allocated(const void *ptr);
@@ -46,6 +67,9 @@ volatile cl_mem_stats_t* cl_malloc_getstats(void);
 void		cl_malloc_forced_for_glib(void);
 		/* Call before using any glib functions(!) */
 		/* See also: g_mem_set_vtable() */
+#ifdef HA_MALLOC_TRACK
+void		 cl_malloc_dump_allocated(int log_level, int filter_seen);
+#endif
 
 #define	MALLOCT(t)	((t *) cl_malloc(sizeof(t)))
 
