@@ -21,4 +21,9 @@
 int	cl_str_to_boolean(const char*, int*);
 
 int	cl_file_exists(const char* filename);
+
+char*	cl_get_env(const char* env_name);
+
+int	cl_binary_to_int(const char* data, int len);
+
 #endif
