@@ -1,3 +1,4 @@
+/* $Id: status.h,v 1.3 2006/06/21 14:48:01 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -23,8 +24,10 @@
 #include <crm/pengine/common.h>
 
 typedef struct node_s node_t;
+typedef struct color_s color_t;
 typedef struct action_s action_t;
 typedef struct resource_s resource_t;
+typedef struct action_wrapper_s action_wrapper_t;
 
 typedef enum no_quorum_policy_e {
 	no_quorum_freeze,
@@ -57,8 +60,6 @@ typedef struct pe_working_set_s
 		gboolean symmetric_cluster;
 		gboolean is_managed_default;
 
-		gboolean start_failure_fatal;
-	
 		gboolean remove_after_stop;
 		gboolean stop_rsc_orphans;
 		gboolean stop_action_orphans;
@@ -69,20 +70,23 @@ typedef struct pe_working_set_s
 
 		GHashTable *config_hash;
 		
+		/* intermediate steps */
+		color_t *no_color;
+		
 		GListPtr nodes;
 		GListPtr resources;
 		GListPtr placement_constraints;
 		GListPtr ordering_constraints;
-		GListPtr colocation_constraints;
 		
+		GListPtr colors;
 		GListPtr actions;
-		crm_data_t *failed;
 
 		/* stats */
 		int num_synapse;
 		int max_valid_nodes;
 		int order_id;
 		int action_id;
+		int color_id;
 
 		/* final output */
 		crm_data_t *graph;
@@ -100,7 +104,6 @@ struct node_shared_s {
 		gboolean is_dc;
 		int	 num_resources;
 		GListPtr running_rsc;	/* resource_t* */
-		GListPtr allocated_rsc;	/* resource_t* */
 		
 		GHashTable *attrs;	/* char* => char* */
 		enum node_type type;
@@ -109,31 +112,11 @@ struct node_shared_s {
 struct node_s { 
 		int	weight; 
 		gboolean fixed;
-		int      count;
 		struct node_shared_s *details;
 };
 
 #include <crm/pengine/complex.h>
 
-#define pe_rsc_orphan		0x00000001ULL
-#define pe_rsc_managed		0x00000002ULL
-
-#define pe_rsc_notify		0x00000010ULL
-#define pe_rsc_unique		0x00000020ULL
-#define pe_rsc_can_migrate	0x00000040ULL
-
-#define pe_rsc_provisional	0x00000100ULL
-#define pe_rsc_allocating	0x00000200ULL
-#define pe_rsc_merging		0x00000400ULL
-
-#define pe_rsc_failed		0x00010000ULL
-#define pe_rsc_shutdown		0x00020000ULL
-#define pe_rsc_runnable		0x00040000ULL
-#define pe_rsc_start_pending	0x00080000ULL
-
-#define pe_rsc_starting		0x00100000ULL
-#define pe_rsc_stopping		0x00200000ULL
-
 struct resource_s { 
 		char *id; 
 		char *clone_name; 
@@ -152,18 +135,29 @@ struct resource_s {
 
 		int	 priority; 
 		int	 stickiness; 
-		int	 sort_index; 
 		int	 fail_stickiness;
 		int	 effective_priority; 
 
-		unsigned long long flags;
-	
-		GListPtr rsc_cons_lhs;     /* rsc_colocation_t* */
+		gboolean notify;
+		gboolean is_managed;
+		gboolean starting;
+		gboolean stopping;
+		gboolean runnable;
+		gboolean provisional;
+		gboolean globally_unique;
+		
+		gboolean failed;
+		gboolean start_pending;
+		
+		gboolean orphan;
+		
+		GListPtr candidate_colors; /* color_t*          */
 		GListPtr rsc_cons;         /* rsc_colocation_t* */
 		GListPtr rsc_location;     /* rsc_to_node_t*    */
 		GListPtr actions;	   /* action_t*         */
 
-		node_t *allocated_to;
+		color_t *color;
+		GListPtr colors;	   /* color_t*  */
 		GListPtr running_on;       /* node_t*   */
 		GListPtr known_on;	   /* node_t* */
 		GListPtr allowed_nodes;    /* node_t*   */
@@ -172,9 +166,7 @@ struct resource_s {
 		enum rsc_role_e next_role;
 
 		GHashTable *meta;	   
-		GHashTable *parameters;
-
-		GListPtr children;	  /* resource_t* */	
+		GHashTable *parameters;	   
 };
 
 struct action_s 
@@ -184,7 +176,7 @@ struct action_s
 		resource_t *rsc;
 		void       *rsc_opaque;
 		node_t     *node;
-		char *task;
+		const char *task;
 
 		char *uuid;
 		crm_data_t *op_entry;
@@ -192,10 +184,7 @@ struct action_s
 		gboolean pseudo;
 		gboolean runnable;
 		gboolean optional;
-		gboolean print_always;
 		gboolean failure_is_fatal;
-		gboolean implied_by_stonith;
-		gboolean allow_reload_conversion;
 
 		enum rsc_start_requirement needs;
 		enum action_fail_response  on_fail;
@@ -219,11 +208,15 @@ struct action_s
 		GListPtr actions_after;  /* action_warpper_t* */
 };
 
+struct action_wrapper_s 
+{
+		enum pe_ordering type;
+		action_t *action;
+};
+
 gboolean cluster_status(pe_working_set_t *data_set);
 extern void set_working_set_defaults(pe_working_set_t *data_set);
 extern void cleanup_calculations(pe_working_set_t *data_set);
 extern resource_t *pe_find_resource(GListPtr rsc_list, const char *id_rh);
-extern node_t *pe_find_node(GListPtr node_list, const char *uname);
-extern node_t *pe_find_node_id(GListPtr node_list, const char *id);
 
 #endif
