@@ -50,7 +50,6 @@
 #define F_LRM_RPROVIDER		"lrm_rprovider"
 #define F_LRM_RPROVIDERS	"lrm_rproviders"
 #define F_LRM_PARAM		"lrm_param"
-#define F_LRM_COPYPARAMS	"lrm_copyparams"
 #define F_LRM_TIMEOUT		"lrm_timeout"
 #define F_LRM_OP		"lrm_op"
 #define F_LRM_OPCNT		"lrm_opcount"
@@ -70,11 +69,6 @@
 #define F_LRM_METADATA		"lrm_metadata"
 #define F_LRM_USERDATA		"lrm_userdata"
 #define F_LRM_DELAY		"lrm_delay"
-#define F_LRM_T_RUN		"lrm_t_run"
-#define F_LRM_T_RCCHANGE	"lrm_t_rcchange"
-#define F_LRM_EXEC_TIME		"lrm_exec_time"
-#define F_LRM_QUEUE_TIME	"lrm_queue_time"
-
 #define	PRINT 	printf("file:%s,line:%d\n",__FILE__,__LINE__);
 
 
