@@ -26,8 +26,8 @@
 
 /* Uniform return value of executing RA */
 enum UNIFORM_RET_EXECRA {
-	EXECRA_EXEC_UNKNOWN_ERROR = 152,
-	EXECRA_NO_RA = 151,
+	EXECRA_EXEC_UNKNOWN_ERROR = -2,
+	EXECRA_NO_RA = -1,
 	EXECRA_OK = 0,
 	EXECRA_UNKNOWN_ERROR = 1,
 	EXECRA_INVALID_PARAM = 2,
@@ -36,6 +36,8 @@ enum UNIFORM_RET_EXECRA {
 	EXECRA_NOT_INSTALLED = 5,
 	EXECRA_NOT_CONFIGURED = 6,
 	EXECRA_NOT_RUNNING = 7,
+	EXECRA_RUNNING_MASTER = 8,
+	EXECRA_FAILED_MASTER = 9,
 		
 	/* For status command only */
 	EXECRA_RA_DEAMON_DEAD1 = 11,
