@@ -1,4 +1,4 @@
-/* $Id: oc_membership.h,v 1.3.4.2 2004/04/29 03:12:24 alan Exp $ */
+/* $Id: oc_membership.h,v 1.10 2005/06/12 07:34:42 alan Exp $ */
 #ifndef OCF_OC_MEMBERSHIP_H
 #	define OCF_OC_MEMBERSHIP_H
 /*
@@ -47,20 +47,25 @@
 
 #include <stddef.h>
 #include <ocf/oc_event.h>
+#include <clplumbing/cl_uuid.h>
+
+/*
+ * This enumeration is used both to indicated the type of an event
+ * received, and to request the types of events one wants delivered.
+ * (see oc_member_request_events() and oc_member_etype() for more
+ * details on how this is used).
+ */
+
+enum oc_member_eventtype_e {
+	OC_NOT_MEMBERSHIP,	/* Not a (valid) membership event */
+	OC_FULL_MEMBERSHIP,	/* full membership update */
+	OC_INCR_MEMBERSHIP	/* incremental membership update */
+};
+typedef enum oc_member_eventtype_e		oc_member_eventtype_t;
 
-#ifdef HAVE_UUID_UUID_H
-#	include <uuid/uuid.h>
-#else
-#ifdef HAVE_UUID_H
-#	include <uuid.h>
-#else
-	typedef unsigned char uuid_t[16];
-#endif
-#endif
 						/* controversial? */
 typedef void *					oc_cluster_handle_t;
-typedef uuid_t					oc_node_id_t;
-typedef enum oc_member_eventtype_e		oc_member_eventtype_t;
+typedef cl_uuid_t				oc_node_id_t;
 typedef struct oc_member_uniqueid_s		oc_member_uniqueid_t;
 
 /*
@@ -106,19 +111,6 @@ struct oc_member_uniqueid_s {
 	unsigned	m_instance;
 	oc_mbr_uniqueid	uniqueid;
 };
-/*
- * This enumeration is used both to indicated the type of an event
- * received, and to request the types of events one wants delivered.
- * (see oc_member_request_events() and oc_member_etype() for more
- * details on how this is used).
- */
-
-enum oc_member_eventtype_e {
-	OC_NOT_MEMBERSHIP,	/* Not a (valid) membership event */
-	OC_FULL_MEMBERSHIP,	/* full membership update */
-	OC_INCR_MEMBERSHIP,	/* incremental membership update */
-};
-
 
 #ifdef __cplusplus
 extern "C" {
@@ -135,7 +127,7 @@ extern "C" {
  *	equality comparisons, and secondarily for use in
  *	sorting them into a canonical order.
  */
-int	oc_cmp_node_id(oc_node_id_t l, oc_node_id_t r);
+int	oc_cmp_node_id(const oc_node_id_t* l, const oc_node_id_t* r);
 
 
 /* Return our local node id */
