@@ -1,4 +1,4 @@
-/* $Id: expect.h,v 1.1.4.1 2004/04/20 07:57:54 alan Exp $ */
+/* $Id: expect.h,v 1.3 2005/01/08 06:01:16 alan Exp $ */
 /*
  * Expect simple tokens.  Simple expect infrastructure for STONITH API
  *
@@ -41,7 +41,8 @@ int ExpectToken(int fd
 ,	int to_secs		/* Timeout value in seconds */
 ,	char * buf		/* If non-NULL, then all the text
 				 * matched/skipped over by this match */
-,	int maxline);		/* Size of 'buf' area in bytes */
+,	int maxline,
+,	int debug);		/* debug level */
 
 
 /*
