@@ -1,5 +1,7 @@
-/* $Id: GSource.c,v 1.52 2005/08/12 19:56:48 gshi Exp $ */
+/* $Id: GSource.c,v 1.84 2006/07/17 16:56:47 davidlee Exp $ */
 /*
+ * Copyright (c) 2002 Alan Robertson <alanr@unix.sh>
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -21,99 +23,109 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/resource.h>
+#include <unistd.h>
 #include <errno.h>
 
 #include <clplumbing/cl_log.h>
 #include <clplumbing/cl_malloc.h>
 #include <clplumbing/cl_signal.h>
-#include <clplumbing/GSource.h>
+#include <clplumbing/GSource_internal.h>
 #include <clplumbing/proctrack.h>
+#include <clplumbing/Gmain_timeout.h>
 #include <clplumbing/timers.h>
 
-#define	MAG_GFDSOURCE	0xfeed0001U
-#define	MAG_GCHSOURCE	0xfeed0002U
-#define	MAG_GWCSOURCE	0xfeed0003U
-#define	MAG_GSIGSOURCE	0xfeed0004U
-#define	MAG_GTRIGSOURCE	0xfeed0005U
+#define		DEFAULT_MAXDISPATCH	0
+#define		DEFAULT_MAXDELAY	0
+#define		OTHER_MAXDELAY		100
 
-#define	IS_FDSOURCE(p)	((p)->magno == MAG_GFDSOURCE)
-#define	IS_CHSOURCE(p)	((p)->magno == MAG_GCHSOURCE)
-#define	IS_WCSOURCE(p)	((p)->magno == MAG_GWCSOURCE)
-#define	IS_SIGSOURCE(p)	((p)->magno == MAG_GSIGSOURCE)
-#define	IS_TRIGSOURCE(p) ((p)->magno == MAG_GTRIGSOURCE)
+/*
+ * On architectures with alignment constraints, our casting between
+ * "(GSource*)" and "(GFDSource_s*)" etc. causes trouble, because of
+ * the massive alignment requirements of "longclock_t".
+ *
+ * Use the following to store and fetch.
+ */
+static
+void
+lc_store(char *destptr, longclock_t value) {
+	longclock_t _ltt;
+	_ltt = value;
+	memcpy((destptr), &_ltt, sizeof(longclock_t));
+}
 
-#ifndef _NSIG
-# define _NSIG 2*NSIG
-#endif
-
-struct GFDSource_s {
-	GSource source;
-	unsigned	magno;	/* MAG_GFDSOURCE */
-	void*		udata;
-	gboolean	(*dispatch)(int fd, gpointer user_data);
-	GPollFD		gpfd;
-	GDestroyNotify	dnotify;
-	guint		gsourceid;
-};
-
-
-typedef gboolean 	(*GCHdispatch)(IPC_Channel* ch, gpointer user_data);
-
-struct GCHSource_s {
-	GSource source;
-	unsigned	magno;	/* MAG_GCHSOURCE */
-	void*		udata;
-	IPC_Channel*	ch;
-	gboolean 	(*dispatch)(IPC_Channel* ch, gpointer user_data);
-	GDestroyNotify	dnotify;
-	gboolean	fd_fdx;
-	GPollFD		infd;
-	GPollFD		outfd;
-	guint		gsourceid;
-	gboolean	dontread;	/* TRUE when we don't want to read
-					 * more input for a while - we're
-					 * flow controlling the writer off
-					 */
-};
-
-struct GWCSource_s {
-	GSource source;
-	unsigned		magno;	/* MAG_GWCSOURCE */
-	void*			udata;
-	GPollFD			gpfd;
-	GDestroyNotify		dnotify;
-	IPC_WaitConnection*	wch;
-	IPC_Auth*		auth_info;
-	gboolean (*dispatch)(IPC_Channel* accept_ch, gpointer udata);
-	guint			gsourceid;
-};
-
-struct GSIGSource_s {
-	GSource source;
-	unsigned	magno;	/* MAG_GCHSOURCE */
-	void*		udata;
-	int		signal;
-	gboolean	signal_triggered;
-	gboolean 	(*dispatch)(int signal, gpointer user_data);
-	GDestroyNotify	dnotify;
-	guint		gsourceid;
-};
-
-struct GTRIGSource_s {
-	GSource source;
-	unsigned	magno;	/* MAG_GCHSOURCE */
-	void*		udata;
-	gboolean	manual_trigger;
-	gboolean 	(*dispatch)(gpointer user_data);
-	GDestroyNotify	dnotify;
-	guint		gsourceid;
-};
+static
+longclock_t
+lc_fetch(char *ptr) {
+	longclock_t _ltt;
+	memcpy(&_ltt, (ptr), sizeof(longclock_t));
+	return _ltt;
+}
 
 #define	ERR_EVENTS	(G_IO_ERR|G_IO_NVAL)
 #define	INPUT_EVENTS	(G_IO_IN|G_IO_PRI|G_IO_HUP)
 #define	OUTPUT_EVENTS	(G_IO_OUT)
 #define	DEF_EVENTS	(INPUT_EVENTS|ERR_EVENTS)
 
+#define	WARN_DELAY(ms, mx, input)	cl_log(LOG_WARNING		\
+	,	"%s: Dispatch function for %s was delayed"		\
+	" %lu ms (> %lu ms) before being called (GSource: 0x%lx)"	\
+	,	__FUNCTION__,	(input)->description, ms, mx		\
+	,	POINTER_TO_ULONG(input))
+
+#define EXPLAINDELAY(started, detected) cl_log(LOG_INFO			\
+	,	"%s: started at %llu should have started at %llu"	\
+	,	__FUNCTION__, (unsigned long long)started 		\
+	,	(unsigned long long)detected)
+	
+
+#define	WARN_TOOLONG(ms, mx, input)	cl_log(LOG_WARNING		\
+	,	"%s: Dispatch function for %s took too long to execute"	\
+	": %lu ms (> %lu ms) (GSource: 0x%lx)"				\
+	,	__FUNCTION__,	(input)->description, ms, mx		\
+	,	POINTER_TO_ULONG(input))
+
+#define CHECK_DISPATCH_DELAY(i)	{ 					\
+	unsigned long	ms;						\
+	longclock_t dettime;						\
+	dispstart = time_longclock();					\
+	dettime = lc_fetch((i)->detecttime);				\
+	ms = longclockto_ms(sub_longclock(dispstart,dettime));		\
+	if ((i)->maxdispatchdelayms > 0					\
+	&&	ms > (i)->maxdispatchdelayms) {				\
+		WARN_DELAY(ms, (i)->maxdispatchdelayms, (i));		\
+		EXPLAINDELAY(dispstart, dettime);			\
+	}								\
+}
+
+#define CHECK_DISPATCH_TIME(i)	{ 					\
+	unsigned long	ms;						\
+	longclock_t	dispend = time_longclock();			\
+	ms = longclockto_ms(sub_longclock(dispend, dispstart));		\
+	if ((i)->maxdispatchms > 0 && ms > (i)->maxdispatchms) {	\
+		WARN_TOOLONG(ms, (i)->maxdispatchms, (i));		\
+	}								\
+	lc_store(((i)->detecttime), zero_longclock);		\
+}
+
+#define	WARN_TOOMUCH(ms, mx, input)	cl_log(LOG_WARNING		\
+	,	"%s: working on %s took %ld ms (> %ld ms)"		\
+	,	__FUNCTION__,	(input)->description, ms, mx);
+
+#define	SAVESTART	{funstart = time_longclock();}
+
+#define	CHECKEND(input)	{						\
+	longclock_t	funend = time_longclock();			\
+	long		ms;						\
+	ms = longclockto_ms(sub_longclock(funend, funstart));		\
+	if (ms > OTHER_MAXDELAY){					\
+		WARN_TOOMUCH(ms, ((long) OTHER_MAXDELAY), input);	\
+	}								\
+}									\
+
+
+#ifndef _NSIG
+# define _NSIG 2*NSIG
+#endif
 
 static gboolean G_fd_prepare(GSource* source,
 			     gint* timeout);
@@ -168,12 +180,15 @@ G_main_add_fd(int priority, int fd, gboolean can_recurse
 	GFDSource* ret = (GFDSource*)source;
 	
 	ret->magno = MAG_GFDSOURCE;
+	ret->maxdispatchdelayms = DEFAULT_MAXDELAY;
+	ret->maxdispatchms = DEFAULT_MAXDISPATCH;
 	ret->udata = userdata;
 	ret->dispatch = dispatch;
 	ret->gpfd.fd = fd;
 	ret->gpfd.events = DEF_EVENTS;
 	ret->gpfd.revents = 0;
 	ret->dnotify = notify;
+	lc_store((ret->detecttime), zero_longclock);
 	
 	g_source_add_poll(source, &ret->gpfd);
 	
@@ -183,6 +198,7 @@ G_main_add_fd(int priority, int fd, gboolean can_recurse
 	g_source_set_can_recurse(source, can_recurse);	
 	
 	ret->gsourceid = g_source_attach(source, NULL);
+	ret->description = "file descriptor";
 	
 	if (ret->gsourceid == 0) {
 		g_source_remove_poll(source, &ret->gpfd);
@@ -201,13 +217,13 @@ G_main_del_fd(GFDSource* fdp)
 
 
 	if (fdp->gsourceid <= 0) {
-		cl_log(LOG_CRIT, "Bad gsource in G_main_del_fd");
 		return FALSE;
 	}
 	
+	g_source_remove_poll(source, &fdp->gpfd);
 	g_source_remove(fdp->gsourceid);
 	fdp->gsourceid = 0;
-	g_source_destroy(source);
+	g_source_unref(source);
 	
 	return TRUE;
 
@@ -245,7 +261,11 @@ G_fd_check(GSource* source)
 {
 	GFDSource*	fdp =  (GFDSource*)source;
 	g_assert(IS_FDSOURCE(fdp));
-	return fdp->gpfd.revents != 0;
+	if (fdp->gpfd.revents) {
+		lc_store((fdp->detecttime), time_longclock());
+		return TRUE;
+	}
+	return FALSE;
 }
 
 /*
@@ -258,10 +278,13 @@ G_fd_dispatch(GSource* source,
 {
 
 	GFDSource*	fdp =  (GFDSource*)source;
+	longclock_t	dispstart;
 	g_assert(IS_FDSOURCE(fdp));
+	CHECK_DISPATCH_DELAY(fdp);
 	
 
-	/* Is output now unblocked? 
+	/*
+	 * Is output now unblocked? 
 	 *
 	 * If so, turn off OUTPUT_EVENTS to avoid going into
 	 * a tight poll(2) loop.
@@ -274,8 +297,10 @@ G_fd_dispatch(GSource* source,
 		if(!(fdp->dispatch(fdp->gpfd.fd, fdp->udata))){
 			g_source_remove_poll(source,&fdp->gpfd);
 			g_source_unref(source);
+			CHECK_DISPATCH_TIME(fdp);
 			return FALSE;
 		}
+		CHECK_DISPATCH_TIME(fdp);
 	}
 	
 	return TRUE;
@@ -289,31 +314,31 @@ G_fd_destroy(GSource* source)
 {
 	GFDSource*	fdp =  (GFDSource*)source;	
 	g_assert(IS_FDSOURCE(fdp));
+	fdp->gsourceid = 0;
 	if (fdp->dnotify) {
 		fdp->dnotify(fdp->udata);
 	}
-	g_source_unref(source);
 }
 
 
 /************************************************************
  *		Functions for IPC_Channels
  ***********************************************************/
-static gboolean G_CH_prepare(GSource* source,
+gboolean G_CH_prepare_int(GSource* source,
 			     gint* timeout);
-static gboolean G_CH_check(GSource* source);
+gboolean G_CH_check_int(GSource* source);
 
-static gboolean G_CH_dispatch(GSource* source,
+gboolean G_CH_dispatch_int(GSource* source,
 			      GSourceFunc callback,
 			      gpointer user_data);
-static void G_CH_destroy(GSource* source);
+void G_CH_destroy_int(GSource* source);
 
 
 static GSourceFuncs G_CH_SourceFuncs = {
-	G_CH_prepare,
-	G_CH_check,
-	G_CH_dispatch,
-	G_CH_destroy,
+	G_CH_prepare_int,
+	G_CH_check_int,
+	G_CH_dispatch_int,
+	G_CH_destroy_int,
 };
 
 
@@ -329,25 +354,20 @@ set_IPC_Channel_dnotify(GCHSource* chp,
  *	Add an IPC_channel to the gmainloop world...
  */
 GCHSource*
-G_main_add_IPC_Channel(int priority, IPC_Channel* ch
-		       ,	gboolean can_recurse
-		       ,	gboolean (*dispatch)(IPC_Channel* source_data,
-						     gpointer        user_data)
-		       ,	gpointer userdata
-		       ,	GDestroyNotify notify)
+G_main_IPC_Channel_constructor(GSource* source, IPC_Channel* ch
+	       ,	gpointer userdata
+	       ,	GDestroyNotify notify)
 {
 	int		rfd, wfd;
 	
 	GCHSource* chp;
-	
-	GSource * source = g_source_new(&G_CH_SourceFuncs, 
-					sizeof(GCHSource));
-	
 	chp = (GCHSource*)source;
 	
 	chp->magno = MAG_GCHSOURCE;
+	chp->maxdispatchdelayms = DEFAULT_MAXDELAY;
+	chp->maxdispatchms = DEFAULT_MAXDISPATCH;
+	lc_store((chp->detecttime), zero_longclock);
 	chp->ch = ch;
-	chp->dispatch = dispatch;
 	chp->udata=userdata;
 	chp->dnotify = notify;
 	chp->dontread = FALSE;
@@ -365,12 +385,34 @@ G_main_add_IPC_Channel(int priority, IPC_Channel* ch
 		chp->outfd.events  = DEF_EVENTS;
 		g_source_add_poll(source, &chp->outfd);
 	}
+	chp->dispatch = NULL;
+	chp->description = "IPC channel(base)";
+	chp->gsourceid = 0;
+	return chp;
+}
 
-	g_source_set_priority(source, priority);
+GCHSource*
+G_main_add_IPC_Channel(int priority, IPC_Channel* ch
+		       ,	gboolean can_recurse
+		       ,	gboolean (*dispatch)(IPC_Channel* source_data,
+						     gpointer        user_data)
+		       ,	gpointer userdata
+		       ,	GDestroyNotify notify)
+{
+	GCHSource* chp;
 	
+	GSource * source = g_source_new(&G_CH_SourceFuncs, 
+					sizeof(GCHSource));
+	G_main_IPC_Channel_constructor(source,ch,userdata,notify);
+	
+	chp = (GCHSource*)source;
+	chp->dispatch = dispatch;
+	
+	g_source_set_priority(source, priority);
 	g_source_set_can_recurse(source, can_recurse);
 	
 	chp->gsourceid = g_source_attach(source, NULL);
+	chp->description = "IPC channel";
 	
 
 	if (chp->gsourceid == 0) {
@@ -421,13 +463,13 @@ G_main_del_IPC_Channel(GCHSource* chp)
 	GSource* source = (GSource*) chp;
 
 	if (chp->gsourceid <= 0) {
-		cl_log(LOG_CRIT, "Bad gsource in G_main_del_IPC_channel");
 		return FALSE;
 	}
-	
+
 	g_source_remove(chp->gsourceid);
 	chp->gsourceid = 0;
-	g_source_destroy(source);
+	g_source_unref(source);
+	
 	return TRUE;
 }
 
@@ -437,13 +479,16 @@ G_main_del_IPC_Channel(GCHSource* chp)
  *
  *	Note that we don't modify 'timeout' either.
  */
-static gboolean
-G_CH_prepare(GSource* source,
+gboolean
+G_CH_prepare_int(GSource* source,
 	     gint* timeout)
 {
 	GCHSource* chp = (GCHSource*)source;
+	longclock_t	funstart;
+	gboolean	ret;
 	
 	g_assert(IS_CHSOURCE(chp));
+	SAVESTART;
 	
 	
 	if (chp->ch->ops->is_sending_blocked(chp->ch)) {
@@ -464,24 +509,32 @@ G_CH_prepare(GSource* source,
 		 */
 		chp->infd.events &= ~INPUT_EVENTS;
 	}
-	
+
 	if (chp->dontread){
 		return FALSE;
 	}
-	return chp->ch->ops->is_message_pending(chp->ch);
+	ret = chp->ch->ops->is_message_pending(chp->ch);
+	if (ret) {
+		lc_store((chp->detecttime), time_longclock());
+	}
+	CHECKEND(chp);
+	return ret;
 }
 
 /*
  *	Did we notice any I/O events?
  */
 
-static gboolean
-G_CH_check(GSource* source)
+gboolean
+G_CH_check_int(GSource* source)
 {
 
 	GCHSource* chp = (GCHSource*)source;
+	gboolean	ret;
+	longclock_t	funstart;
 
 	g_assert(IS_CHSOURCE(chp));
+	SAVESTART;
 	
 
 	if (chp->dontread){
@@ -490,22 +543,30 @@ G_CH_check(GSource* source)
 		return FALSE;
 	}
 	
-	return (chp->infd.revents != 0
+	ret = (chp->infd.revents != 0
 		||	(!chp->fd_fdx && chp->outfd.revents != 0)
 		||	chp->ch->ops->is_message_pending(chp->ch));
+	if (ret) {
+		lc_store((chp->detecttime), time_longclock());
+	}
+	CHECKEND(chp);
+	return ret;
 }
 
 /*
  *	Some kind of event occurred - notify the user.
  */
-static gboolean
-G_CH_dispatch(GSource * source,
+gboolean
+G_CH_dispatch_int(GSource * source,
 	      GSourceFunc callback,
 	      gpointer user_data)
 {
 	GCHSource* chp = (GCHSource*)source;
+	longclock_t	dispstart;
+	longclock_t	resume_start = zero_longclock;
 
 	g_assert(IS_CHSOURCE(chp));
+	CHECK_DISPATCH_DELAY(chp);
 
 
 	if (chp->dontread){
@@ -524,28 +585,39 @@ G_CH_dispatch(GSource * source,
 	}else if (chp->outfd.revents & OUTPUT_EVENTS) {
 		chp->outfd.events &= ~OUTPUT_EVENTS;
 	}
-#if 0
-	/* If we got a HUP then mark channel as disconnected */
-	if ((apend->infd.revents|chp->outfd.revents) & G_IO_HUP) {
-		/* CHEAT!! */
-		chp->ch->ch_status = IPC_DISCONNECT;
-	}else{
-		chp->ch->ops->resume_io(chp->ch);
+		
+	if (ANYDEBUG) {
+		resume_start = time_longclock();
 	}
-#else
-	chp->ch->ops->resume_io(chp->ch);
-#endif
 
-	if(chp->dispatch) {
+	chp->ch->ops->resume_io(chp->ch);
+
+	if (ANYDEBUG) {
+		longclock_t resume_end = time_longclock();
+		unsigned long	ms;
+		ms = longclockto_ms(sub_longclock(resume_end
+		,	resume_start));
+		if (ms > 10) {
+			cl_log(LOG_WARNING
+			,	"%s: resume_io() for %s took %lu ms"
+			,	__FUNCTION__
+			,	chp->description, ms);
+		}
+	}
+
+
+	if(chp->dispatch && chp->ch->ops->is_message_pending(chp->ch)) {
 		if(!(chp->dispatch(chp->ch, chp->udata))){
 			g_source_remove_poll(source, &chp->infd);
 			if (!chp->fd_fdx) {
 				g_source_remove_poll(source, &chp->outfd);
 			}
+			CHECK_DISPATCH_TIME(chp);
 			g_source_unref(source);
 			return FALSE;
 		}
 	}
+	CHECK_DISPATCH_TIME(chp);
 
 	if (chp->ch->ch_status == IPC_DISCONNECT){
 		return FALSE;
@@ -556,12 +628,12 @@ G_CH_dispatch(GSource * source,
 /*
  *	Free up our data, and notify the user process...
  */
-static void
-G_CH_destroy(GSource* source)
+void
+G_CH_destroy_int(GSource* source)
 {
 	GCHSource* chp = (GCHSource*)source;
 	
-
+	chp->gsourceid = 0;
 	g_assert(IS_CHSOURCE(chp));
 	
 	if (chp->dnotify) {
@@ -569,7 +641,6 @@ G_CH_destroy(GSource* source)
 	}	
 	chp->ch->ops->destroy(chp->ch);
 	
-	g_source_destroy(source);
 }
 
 
@@ -613,6 +684,9 @@ G_main_add_IPC_WaitConnection(int priority
 	wcp = (GWCSource*)source;
 	
 	wcp->magno = MAG_GWCSOURCE;
+	wcp->maxdispatchdelayms = DEFAULT_MAXDELAY;
+	wcp->maxdispatchms = DEFAULT_MAXDISPATCH;
+	lc_store((wcp->detecttime), zero_longclock);
 	wcp->udata = userdata;
 	wcp->gpfd.fd = wch->ops->get_select_fd(wch);
 	wcp->gpfd.events = DEF_EVENTS;
@@ -629,6 +703,7 @@ G_main_add_IPC_WaitConnection(int priority
 	g_source_set_can_recurse(source, can_recurse);
 	
 	wcp->gsourceid = g_source_attach(source, NULL);
+	wcp->description = "IPC wait for connection";
 	
 	if (wcp->gsourceid == 0) {
 		g_source_remove_poll(source, &wcp->gpfd);
@@ -641,21 +716,20 @@ G_main_add_IPC_WaitConnection(int priority
 
 
 /* Delete the given IPC_WaitConnection from the gmainloop world */
-gboolean G_main_del_IPC_WaitConnection(GWCSource* wcp)
+gboolean
+G_main_del_IPC_WaitConnection(GWCSource* wcp)
 {
 
 	GSource* source =  (GSource*) wcp;
 
 	
 	if (wcp->gsourceid <= 0) {
-		cl_log(LOG_CRIT, "%s:Bad gsource",
-		       __FUNCTION__);
 		return FALSE;
 	}
 	
 	g_source_remove(wcp->gsourceid);
 	wcp->gsourceid = 0;
-	g_source_destroy(source);
+	g_source_unref(source);
 	
 	return TRUE;
 }
@@ -687,7 +761,11 @@ G_WC_check(GSource * source)
 	GWCSource* wcp = (GWCSource*)source;
 	g_assert(IS_WCSOURCE(wcp));
 
-	return wcp->gpfd.revents != 0;
+	if (wcp->gpfd.revents != 0) {
+		lc_store((wcp->detecttime), time_longclock());
+		return TRUE;
+	}
+	return FALSE;
 }
 
 /*
@@ -703,8 +781,10 @@ G_WC_dispatch(GSource* source,
 	IPC_Channel*	ch;
 	gboolean	rc = TRUE;
 	int		count = 0;
+	longclock_t	dispstart;
 	
 	g_assert(IS_WCSOURCE(wcp));
+	CHECK_DISPATCH_DELAY(wcp);
 	
         while(1) {
 		ch = wcp->wch->ops->accept_connection(wcp->wch, wcp->auth_info);
@@ -724,6 +804,7 @@ G_WC_dispatch(GSource* source,
 			break;
 		}
 	}
+	CHECK_DISPATCH_TIME(wcp);
 	return rc;
 }
 
@@ -735,13 +816,12 @@ G_WC_destroy(GSource* source)
 {
 	
 	GWCSource* wcp = (GWCSource*)source;
-	
+	wcp->gsourceid = 0;
 	g_assert(IS_WCSOURCE(wcp));
 	wcp->wch->ops->destroy(wcp->wch);
 	if (wcp->dnotify) {
 		wcp->dnotify(wcp->udata);
 	}
-	g_source_destroy(source);
 }
 
 
@@ -789,6 +869,8 @@ G_main_add_SignalHandler(int priority, int signal,
 	sig_src = (GSIGSource*)source;
 	
 	sig_src->magno		= MAG_GSIGSOURCE;
+	sig_src->maxdispatchdelayms = DEFAULT_MAXDELAY;
+	sig_src->maxdispatchms	= DEFAULT_MAXDISPATCH;
 	sig_src->signal		= signal;
 	sig_src->dispatch	= dispatch;
 	sig_src->udata		= userdata;
@@ -807,6 +889,7 @@ G_main_add_SignalHandler(int priority, int signal,
 	}
 	if(!failed) {
 		sig_src->gsourceid = g_source_attach(source, NULL);
+		sig_src->description = "signal";
 		if (sig_src->gsourceid < 1) {
 			cl_log(LOG_ERR
 			,	"%s: Could not attach source for signal %d (%d)"
@@ -851,18 +934,20 @@ G_main_del_SignalHandler(GSIGSource* sig_src)
 	GSource* source = (GSource*) sig_src;
 
 	if (sig_src->gsourceid <= 0) {
-		cl_log(LOG_CRIT, "Bad gsource in G_main_del_IPC_channel");
 		return FALSE;
 	}
-	g_assert(_NSIG > sig_src->signal);
-
+	if(_NSIG <= sig_src->signal) {
+		g_assert(_NSIG > sig_src->signal);
+		return FALSE;
+	}
+	
 	CL_SIGNAL(sig_src->signal, NULL);
 
 	sig_src->gsourceid = 0;
 	sig_src->signal_triggered = FALSE;
 	g_source_remove(sig_src->gsourceid);
 	G_main_signal_list[sig_src->signal] = NULL;
-	g_source_destroy(source);
+	g_source_unref(source);
 	
 	return TRUE;
 }
@@ -874,9 +959,36 @@ G_SIG_prepare(GSource* source, gint* timeoutms)
 	
 	g_assert(IS_SIGSOURCE(sig_src));
 	
-	/* Don't let a timing window keep us in poll() forever */
-	*timeoutms = 1000;
-	return sig_src->signal_triggered;
+	/* Don't let a timing window keep us in poll() forever
+	 *
+	 * The timing window in question looks like this:
+	 * No signal has occurred up to the point of prepare being called.
+	 * Signal comes in _after_ prepare was called, but _before_ poll.
+	 * signal_detected gets set, but no one checks it before going into poll
+	 * We wait in poll forever...  It's not a pretty sight :-(.
+	 */
+	*timeoutms = 1000;	/* Sigh... */
+
+	if (sig_src->signal_triggered) {
+		static struct tms	dummy_tms_struct;
+		clock_t			now;
+		clock_t			diff;
+
+		/* detecttime is reset in the dispatch function */
+		if (cmp_longclock(lc_fetch(sig_src->detecttime), zero_longclock) != 0) {
+			cl_log(LOG_ERR, "%s: detecttime already set?", __FUNCTION__);
+			return TRUE;
+		}
+		/* Otherwise, this is when it was first detected */
+		now = times(&dummy_tms_struct);
+		diff = now - sig_src->sh_detecttime;	/* How long since signal occurred? */
+		lc_store(
+			sig_src->detecttime,
+			sub_longclock(time_longclock(), (longclock_t)diff)
+		);
+		return TRUE;
+	}
+	return FALSE;
 }
 
 /*
@@ -891,7 +1003,23 @@ G_SIG_check(GSource* source)
 
 	g_assert(IS_SIGSOURCE(sig_src));
 	
-	return sig_src->signal_triggered;
+	if (sig_src->signal_triggered) {
+		static struct tms	dummy_tms_struct;
+		clock_t			now;
+		clock_t			diff;
+		if (cmp_longclock(lc_fetch(sig_src->detecttime), zero_longclock) != 0){
+			return TRUE;
+		}
+		/* Otherwise, this is when it was first detected */
+		now = times(&dummy_tms_struct);
+		diff = now - sig_src->sh_detecttime;
+		lc_store(
+			sig_src->detecttime,
+			sub_longclock(time_longclock(), (longclock_t)diff)
+		);
+		return TRUE;
+	}
+	return FALSE;
 }
 
 /*
@@ -903,17 +1031,22 @@ G_SIG_dispatch(GSource * source,
 	      gpointer user_data)
 {
 	GSIGSource* sig_src = (GSIGSource*)source;
+	longclock_t	dispstart;
 
 	g_assert(IS_SIGSOURCE(sig_src));
+	CHECK_DISPATCH_DELAY(sig_src);
 
+	sig_src->sh_detecttime = 0UL;
 	sig_src->signal_triggered = FALSE;
 
 	if(sig_src->dispatch) {
 		if(!(sig_src->dispatch(sig_src->signal, sig_src->udata))){
 			G_main_del_SignalHandler(sig_src);
+			CHECK_DISPATCH_TIME(sig_src);
 			return FALSE;
 		}
 	}
+	CHECK_DISPATCH_TIME(sig_src);
 	
 	return TRUE;
 }
@@ -927,11 +1060,11 @@ G_SIG_destroy(GSource* source)
 	GSIGSource* sig_src = (GSIGSource*)source;
 	
 	g_assert(IS_SIGSOURCE(sig_src));
-	
+	sig_src->gsourceid = 0;
+
 	if (sig_src->dnotify) {
 		sig_src->dnotify(sig_src->udata);
 	}	
-	g_source_destroy(source);
 }
 
 /* Find and set the correct mainloop input */
@@ -939,20 +1072,31 @@ G_SIG_destroy(GSource* source)
 static void
 G_main_signal_handler(int nsig)
 {
+	static struct tms	dummy_tms_struct;
 	GSIGSource* sig_src = NULL;
 
-	g_assert(G_main_signal_list != NULL);
-	g_assert(_NSIG > nsig);
+	if(G_main_signal_list == NULL) {
+		g_assert(G_main_signal_list != NULL);
+		return;
+	}
+	if(_NSIG <= nsig) {
+		g_assert(_NSIG > nsig);
+		return;
+	}
 	
 	sig_src = G_main_signal_list[nsig];
 
-/* 	g_assert(sig_src != NULL); */
 	if(sig_src == NULL) {
-		cl_log(LOG_CRIT, "No handler for signal -%d", nsig);
+		/* cl_log(LOG_CRIT, "No handler for signal -%d", nsig); */
 		return;
 	}
 	
 	g_assert(IS_SIGSOURCE(sig_src));
+	/* Time from first occurance of signal */
+	if (!sig_src->signal_triggered) {
+		/* Avoid calling longclock_time() on a signal */
+		sig_src->sh_detecttime=times(&dummy_tms_struct);
+	}
 	sig_src->signal_triggered = TRUE;
 }
 
@@ -1032,8 +1176,12 @@ child_death_dispatch(int sig, gpointer notused)
 void
 set_sigchld_proctrack(int priority)
 {
-	G_main_add_SignalHandler(priority, SIGCHLD
+	GSIGSource* src = G_main_add_SignalHandler(priority, SIGCHLD
 	,	child_death_dispatch, NULL, NULL);
+
+	G_main_setmaxdispatchdelay((GSource*) src, 100);
+	G_main_setmaxdispatchtime((GSource*) src, 10);
+	G_main_setdescription((GSource*)src, "SIGCHLD");
 	return;
 }
 
@@ -1078,9 +1226,12 @@ G_main_add_TriggerHandler(int priority,
 	trig_src = (GTRIGSource*)source;
 	
 	trig_src->magno		= MAG_GTRIGSOURCE;
+	trig_src->maxdispatchdelayms = DEFAULT_MAXDELAY;
+	trig_src->maxdispatchms	= DEFAULT_MAXDISPATCH;
 	trig_src->dispatch	= dispatch;
 	trig_src->udata		= userdata;
 	trig_src->dnotify	= notify;
+	lc_store((trig_src->detecttime), zero_longclock);
 
 	trig_src->manual_trigger = FALSE;
 
@@ -1089,6 +1240,7 @@ G_main_add_TriggerHandler(int priority,
 
 	if(!failed) {
 		trig_src->gsourceid = g_source_attach(source, NULL);
+		trig_src->description = "trigger";
 		if (trig_src->gsourceid < 1) {
 			cl_log(LOG_ERR, "G_main_add_TriggerHandler: Could not attach new source (%d)",
 			       trig_src->gsourceid);
@@ -1117,6 +1269,7 @@ G_main_set_trigger(GTRIGSource* source)
 	g_assert(IS_TRIGSOURCE(trig_src));
 	
 	trig_src->manual_trigger = TRUE;
+	lc_store((trig_src->detecttime), time_longclock());
 }
 
 
@@ -1129,13 +1282,12 @@ G_main_del_TriggerHandler(GTRIGSource* trig_src)
 	GSource* source = (GSource*) trig_src;
 
 	if (trig_src->gsourceid <= 0) {
-		cl_log(LOG_CRIT, "Bad gsource in G_main_del_TriggerHandler");
 		return FALSE;
 	}
 	trig_src->gsourceid = 0;
 	trig_src->manual_trigger = FALSE;
 	g_source_remove(trig_src->gsourceid);
-	g_source_destroy(source);
+	g_source_unref(source);
 	
 	return TRUE;
 }
@@ -1147,6 +1299,11 @@ G_TRIG_prepare(GSource* source, gint* timeout)
 	
 	g_assert(IS_TRIGSOURCE(trig_src));
 	
+
+	if (trig_src->manual_trigger
+	&&	cmp_longclock(lc_fetch(trig_src->detecttime), zero_longclock) == 0) {
+		lc_store((trig_src->detecttime), time_longclock());
+	}
 	return trig_src->manual_trigger;
 }
 
@@ -1161,7 +1318,10 @@ G_TRIG_check(GSource* source)
 	GTRIGSource* trig_src = (GTRIGSource*)source;
 
 	g_assert(IS_TRIGSOURCE(trig_src));
-	
+	if (trig_src->manual_trigger
+	&&	cmp_longclock(lc_fetch(trig_src->detecttime), zero_longclock) == 0) {
+		lc_store((trig_src->detecttime), time_longclock());
+	}
 	return trig_src->manual_trigger;
 }
 
@@ -1174,17 +1334,22 @@ G_TRIG_dispatch(GSource * source,
 	      gpointer user_data)
 {
 	GTRIGSource* trig_src = (GTRIGSource*)source;
+	longclock_t	dispstart;
 
 	g_assert(IS_TRIGSOURCE(trig_src));
+	CHECK_DISPATCH_DELAY(trig_src);
 
 	trig_src->manual_trigger = FALSE;
 
 	if(trig_src->dispatch) {
 		if(!(trig_src->dispatch(trig_src->udata))){
 			G_main_del_TriggerHandler(trig_src);
+			CHECK_DISPATCH_TIME(trig_src);
 			return FALSE;
 		}
+		CHECK_DISPATCH_TIME(trig_src);
 	}
+	lc_store((trig_src->detecttime), zero_longclock);
 	
 	return TRUE;
 }
@@ -1198,10 +1363,426 @@ G_TRIG_destroy(GSource* source)
 	GTRIGSource* trig_src = (GTRIGSource*)source;
 	
 	g_assert(IS_TRIGSOURCE(trig_src));
-	
+	trig_src->gsourceid = 0;
+
 	if (trig_src->dnotify) {
 		trig_src->dnotify(trig_src->udata);
 	}	
-	g_source_destroy(source);
+}
+/*
+ * Glib mainloop timeout handling code.
+ *
+ * These functions work correctly even if someone resets the 
+ * time-of-day clock.  The g_main_timeout_add() function does not have
+ * this property, since it relies on gettimeofday().
+ *
+ * Our functions have the same semantics - except they always work ;-)
+ *
+ * This is because we use longclock_t for our time values.
+ *
+ */
+
+
+static gboolean
+Gmain_timeout_prepare(GSource* src,  gint* timeout);
+
+static gboolean
+Gmain_timeout_check(GSource* src);
+
+static gboolean
+Gmain_timeout_dispatch(GSource* src, GSourceFunc func, gpointer user_data);
+
+static GSourceFuncs Gmain_timeout_funcs = {
+	prepare: Gmain_timeout_prepare,
+	check: Gmain_timeout_check,
+	dispatch: Gmain_timeout_dispatch,
+};
+
+
+struct GTimeoutAppend {
+	COMMON_STRUCTSTART;
+	longclock_t	nexttime;
+	guint		interval;
+};
+
+#define        GTIMEOUT(GS)    ((struct GTimeoutAppend*)((void*)(GS)))
+
+guint
+Gmain_timeout_add(guint interval
+,	GSourceFunc	function
+,	gpointer	data)
+{
+	return Gmain_timeout_add_full(G_PRIORITY_DEFAULT
+	,	interval, function, data, NULL);
 }
 
+guint
+Gmain_timeout_add_full(gint priority
+,	guint interval
+,	GSourceFunc	function
+,	gpointer	data
+,	GDestroyNotify	notify)
+{
+	
+	struct GTimeoutAppend* append;
+	
+	GSource* source = g_source_new( &Gmain_timeout_funcs, 
+					sizeof(struct GTimeoutAppend));
+	
+	append = GTIMEOUT(source);
+	append->magno = MAG_GTIMEOUTSRC;
+	append->maxdispatchms = DEFAULT_MAXDISPATCH;
+	append->maxdispatchdelayms = DEFAULT_MAXDELAY;
+	append->description = "(timeout)";
+	lc_store((append->detecttime), zero_longclock);
+	append->udata = NULL;
+	
+	append->nexttime = add_longclock(time_longclock()
+	,	msto_longclock(interval));
+  	append->interval = interval; 
+	
+	g_source_set_priority(source, priority);
+	
+	g_source_set_can_recurse(source, FALSE);
+	
+	g_source_set_callback(source, function, data, notify); 
+
+	append->gsourceid = g_source_attach(source, NULL);
+	return append->gsourceid;
+
+}
+
+void
+Gmain_timeout_remove(guint tag)
+{
+	GSource* source = g_main_context_find_source_by_id(NULL,tag);
+	struct GTimeoutAppend* append = GTIMEOUT(source);
+	
+	g_source_remove(tag);
+	
+	if (source == NULL){
+		cl_log(LOG_ERR, "Attempt to remove timeout (%u)"
+		" with NULL source",	tag);
+	}else{
+		g_assert(IS_TIMEOUTSRC(append));
+		g_source_unref(source);
+	}
+	
+	return;
+}
+
+/* g_main_loop-style prepare function */
+static gboolean
+Gmain_timeout_prepare(GSource* src,  gint* timeout)
+{
+	
+	struct GTimeoutAppend* append = GTIMEOUT(src);
+	longclock_t	lnow = time_longclock();
+	longclock_t	remain;
+	
+	g_assert(IS_TIMEOUTSRC(append));
+	if (cmp_longclock(lnow, append->nexttime) >= 0) {
+		*timeout = 0L;
+		return TRUE;
+	}
+	/* This is safe - we will always have a positive result */
+	remain = sub_longclock(append->nexttime, lnow);
+	/* This is also safe - we started out in 'ms' */
+	*timeout = longclockto_ms(remain);
+	return ((*timeout) == 0);
+}
+
+/* g_main_loop-style check function */
+static gboolean
+Gmain_timeout_check    (GSource* src)
+{
+	struct GTimeoutAppend* append = GTIMEOUT(src);
+	longclock_t	lnow = time_longclock();
+	
+	g_assert(IS_TIMEOUTSRC(append));
+	if (cmp_longclock(lnow, append->nexttime) >= 0) {
+		return TRUE;
+	}
+	return FALSE;
+}
+
+/* g_main_loop-style dispatch function */
+static gboolean
+Gmain_timeout_dispatch(GSource* src, GSourceFunc func, gpointer user_data)
+{
+	struct GTimeoutAppend* append = GTIMEOUT(src);
+	longclock_t	dispstart;
+	gboolean	ret;
+
+	g_assert(IS_TIMEOUTSRC(append));
+	lc_store(append->detecttime, append->nexttime);
+	CHECK_DISPATCH_DELAY(append);
+	
+
+	/* Schedule our next dispatch */
+	append->nexttime = add_longclock(time_longclock()
+	,	msto_longclock(append->interval));
+
+	/* Then call the user function */
+	ret = func(user_data);
+
+	CHECK_DISPATCH_TIME(append);
+	return ret;
+}
+void
+G_main_setmaxdispatchdelay(GSource* s, unsigned long delayms)
+{
+	GFDSource*	fdp =  (GFDSource*)s;
+	if (!IS_ONEOFOURS(fdp)) {
+		cl_log(LOG_ERR
+		,	"Attempt to set max dispatch delay on wrong object");
+		return;
+	}
+	fdp->maxdispatchdelayms = delayms;
+}
+void
+G_main_setmaxdispatchtime(GSource* s, unsigned long dispatchms)
+{
+	GFDSource*	fdp =  (GFDSource*)s;
+	if (!IS_ONEOFOURS(fdp)) {
+		cl_log(LOG_ERR
+		,	"Attempt to set max dispatch time on wrong object");
+		return;
+	}
+	fdp->maxdispatchms = dispatchms;
+}
+void
+G_main_setdescription(GSource* s, const char * description)
+{
+	GFDSource*	fdp =  (GFDSource*)s;
+	if (!IS_ONEOFOURS(fdp)) {
+		cl_log(LOG_ERR
+		,	"Attempt to set max dispatch time on wrong object");
+		return;
+	}
+	fdp->description = description;
+}
+void
+G_main_setmaxdispatchdelay_id(guint id, unsigned long delayms)
+{
+	GSource* source = g_main_context_find_source_by_id(NULL,id);
+	
+	if (source) {
+		G_main_setmaxdispatchdelay(source, delayms);
+	}
+}
+void
+G_main_setmaxdispatchtime_id(guint id, unsigned long dispatchms)
+{
+	GSource* source = g_main_context_find_source_by_id(NULL,id);
+	
+	if (source) {
+		G_main_setmaxdispatchtime(source, dispatchms);
+	}
+}
+void
+G_main_setdescription_id(guint id, const char * description)
+{
+	GSource* source = g_main_context_find_source_by_id(NULL,id);
+	
+	if (source) {
+		G_main_setdescription(source, description);
+	}
+}
+void
+G_main_setall_id(guint id, const char * description, unsigned long delay
+,	unsigned long elapsed)
+{
+	G_main_setdescription_id(id, description);
+	G_main_setmaxdispatchdelay_id(id, delay);
+	G_main_setmaxdispatchtime_id(id, delay);
+}
+
+static void		TempProcessRegistered(ProcTrack* p);
+static void		TempProcessDied(ProcTrack* p, int status, int signo
+,			int exitcode, int waslogged);
+static const char*	TempProcessName(ProcTrack* p);
+
+/***********************************************************************
+ * Track our temporary child processes...
+ *
+ * We run no more than one of each type at once.
+ * If we need to run some and one is still running we run another one
+ * when this one exits.
+ *
+ * Requests to run a child process don't add up.  So, 3 requests to run
+ * a child while one is running only cause it to be run once more, not
+ * three times.
+ * 
+ * The only guarantee is that a new child process will run after a request
+ * was made.
+ *
+ * To create the possibility of running a particular type of child process
+ * call G_main_add_tempproc_trigger().
+ *
+ * To cause it to be run, call G_main_set_trigger().
+ *
+ ***********************************************************************/
+
+static ProcTrack_ops		TempProcessTrackOps = {
+	TempProcessDied,
+	TempProcessRegistered,
+	TempProcessName
+};
+
+/*
+ *	Information for tracking our generic temporary child processes.
+ */
+struct tempproc_track {
+	const char *	procname;	/* name of the process*/
+	GTRIGSource*	trigger;	/* Trigger for this event */
+	int		(*fun)(gpointer userdata); /* Function to call
+					 * in child process */
+	void		(*prefork)(gpointer userdata);/* Call before fork */
+	void		(*postfork)(gpointer userdata);/* Call after fork */
+	void		(*complete)(gpointer userdata, int status, int signo, int exitcode);/* Call after complete */
+	gpointer	userdata;	/* Info to pass 'fun' */
+	gboolean	isrunning;	/* TRUE if child is running */
+	gboolean	runagain;	/* TRUE if we need to run
+						 * again after child process
+						 * finishes.
+						 */
+};
+static void
+TempProcessRegistered(ProcTrack* p)
+{
+	return;	 /* Don't need to do much here... */
+}
+
+static void
+TempProcessDied(ProcTrack* p, int status, int signo, int exitcode
+,	int waslogged)
+{
+	struct tempproc_track *	pt = p->privatedata;
+ 
+	if (pt->complete) {
+		pt->complete(pt->userdata, status, signo, exitcode);
+	}
+
+	pt->isrunning=FALSE;
+	if (pt->runagain) {
+		pt->runagain=FALSE;
+
+ 		/*  Do it again, Sam! */
+		G_main_set_trigger(pt->trigger);
+
+		/* Note that we set the trigger for this, we don't
+		 * fork or call the function now.
+		 *
+		 * This allows the mainloop scheduler to decide
+		 * when the fork should happen according to the priority
+		 * of this trigger event - NOT according to the priority
+		 * of general SIGCHLD handling.
+		 */
+	}
+	p->privatedata = NULL;	/* Don't free until trigger is destroyed */
+	return;
+}
+
+static const char *
+TempProcessName(ProcTrack* p)
+{
+	struct tempproc_track *	pt = p->privatedata;
+	return pt->procname;
+}
+/*
+ *	 Make sure only one copy is running at a time...
+ */
+static gboolean
+TempProcessTrigger(gpointer ginfo)
+{
+	struct tempproc_track*	info = ginfo;
+	int			pid;
+
+ 	/* Make sure only one copy is running at a time. */
+	/* This avoids concurrency problems. */
+	if (info->isrunning) {
+		info->runagain = TRUE;
+		return TRUE;
+	}
+	info->isrunning = TRUE;
+
+	if (info->prefork) {
+		info->prefork(info->userdata);
+	}
+	switch ((pid=fork())) {
+		int		rc;
+		case -1:	cl_perror("%s: Can't fork temporary child"
+				" process [%s]!",	__FUNCTION__
+				,	info->procname);
+				info->isrunning = FALSE;
+				break;
+
+		case 0:		/* Child */
+				if ((rc=info->fun(info->userdata)) == HA_OK) {
+					exit(0);
+				}
+				cl_log(LOG_WARNING
+				,	"%s: %s returns %d", __FUNCTION__
+				,	info->procname, rc);
+				exit(1);
+				break;
+		default:
+				/* Fall out */;
+
+	}
+	if (pid > 0) {
+		NewTrackedProc(pid,0,PT_LOGNORMAL,ginfo,&TempProcessTrackOps);
+	}
+	if (info->postfork) {
+		info->postfork(info->userdata);
+	}
+	return TRUE;
+}
+
+static void
+tempproc_destroy_notify(gpointer userdata)
+{
+	if (userdata != NULL) {
+		cl_free(userdata);
+		userdata = NULL;
+	}
+}
+
+GTRIGSource*
+G_main_add_tempproc_trigger(int priority
+,	int		(*triggerfun) (gpointer p)
+,	const char *	procname
+,	gpointer	userdata
+,	void		(*prefork)(gpointer p)
+,	void		(*postfork)(gpointer p)
+,	void		(*complete)(gpointer userdata, int status, int signo, int exitcode))
+{
+
+	struct tempproc_track* 	p;
+	GTRIGSource*		ret;
+
+	p = (struct tempproc_track *) cl_malloc(sizeof(struct tempproc_track));
+	if (p == NULL) {
+		return NULL;
+	}
+
+	memset(p, 0, sizeof(*p));
+	p->procname = procname;
+	p->fun = triggerfun;
+	p->userdata = userdata;
+	p->prefork = prefork;
+	p->postfork = postfork;
+	p->complete = complete;
+
+	ret = G_main_add_TriggerHandler(priority
+	,	TempProcessTrigger, p,	tempproc_destroy_notify);
+
+	if (ret == NULL) {
+		cl_free(p);
+		p = NULL;
+	}else{
+		p->trigger = ret;
+	}
+	return ret;
+}
