@@ -1,3 +1,4 @@
+/* $Id: base64.c,v 1.13 2005/07/29 05:31:27 panjiam Exp $ */
 /*
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,7 +15,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <heartbeat.h>
 #include <syslog.h>
 #include <string.h>
