@@ -1,4 +1,5 @@
-#include <lha_internal.h>
+/* $Id: cl_poll.c,v 1.22 2005/03/16 17:11:15 lars Exp $ */
+#include <portability.h>
 #include <stdlib.h>
 #include <unistd.h>
 /*
