@@ -1,3 +1,4 @@
+/* $Id: cib_version.c,v 1.5 2006/03/29 06:14:57 andrew Exp $ */
 
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
@@ -17,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 
@@ -36,14 +37,13 @@
 #include <crm/common/xml.h>
 #include <crm/cib.h>
 
+#include <crm/dmalloc_wrapper.h>
 
 
 
 const char * feature_sets[] = {
 	"1.1",
 	"1.2",
-	"1.3",
-	"2.0",
 };
 
 
@@ -55,43 +55,15 @@ typedef struct tag_set_s
 
 
 const char *feature_tags_12[] = { "master_slave", };
-const char *feature_tags_20[] = {
-    "cluster_property_set",
-    "meta_attributes",
-    "transient_attributes",
-};
-
 tag_set_t feature_tags[] = {
 	{ 0, NULL },
 	{ 1, feature_tags_12 },
-	{ 0, NULL },
-	{ 3, feature_tags_20 },
 };
 
 const char *feature_attrs_12[] = { "master_node_max", };
-const char *feature_attrs_20[] = {
-    "start_delay",
-    "disabled",
-    "on_fail",
-    "prereq",
-    "collocated",
-    "globally_unique",
-    "from_role",
-    "to_role",
-    "node_attribute",
-    "score_attribute",
-    "hours",
-    "transition_key",
-    "op_digest",
-    "op_restart_digest",
-    "op_force_restart",
-};
-
 tag_set_t feature_attrs[] = {
 	{ 0, NULL },
 	{ 1, feature_attrs_12 },
-	{ 0, NULL },
-	{ 15, feature_attrs_20 },
 };
 
 static int
@@ -107,6 +79,7 @@ internal_update_feature_set(crm_data_t *xml_obj, int current)
 		  return num_sets-1);
 
 	for(;lpc < num_sets; lpc++) {
+			
 		const char *tag = crm_element_name(xml_obj);
 		crm_debug_3("Checking set %d with %d tags", lpc,
 			  feature_tags[lpc].length);
@@ -132,8 +105,8 @@ internal_update_feature_set(crm_data_t *xml_obj, int current)
 			crm_debug_4("Checking for %s", name);
 			value = crm_element_value(xml_obj, name);
 			if(value != NULL) {
-				crm_info("Found feature '%s' from set %s",
-					 name, feature_sets[lpc]);
+				crm_err("Found feature %s from set %s",
+					name, feature_sets[lpc]);
 				current = lpc;
 				break;
 			}
