@@ -1,3 +1,4 @@
+/* $Id: group.c,v 1.5 2006/06/25 14:56:13 lars Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,16 +17,38 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <crm/pengine/status.h>
-#include <unpack.h>
 #include <utils.h>
 #include <crm/msg_xml.h>
 #include <clplumbing/cl_misc.h>
+	
 
-#define VARIANT_GROUP 1
-#include <lib/crm/pengine/variant.h>
+typedef struct group_variant_data_s
+{
+		int num_children;
+		GListPtr child_list; /* resource_t* */
+		resource_t *self;
+		resource_t *first_child;
+		resource_t *last_child;
+
+		gboolean colocated;
+		gboolean ordered;
+		
+		gboolean child_starting;
+		gboolean child_stopping;
+		
+} group_variant_data_t;
+
+
+#define get_group_variant_data(data, rsc)				\
+	CRM_ASSERT(rsc != NULL);					\
+	CRM_ASSERT(rsc->variant == pe_group);				\
+	CRM_ASSERT(rsc->variant_opaque != NULL);			\
+	data = (group_variant_data_t *)rsc->variant_opaque;		\
+
+void group_assign_color(resource_t *rsc, color_t *group_color);
 
 gboolean group_unpack(resource_t *rsc, pe_working_set_t *data_set)
 {
@@ -44,6 +67,7 @@ gboolean group_unpack(resource_t *rsc, pe_working_set_t *data_set)
 	crm_malloc0(group_data, sizeof(group_variant_data_t));
 	group_data->num_children = 0;
 	group_data->self	 = NULL;
+	group_data->child_list   = NULL;
 	group_data->first_child  = NULL;
 	group_data->last_child   = NULL;
 	rsc->variant_opaque = group_data;
@@ -87,7 +111,8 @@ gboolean group_unpack(resource_t *rsc, pe_working_set_t *data_set)
 		}
 
 		group_data->num_children++;
-		rsc->children = g_list_append(rsc->children, new_rsc);
+		group_data->child_list = g_list_append(
+			group_data->child_list, new_rsc);
 		
 		if(group_data->first_child == NULL) {
 			group_data->first_child = new_rsc;			
@@ -99,10 +124,10 @@ gboolean group_unpack(resource_t *rsc, pe_working_set_t *data_set)
 	if(group_data->num_children == 0) {
 #if 0
 		/* Bug #1287 */
-		crm_config_err("Group %s did not have any children", rsc->id);
+		pe_config_err("Group %s did not have any children", rsc->id);
 		return FALSE;
 #else
-		crm_config_warn("Group %s did not have any children", rsc->id);
+		pe_config_warn("Group %s did not have any children", rsc->id);
 		return TRUE;
 #endif
 	}
@@ -114,13 +139,28 @@ gboolean group_unpack(resource_t *rsc, pe_working_set_t *data_set)
 }
 
 
+resource_t *
+group_find_child(resource_t *rsc, const char *id)
+{
+	group_variant_data_t *group_data = NULL;
+	get_group_variant_data(group_data, rsc);
+	return pe_find_resource(group_data->child_list, id);
+}
+
+GListPtr group_children(resource_t *rsc)
+{
+	group_variant_data_t *group_data = NULL;
+	get_group_variant_data(group_data, rsc);
+	return group_data->child_list;
+}
+
 gboolean group_active(resource_t *rsc, gboolean all)
 {
 	group_variant_data_t *group_data = NULL;
 	get_group_variant_data(group_data, rsc);
 
 	slist_iter(
-		child_rsc, resource_t, rsc->children, lpc,
+		child_rsc, resource_t, group_data->child_list, lpc,
 		gboolean child_active = child_rsc->fns->active(child_rsc, all);
 		if(all == FALSE && child_active) {
 			return TRUE;
@@ -158,7 +198,7 @@ void group_print(
 	}
 	
 	slist_iter(
-		child_rsc, resource_t, rsc->children, lpc,
+		child_rsc, resource_t, group_data->child_list, lpc,
 		
 		if(options & pe_print_html) {
 			status_print("<li>\n");
@@ -184,14 +224,14 @@ void group_free(resource_t *rsc)
 	crm_debug_3("Freeing %s", rsc->id);
 
 	slist_iter(
-		child_rsc, resource_t, rsc->children, lpc,
+		child_rsc, resource_t, group_data->child_list, lpc,
 
 		crm_debug_3("Freeing child %s", child_rsc->id);
 		child_rsc->fns->free(child_rsc);
 		);
 
 	crm_debug_3("Freeing child list");
-	pe_free_shallow_adv(rsc->children, FALSE);
+	pe_free_shallow_adv(group_data->child_list, FALSE);
 
 	if(group_data->self != NULL) {
 		free_xml(group_data->self->xml);
@@ -202,18 +242,21 @@ void group_free(resource_t *rsc)
 }
 
 enum rsc_role_e
-group_resource_state(const resource_t *rsc, gboolean current)
+group_resource_state(resource_t *rsc)
 {
 	enum rsc_role_e group_role = RSC_ROLE_UNKNOWN;
+	group_variant_data_t *group_data = NULL;
+	get_group_variant_data(group_data, rsc);
 
 	slist_iter(
-		child_rsc, resource_t, rsc->children, lpc,
-		enum rsc_role_e role = child_rsc->fns->state(child_rsc, current);
-		if(role > group_role) {
-			group_role = role;
-		}
-	    );
+		child_rsc, resource_t, group_data->child_list, lpc,
 
-	crm_debug_3("%s role: %s", rsc->id, role2text(group_role));
+		if(child_rsc->next_role > group_role) {
+			group_role = rsc->next_role;
+		}
+		if(child_rsc->failed) {
+			rsc->failed = TRUE;
+		}
+		);
 	return group_role;
 }
