@@ -1,3 +1,4 @@
+/* $Id: rules.c,v 1.3 2006/06/21 14:53:48 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
@@ -233,7 +234,7 @@ test_attr_expression(crm_data_t *expr, GHashTable *hash, ha_time_t *now)
 	
 	if(value != NULL && h_val != NULL) {
 		if(type == NULL || (safe_str_eq(type, "string"))) {
-			cmp = strcasecmp(h_val, value);
+			cmp = strcmp(h_val, value);
 			
 		} else if(safe_str_eq(type, "number")) {
 			int h_val_f = crm_parse_int(h_val, NULL);
@@ -339,12 +340,9 @@ phase_of_the_moon(ha_time_t *now)
 		}							\
 		value_low_i = crm_parse_int(value_low, "0");		\
 		value_high_i = crm_parse_int(value_high, "-1");		\
-		if(value_high_i < 0) {					\
-			if(value_low_i != time_field) {			\
-				pass = FALSE;				\
-			}						\
-		} else if(value_low_i > time_field) {			\
+		if(value_low_i > time_field) {				\
 			pass = FALSE;					\
+		} else if(value_high_i < 0) {				\
 		} else if(value_high_i < time_field) {			\
 			pass = FALSE;					\
 		}							\
@@ -367,8 +365,6 @@ cron_range_satisfied(ha_time_t *now, crm_data_t *cron_spec)
 	int value_low_i = 0;
 	int value_high_i = 0;
 
-	CRM_CHECK(now != NULL, return FALSE);
-	
 	cron_check("seconds",	now->seconds);
 	cron_check("minutes",	now->minutes);
 	cron_check("hours",	now->hours);
@@ -531,7 +527,6 @@ populate_hash(crm_data_t *nvpair_list, GHashTable *hash)
 {
 	const char *name = NULL;
 	const char *value = NULL;
-
 	xml_child_iter_filter(
 		nvpair_list, an_attr, XML_CIB_TAG_NVPAIR,
 		
@@ -542,11 +537,11 @@ populate_hash(crm_data_t *nvpair_list, GHashTable *hash)
 			an_attr, XML_NVPAIR_ATTR_VALUE);
 		
 		if(name == NULL || value == NULL) {
-			continue;
+			return;
 
 		} else if(safe_str_eq(value, "#default")) {
-			continue;
-
+			return;
+		
 		} else if(g_hash_table_lookup(hash, name) == NULL) {
 			g_hash_table_insert(
 				hash, crm_strdup(name), crm_strdup(value));
