@@ -1,3 +1,4 @@
+/* $Id: status.c,v 1.7 2006/07/04 07:36:01 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 
@@ -81,10 +82,6 @@ cluster_status(pe_working_set_t *data_set)
 		return FALSE;
 	}
 
-	if(data_set->now == NULL) {
-	    data_set->now = new_ha_date(TRUE);
-	}
-	
 	if(data_set->input != NULL
 	   && crm_element_value(data_set->input, XML_ATTR_DC_UUID) != NULL) {
 		/* this should always be present */
@@ -158,7 +155,6 @@ pe_free_nodes(GListPtr nodes)
 				g_hash_table_destroy(details->attrs);
 			}
 			pe_free_shallow_adv(details->running_rsc, FALSE);
-			pe_free_shallow_adv(details->allocated_rsc, FALSE);
 			crm_free(details);
 		}
 		crm_free(node);
@@ -182,21 +178,21 @@ cleanup_calculations(pe_working_set_t *data_set)
 	crm_free(data_set->dc_uuid);
 	crm_free(data_set->transition_idle_timeout);
 	
-	crm_debug_3("deleting resources");
-	pe_free_resources(data_set->resources); 
-	
 	crm_debug_3("deleting actions");
 	pe_free_actions(data_set->actions);
 
+	crm_debug_3("deleting resources");
+	pe_free_resources(data_set->resources); 
+	
 	crm_debug_3("deleting nodes");
 	pe_free_nodes(data_set->nodes);
 	
 	free_xml(data_set->graph);
 	free_ha_date(data_set->now);
 	free_xml(data_set->input);
-	free_xml(data_set->failed);
 	data_set->stonith_action = NULL;
 
+	CRM_CHECK(data_set->colors == NULL, ;);
 	CRM_CHECK(data_set->ordering_constraints == NULL, ;);
 	CRM_CHECK(data_set->placement_constraints == NULL, ;);
 }
@@ -208,9 +204,8 @@ set_working_set_defaults(pe_working_set_t *data_set)
 	data_set->input = NULL;
 	data_set->now = NULL;
 	data_set->graph = NULL;
-	data_set->failed = create_xml_node(NULL, "failed-ops");
 	
-	data_set->transition_idle_timeout = NULL;
+	data_set->transition_idle_timeout = crm_strdup("60s");
 	data_set->dc_uuid            = NULL;
 	data_set->dc_node            = NULL;
 	data_set->have_quorum        = FALSE;
@@ -229,14 +224,16 @@ set_working_set_defaults(pe_working_set_t *data_set)
 	data_set->resources   = NULL;
 	data_set->ordering_constraints  = NULL;
 	data_set->placement_constraints = NULL;
-	data_set->colocation_constraints = NULL;
 
+	data_set->no_color = NULL;
+	data_set->colors   = NULL;
 	data_set->actions  = NULL;	
 
 	data_set->num_synapse     = 0;
 	data_set->max_valid_nodes = 0;
 	data_set->order_id        = 1;
 	data_set->action_id       = 1;
+	data_set->color_id        = 0;
 
 	data_set->default_resource_stickiness = 0;
 	data_set->default_resource_fail_stickiness = 0;	
@@ -282,35 +279,3 @@ pe_find_resource(GListPtr rsc_list, const char *id)
 	crm_debug_2("No match for %s", id);
 	return NULL;
 }
-
-node_t *
-pe_find_node_id(GListPtr nodes, const char *id)
-{
-	unsigned lpc = 0;
-	node_t *node = NULL;
-  
-	for(lpc = 0; lpc < g_list_length(nodes); lpc++) {
-		node = g_list_nth_data(nodes, lpc);
-		if(safe_str_eq(node->details->id, id)) {
-			return node;
-		}
-	}
-	/* error */
-	return NULL;
-}
-
-node_t *
-pe_find_node(GListPtr nodes, const char *uname)
-{
-	unsigned lpc = 0;
-	node_t *node = NULL;
-  
-	for(lpc = 0; lpc < g_list_length(nodes); lpc++) {
-		node = g_list_nth_data(nodes, lpc);
-		if(safe_str_eq(node->details->uname, uname)) {
-			return node;
-		}
-	}
-	/* error */
-	return NULL;
-}
