@@ -1,3 +1,4 @@
+/* $Id: unpack.c,v 1.13 2006/07/18 06:17:34 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -15,7 +16,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#include <lha_internal.h>
+#include <portability.h>
 #include <crm/crm.h>
 #include <crm/cib.h>
 #include <crm/msg_xml.h>
@@ -34,15 +35,22 @@
 #include <crm/pengine/rules.h>
 #include <unpack.h>
 
+#define get_cluster_pref(pref) value = g_hash_table_lookup(config_hash, pref); \
+	if(value == NULL) {						\
+		pe_config_warn("No value specified for cluster preference: %s", pref); \
+	}
+
+
+
 gboolean
-unpack_config(crm_data_t *config, pe_working_set_t *data_set)
+unpack_config(crm_data_t * config, pe_working_set_t *data_set)
 {
 	const char *name = NULL;
 	const char *value = NULL;
 	GHashTable *config_hash = g_hash_table_new_full(
 		g_str_hash,g_str_equal, g_hash_destroy_str,g_hash_destroy_str);
 
-	data_set->config_hash = config_hash;
+	data_set->config_hash = config_hash;	
 
 	unpack_instance_attributes(
 		config, XML_CIB_TAG_PROPSET, NULL, config_hash,
@@ -59,7 +67,7 @@ unpack_config(crm_data_t *config, pe_working_set_t *data_set)
 			g_hash_table_insert(
 				config_hash,crm_strdup(name),crm_strdup(value));
 		}
-		crm_config_err("Creating <nvpair id=%s name=%s/> directly"
+		pe_config_err("Creating <nvpair id=%s name=%s/> directly"
 			      "beneath <crm_config> has been depreciated since"
 			      " 2.0.1", ID(a_child), name);
 		);
@@ -68,43 +76,63 @@ unpack_config(crm_data_t *config, pe_working_set_t *data_set)
 		config, a_child, XML_CIB_TAG_NVPAIR,
 
 		name = crm_element_value(a_child, XML_NVPAIR_ATTR_NAME);
-		crm_config_err("Creating <nvpair id=%s name=%s/> directly"
+		pe_config_err("Creating <nvpair id=%s name=%s/> directly"
 			      "beneath <crm_config> has been depreciated since"
 			      " 2.0.1 and is now disabled", ID(a_child), name);
 		);
 #endif
-	verify_pe_options(data_set->config_hash);
 	
-	value = pe_pref(data_set->config_hash, "default-action-timeout");
-	data_set->transition_idle_timeout = crm_strdup(value);
-	crm_debug("Default action timeout: %s", data_set->transition_idle_timeout);
+	get_cluster_pref("default_action_timeout");
+	if(value == NULL) {
+		get_cluster_pref("transition_idle_timeout");
+	}
+	if(value != NULL) {
+		long tmp = crm_get_msec(value);
+		if(tmp > 0) {
+			crm_free(data_set->transition_idle_timeout);
+			data_set->transition_idle_timeout = crm_strdup(value);
+		} else {
+			crm_err("Invalid value for default_action_timeout: %s",
+				value);
+		}
+	}
+	
+	crm_debug("default_action_timeout set to: %s", data_set->transition_idle_timeout);
 
-	value = pe_pref(data_set->config_hash, "default-resource-stickiness");
+	get_cluster_pref("default_"XML_RSC_ATTR_STICKINESS);
 	data_set->default_resource_stickiness = char2score(value);
-	crm_debug("Default stickiness: %d",
+	crm_info("Default stickiness: %d",
 		 data_set->default_resource_stickiness);
 
-	value = pe_pref(data_set->config_hash, "default-resource-failure-stickiness");
+	get_cluster_pref("default_"XML_RSC_ATTR_FAIL_STICKINESS);
 	data_set->default_resource_fail_stickiness = char2score(value);
-	crm_debug("Default failure stickiness: %d",
+	crm_info("Default failure stickiness: %d",
 		 data_set->default_resource_fail_stickiness);
 	
-	value = pe_pref(data_set->config_hash, "stonith-enabled");
-	cl_str_to_boolean(value, &data_set->stonith_enabled);
-	crm_debug("STONITH of failed nodes is %s",
+	get_cluster_pref("stonith_enabled");
+	if(value != NULL) {
+		cl_str_to_boolean(value, &data_set->stonith_enabled);
+	}
+	crm_info("STONITH of failed nodes is %s",
 		 data_set->stonith_enabled?"enabled":"disabled");	
 
-	data_set->stonith_action = pe_pref(data_set->config_hash, "stonith-action");
-	crm_debug_2("STONITH will %s nodes", data_set->stonith_action);	
+	get_cluster_pref("stonith_action");
+	if(value == NULL || safe_str_neq(value, "poweroff")) {
+		value = "reboot";
+	}
+	data_set->stonith_action = value;
+	crm_info("STONITH will %s nodes", data_set->stonith_action);	
 	
-	value = pe_pref(data_set->config_hash, "symmetric-cluster");
-	cl_str_to_boolean(value, &data_set->symmetric_cluster);
+	get_cluster_pref("symmetric_cluster");
+	if(value != NULL) {
+		cl_str_to_boolean(value, &data_set->symmetric_cluster);
+	}
 	if(data_set->symmetric_cluster) {
-		crm_debug("Cluster is symmetric"
+		crm_info("Cluster is symmetric"
 			 " - resources can run anywhere by default");
 	}
 
-	value = pe_pref(data_set->config_hash, "no-quorum-policy");
+	get_cluster_pref("no_quorum_policy");
 	if(safe_str_eq(value, "ignore")) {
 		data_set->no_quorum_policy = no_quorum_ignore;
 		
@@ -117,41 +145,44 @@ unpack_config(crm_data_t *config, pe_working_set_t *data_set)
 	
 	switch (data_set->no_quorum_policy) {
 		case no_quorum_freeze:
-			crm_debug("On loss of CCM Quorum: Freeze resources");
+			crm_info("On loss of CCM Quorum: Freeze resources");
 			break;
 		case no_quorum_stop:
-			crm_debug("On loss of CCM Quorum: Stop ALL resources");
+			crm_info("On loss of CCM Quorum: Stop ALL resources");
 			break;
 		case no_quorum_ignore:
 			crm_notice("On loss of CCM Quorum: Ignore");
 			break;
 	}
 
-	value = pe_pref(data_set->config_hash, "stop-orphan-resources");
-	cl_str_to_boolean(value, &data_set->stop_rsc_orphans);
-	crm_debug_2("Orphan resources are %s",
-		    data_set->stop_rsc_orphans?"stopped":"ignored");	
+	get_cluster_pref("stop_orphan_resources");
+	if(value != NULL) {
+		cl_str_to_boolean(value, &data_set->stop_rsc_orphans);
+	}
+	crm_info("Orphan resources are %s",
+		 data_set->stop_rsc_orphans?"stopped":"ignored");	
 	
-	value = pe_pref(data_set->config_hash, "stop-orphan-actions");
-	cl_str_to_boolean(value, &data_set->stop_action_orphans);
-	crm_debug_2("Orphan resource actions are %s",
-		    data_set->stop_action_orphans?"stopped":"ignored");	
+	get_cluster_pref("stop_orphan_actions");
+	if(value != NULL) {
+		cl_str_to_boolean(value, &data_set->stop_action_orphans);
+	}
+	crm_info("Orphan resource actions are %s",
+		 data_set->stop_action_orphans?"stopped":"ignored");	
 
-	value = pe_pref(data_set->config_hash, "remove-after-stop");
-	cl_str_to_boolean(value, &data_set->remove_after_stop);
-	crm_debug_2("Stopped resources are removed from the status section: %s",
-		    data_set->remove_after_stop?"true":"false");	
+	get_cluster_pref("remove_after_stop");
+	if(value != NULL) {
+		cl_str_to_boolean(value, &data_set->remove_after_stop);
+	}
+	crm_info("Stopped resources are removed from the status section: %s",
+		 data_set->remove_after_stop?"true":"false");	
 	
-	value = pe_pref(data_set->config_hash, "is-managed-default");
-	cl_str_to_boolean(value, &data_set->is_managed_default);
-	crm_debug_2("By default resources are %smanaged",
-		    data_set->is_managed_default?"":"not ");
+	get_cluster_pref("is_managed_default");
+	if(value != NULL) {
+		cl_str_to_boolean(value, &data_set->is_managed_default);
+	}
+	crm_info("By default resources are %smanaged",
+		 data_set->is_managed_default?"":"not ");
 
-	value = pe_pref(data_set->config_hash, "start-failure-is-fatal");
-	cl_str_to_boolean(value, &data_set->start_failure_fatal);
-	crm_debug_2("Start failures are %s",
-		    data_set->start_failure_fatal?"always fatal":"handled by failcount");
-	
 	return TRUE;
 }
 
@@ -162,15 +193,11 @@ unpack_nodes(crm_data_t * xml_nodes, pe_working_set_t *data_set)
 	const char *id     = NULL;
 	const char *uname  = NULL;
 	const char *type   = NULL;
-	gboolean unseen_are_unclean = TRUE;
-	const char *blind_faith = pe_pref(
-		data_set->config_hash, "startup-fencing");
-	
-	if(crm_is_true(blind_faith) == FALSE) {
-		unseen_are_unclean = FALSE;
-		crm_warn("Blind faith: not fencing unseen nodes");
-	}
+	const char *blind_faith = g_hash_table_lookup(
+		data_set->config_hash, "startup_fencing");
 
+	crm_debug_2("Begining unpack... %s",
+		    xml_nodes?crm_element_name(xml_nodes):"<none>");
 	xml_child_iter_filter(
 		xml_nodes, xml_obj, XML_CIB_TAG_NODE,
 
@@ -182,18 +209,13 @@ unpack_nodes(crm_data_t * xml_nodes, pe_working_set_t *data_set)
 		crm_debug_3("Processing node %s/%s", uname, id);
 
 		if(id == NULL) {
-			crm_config_err("Must specify id tag in <node>");
+			pe_config_err("Must specify id tag in <node>");
 			continue;
 		}
 		if(type == NULL) {
-			crm_config_err("Must specify type tag in <node>");
+			pe_config_err("Must specify type tag in <node>");
 			continue;
 		}
-		if(pe_find_node(data_set->nodes, uname) != NULL) {
-		    crm_config_warn("Detected multiple node entries with uname=%s"
-				    " - this is rarely intended", uname);
-		}
-
 		crm_malloc0(new_node, sizeof(node_t));
 		if(new_node == NULL) {
 			return FALSE;
@@ -226,7 +248,7 @@ unpack_nodes(crm_data_t * xml_nodes, pe_working_set_t *data_set)
 /* 			new_node->weight = -INFINITY; */
 /* 		} */
 		
-		if(data_set->stonith_enabled == FALSE || unseen_are_unclean == FALSE) {
+		if(data_set->stonith_enabled == FALSE || crm_is_true(blind_faith)) {
 			/* blind faith... */
 			new_node->details->unclean = FALSE; 
 
@@ -256,6 +278,8 @@ unpack_nodes(crm_data_t * xml_nodes, pe_working_set_t *data_set)
 		data_set->nodes = g_list_append(data_set->nodes, new_node);    
 		crm_debug_3("Done with node %s",
 			    crm_element_value(xml_obj, XML_ATTR_UNAME));
+
+		crm_action_debug_3(print_node("Added", new_node, FALSE));
 		);
   
 	return TRUE;
@@ -264,11 +288,13 @@ unpack_nodes(crm_data_t * xml_nodes, pe_working_set_t *data_set)
 gboolean 
 unpack_resources(crm_data_t * xml_resources, pe_working_set_t *data_set)
 {
+	crm_debug_2("Begining unpack... %s",
+		    xml_resources?crm_element_name(xml_resources):"<none>");
 	xml_child_iter(
 		xml_resources, xml_obj, 
 
 		resource_t *new_rsc = NULL;
-		crm_debug_3("Begining unpack... %s",
+		crm_debug_2("Begining unpack... %s",
 			    xml_obj?crm_element_name(xml_obj):"<none>");
 		if(common_unpack(xml_obj, &new_rsc, NULL, data_set)) {
 			data_set->resources = g_list_append(
@@ -277,7 +303,7 @@ unpack_resources(crm_data_t * xml_resources, pe_working_set_t *data_set)
 			print_resource(LOG_DEBUG_3, "Added", new_rsc, FALSE);
 
 		} else {
-			crm_config_err("Failed unpacking %s %s",
+			pe_config_err("Failed unpacking %s %s",
 				      crm_element_name(xml_obj),
 				      crm_element_value(xml_obj, XML_ATTR_ID));
 			if(new_rsc != NULL && new_rsc->fns != NULL) {
@@ -326,7 +352,7 @@ unpack_status(crm_data_t * status, pe_working_set_t *data_set)
 			continue;
 
 		} else if(this_node == NULL) {
-			crm_config_warn("Node %s in status section no longer exists",
+			pe_config_warn("Node %s in status section no longer exists",
 				       uname);
 			continue;
 		}
@@ -425,10 +451,9 @@ determine_online_status_fencing(crm_data_t * node_state, node_t *this_node)
 	   && safe_str_eq(crm_state, ONLINESTATUS)) {
 		online = TRUE;
 		if(safe_str_neq(join_state, CRMD_JOINSTATE_MEMBER)) {
-			crm_info("Node %s is not ready to run resources",
-				 this_node->details->uname);
+			crm_debug("Node is not ready to run resources: %s", join_state);
 			this_node->details->standby = TRUE;
-		}
+		}			
 		
 	} else if(crm_is_true(ccm_state) == FALSE
  		  && safe_str_eq(ha_state, DEADSTATUS)
@@ -472,7 +497,7 @@ determine_online_status(
 		crm_element_value(node_state, XML_CIB_ATTR_EXPSTATE);
 	
 	if(this_node == NULL) {
-		crm_config_err("No node to check");
+		pe_config_err("No node to check");
 		return online;
 	}
 
@@ -518,8 +543,7 @@ determine_online_status(
 
 	} else if(this_node->details->online) {
 		crm_info("Node %s is %s", this_node->details->uname,
-			 this_node->details->shutdown?"shutting down":
-			 this_node->details->standby?"standby":"online");
+			 this_node->details->shutdown?"shutting down":"online");
 
 	} else {
 		crm_debug_2("Node %s is offline", this_node->details->uname);
@@ -530,26 +554,23 @@ determine_online_status(
 	return online;
 }
 
-#define set_char(x) last_rsc_id[lpc] = x; complete = TRUE;
+#define set_char(x) last_rsc_id[len] = x; complete = TRUE;
 
-static char *
+static void
 increment_clone(char *last_rsc_id)
 {
-	int lpc = 0;
-	int len = 0;
-	char *tmp = NULL;
 	gboolean complete = FALSE;
+	int len = 0;
 
-	CRM_CHECK(last_rsc_id != NULL, return NULL);
+	CRM_CHECK(last_rsc_id != NULL, return);
 	if(last_rsc_id != NULL) {
 		len = strlen(last_rsc_id);
 	}
-	
-	lpc = len-1;
-	while(complete == FALSE && lpc > 0) {
-		switch (last_rsc_id[lpc]) {
+	len--;
+	while(complete == FALSE && len > 0) {
+		switch (last_rsc_id[len]) {
 			case 0:
-				lpc--;
+				len--;
 				break;
 			case '0':
 				set_char('1');
@@ -579,26 +600,15 @@ increment_clone(char *last_rsc_id)
 				set_char('9');
 				break;
 			case '9':
-				last_rsc_id[lpc] = '0';
-				lpc--;
-				break;
-			case ':':
-				tmp = last_rsc_id;
-				crm_malloc0(last_rsc_id, len + 2);
-				memcpy(last_rsc_id, tmp, len);
-				last_rsc_id[++lpc] = '1';
 				last_rsc_id[len] = '0';
-				last_rsc_id[len+1] = 0;
-				complete = TRUE;
-				crm_free(tmp);
+				len--;
 				break;
 			default:
 				crm_err("Unexpected char: %c (%d)",
-					last_rsc_id[lpc], lpc);
+					last_rsc_id[len], len);
 				break;
 		}
 	}
-	return last_rsc_id;
 }
 
 static resource_t *
@@ -606,25 +616,22 @@ create_fake_resource(const char *rsc_id, crm_data_t *rsc_entry, pe_working_set_t
 {
 	resource_t *rsc = NULL;
 	crm_data_t *xml_rsc  = create_xml_node(NULL, XML_CIB_TAG_RESOURCE);
+	crm_log_xml_info(rsc_entry, "Orphan resource");
 	copy_in_properties(xml_rsc, rsc_entry);
 	crm_xml_add(xml_rsc, XML_ATTR_ID, rsc_id);
-	crm_log_xml_info(xml_rsc, "Orphan resource");
 	
 	common_unpack(xml_rsc, &rsc, NULL, data_set);
-	set_bit(rsc->flags, pe_rsc_orphan);
+	rsc->orphan = TRUE;
 	
 	data_set->resources = g_list_append(data_set->resources, rsc);
 	return rsc;
 }
 
-extern gboolean create_child_clone(resource_t *rsc, int sub_id, pe_working_set_t *data_set);
-
 static resource_t *
 unpack_find_resource(
 	pe_working_set_t *data_set, node_t *node, const char *rsc_id, crm_data_t *rsc_entry)
 {
 	resource_t *rsc = NULL;
-	resource_t *clone_parent = NULL;
 	gboolean is_duped_clone = FALSE;
 	char *alt_rsc_id = crm_strdup(rsc_id);
 	
@@ -634,23 +641,14 @@ unpack_find_resource(
 		/* no match */
 		if(rsc == NULL) {
 			crm_debug_2("%s not found: %d", alt_rsc_id, is_duped_clone);
-			if(is_duped_clone == FALSE) {
-			    break;
+			if(is_duped_clone) {
+				/* create one */
+				rsc = create_fake_resource(alt_rsc_id, rsc_entry, data_set);
+				crm_info("Making sure orphan %s/%s is stopped on %s",
+					 rsc_id, rsc->id, node->details->uname);
+				resource_location(rsc, NULL, -INFINITY, "__orphan_clone_dont_run__", data_set);
 			}
-			    
-			/* create one */
-#if 1
-			create_child_clone(clone_parent, -1, data_set);
-			crm_debug("Looking again for %s", alt_rsc_id);
-			rsc = pe_find_resource(data_set->resources, alt_rsc_id);
-			CRM_CHECK(rsc != NULL, crm_err("%s stil not found", alt_rsc_id); continue);
-#else
-			rsc = create_fake_resource(alt_rsc_id, rsc_entry, data_set);
-			crm_info("Making sure orphan %s/%s of %s is stopped on %s",
-				 rsc_id, rsc->id, clone_parent->id, node->details->uname);
-			resource_location(rsc, NULL, -INFINITY, "__orphan_clone_dont_run__", data_set);
 			break;
-#endif
 			
 			/* not running anywhere else */
 		} else if(rsc->running_on == NULL) {
@@ -658,7 +656,7 @@ unpack_find_resource(
 			break;
 			
 			/* always unique */
-		} else if(is_set(rsc->flags, pe_rsc_unique)) {
+		} else if(rsc->globally_unique) {
 			crm_debug_3("unique");
 			break;
 			
@@ -666,11 +664,10 @@ unpack_find_resource(
 			 *   find another clone instead
 			 */
 		} else {
-			crm_debug_3("find another one");
-			clone_parent = uber_parent(rsc);
+			crm_debug_2("find another one");
 			rsc = NULL;
 			is_duped_clone = TRUE;
-			alt_rsc_id = increment_clone(alt_rsc_id);
+			increment_clone(alt_rsc_id);
 		}
 	}
 	crm_free(alt_rsc_id);
@@ -695,12 +692,12 @@ process_orphan_resource(crm_data_t *rsc_entry, node_t *node, pe_working_set_t *d
 	const char *rsc_id   = crm_element_value(rsc_entry, XML_ATTR_ID);
 	
 	crm_log_xml_info(rsc_entry, "Orphan resource");
-	crm_config_warn("Nothing known about resource %s running on %s",
+	pe_config_warn("Nothing known about resource %s running on %s",
 		       rsc_id, node->details->uname);
 	rsc = create_fake_resource(rsc_id, rsc_entry, data_set);
 	
 	if(data_set->stop_rsc_orphans == FALSE) {
-	    clear_bit(rsc->flags, pe_rsc_managed);
+		rsc->is_managed = FALSE;
 		
 	} else {
 		crm_info("Making sure orphan %s is stopped", rsc_id);
@@ -716,23 +713,12 @@ process_orphan_resource(crm_data_t *rsc_entry, node_t *node, pe_working_set_t *d
 static void
 process_rsc_state(resource_t *rsc, node_t *node,
 		  enum action_fail_response on_fail,
-		  crm_data_t *migrate_op,
 		  pe_working_set_t *data_set) 
 {
-	if(on_fail == action_migrate_failure) {
-		node_t *from = NULL;
-		const char *uuid = NULL;
-		uuid = crm_element_value(migrate_op, CRMD_ACTION_MIGRATED);
-		from = pe_find_node_id(data_set->nodes, uuid);
-		process_rsc_state(rsc, from, action_fail_recover,NULL,data_set);
-		on_fail = action_fail_recover;
-	}
-	
 	crm_debug_2("Resource %s is %s on %s",
 		    rsc->id, role2text(rsc->role),
 		    node->details->uname);
 
-	/* process current state */
 	if(rsc->role != RSC_ROLE_UNKNOWN) { 
 		rsc->known_on = g_list_append(rsc->known_on, node);
 	}
@@ -740,32 +726,14 @@ process_rsc_state(resource_t *rsc, node_t *node,
 	if(rsc->role != RSC_ROLE_STOPPED
 		&& rsc->role != RSC_ROLE_UNKNOWN) { 
 		if(on_fail != action_fail_ignore) {
-		    set_bit(rsc->flags, pe_rsc_failed);
-		    crm_debug_2("Force stop");
+			rsc->failed = TRUE;
+			crm_debug_2("Force stop");
 		}
 
+		crm_debug_2("Adding %s to %s",
+			    rsc->id, node->details->uname);
 		native_add_running(rsc, node, data_set);
-
-		if(is_set(rsc->flags, pe_rsc_managed) && rsc->stickiness != 0) {
-		    node_t *match = pe_find_node_id(rsc->allowed_nodes, node->details->id);
-
-		    if(match != NULL || data_set->symmetric_cluster) {
-			resource_t *sticky_rsc = rsc;
-			if(rsc->parent && rsc->parent->variant == pe_group) {
-			    sticky_rsc = rsc->parent;
-			}
 			
-			resource_location(sticky_rsc, node, rsc->stickiness,
-					  "stickiness", data_set);
-			crm_debug_2("Resource %s: preferring current location"
-				    " (node=%s, weight=%d)", sticky_rsc->id,
-				    node->details->uname, rsc->stickiness);
-		    } else {
-			crm_debug("Ignoring stickiness for %s: the cluster is asymmetric and node %s is no longer explicitly allowed",
-				  rsc->id, node->details->uname);
-		    }
-		}
-	
 		if(on_fail == action_fail_ignore) {
 			/* nothing to do */
 		} else if(node->details->unclean) {
@@ -782,7 +750,7 @@ process_rsc_state(resource_t *rsc, node_t *node,
 			/* is_managed == FALSE will prevent any
 			 * actions being sent for the resource
 			 */
-		    clear_bit(rsc->flags, pe_rsc_managed);
+			rsc->is_managed = FALSE;
 				
 		} else if(on_fail == action_fail_migrate) {
 			stop_action(rsc, node, FALSE);
@@ -822,7 +790,7 @@ process_recurring(node_t *node, resource_t *rsc,
 	const char *task = NULL;
 	const char *status = NULL;
 	
-	crm_debug_3("%s: Start index %d, stop index = %d",
+	crm_debug_2("%s: Start index %d, stop index = %d",
 		    rsc->id, start_index, stop_index);
 	slist_iter(rsc_op, crm_data_t, sorted_op_list, lpc,
 		   int interval = 0;
@@ -844,8 +812,8 @@ process_recurring(node_t *node, resource_t *rsc,
 				       id, node->details->uname);
 			   continue;
 		   }
-		   	
-		   interval_s = crm_element_value(rsc_op,XML_LRM_ATTR_INTERVAL);
+		   
+		   interval_s = get_interval(rsc_op);
 		   interval = crm_parse_int(interval_s, "0");
 		   if(interval == 0) {
 			   crm_debug_4("Skipping %s/%s: non-recurring",
@@ -867,56 +835,26 @@ process_recurring(node_t *node, resource_t *rsc,
 		);
 }
 
-void
-calculate_active_ops(GListPtr sorted_op_list, int *start_index, int *stop_index) 
-{
-	const char *task = NULL;
-	const char *status = NULL;
-
-	*stop_index = -1;
-	*start_index = -1;
-	
-	slist_iter(
-		rsc_op, crm_data_t, sorted_op_list, lpc,
-
-		task = crm_element_value(rsc_op, XML_LRM_ATTR_TASK);
-		status = crm_element_value(rsc_op, XML_LRM_ATTR_OPSTATUS);
-
-		if(safe_str_eq(task, CRMD_ACTION_STOP)
-		   && safe_str_eq(status, "0")) {
-			*stop_index = lpc;
-			
-		} else if(safe_str_eq(task, CRMD_ACTION_START)) {
-			*start_index = lpc;
-			
-		} else if(*start_index <= *stop_index
-			  && safe_str_eq(task, CRMD_ACTION_STATUS)) {
-			const char *rc = crm_element_value(rsc_op, XML_LRM_ATTR_RC);
-			if(safe_str_eq(rc, "0") || safe_str_eq(rc, "8")) {
-				*start_index = lpc;
-			}
-		}
-		);
-}
-
 static void
 unpack_lrm_rsc_state(
 	node_t *node, crm_data_t * rsc_entry, pe_working_set_t *data_set)
 {	
+	int fail_count = 0;
 	int stop_index = -1;
 	int start_index = -1;
 	int max_call_id = -1;
 
+	char *fail_attr = NULL;
 	const char *task = NULL;
+	const char *status = NULL;
 	const char *value = NULL;
+	const char *fail_val = NULL;
 	const char *rsc_id  = crm_element_value(rsc_entry, XML_ATTR_ID);
 
 	resource_t *rsc = NULL;
 	GListPtr op_list = NULL;
 	GListPtr sorted_op_list = NULL;
 
-	crm_data_t *migrate_op = NULL;
-	
 	enum action_fail_response on_fail = FALSE;
 	enum rsc_role_e saved_role = RSC_ROLE_UNKNOWN;
 	
@@ -944,6 +882,23 @@ unpack_lrm_rsc_state(
 	} 
 	CRM_ASSERT(rsc != NULL);
 	
+	/* process failure stickiness */
+	fail_count = 0;
+	fail_attr = crm_concat("fail-count", rsc->id, '-');
+	fail_val = g_hash_table_lookup(node->details->attrs, fail_attr);
+	if(fail_val != NULL) {
+		crm_debug("%s: %s", fail_attr, fail_val);
+		fail_count = crm_parse_int(fail_val, "0");
+	}
+	crm_free(fail_attr);
+	if(fail_count > 0 && rsc->fail_stickiness != 0) {
+		resource_location(rsc, node, fail_count * rsc->fail_stickiness,
+				  "fail_stickiness", data_set);
+		crm_debug("Setting failure stickiness for %s on %s: %d",
+			  rsc->id, node->details->uname,
+			  fail_count * rsc->fail_stickiness);
+	}
+
 	/* process operations */
 	max_call_id = -1;
 
@@ -954,10 +909,22 @@ unpack_lrm_rsc_state(
 	
 	slist_iter(
 		rsc_op, crm_data_t, sorted_op_list, lpc,
-
 		task = crm_element_value(rsc_op, XML_LRM_ATTR_TASK);
-		if(safe_str_eq(task, CRMD_ACTION_MIGRATED)) {
-			migrate_op = rsc_op;
+		status = crm_element_value(rsc_op, XML_LRM_ATTR_OPSTATUS);
+		if(safe_str_eq(task, CRMD_ACTION_STOP)
+		   && safe_str_eq(status, "0")) {
+			stop_index = lpc;
+			
+		} else if(safe_str_eq(task, CRMD_ACTION_START)) {
+			start_index = lpc;
+			
+		} else if(start_index <= stop_index
+			  && safe_str_eq(task, CRMD_ACTION_STATUS)) {
+			const char *rc = crm_element_value(rsc_op, XML_LRM_ATTR_RC);
+			if(safe_str_eq(rc, "0")
+			   || safe_str_eq(rc, "8")) {
+				start_index = lpc;
+			}
 		}
 		
 		unpack_rsc_op(rsc, node, rsc_op,
@@ -965,14 +932,13 @@ unpack_lrm_rsc_state(
 		);
 
 	/* create active recurring operations as optional */ 
-	calculate_active_ops(sorted_op_list, &start_index, &stop_index);
 	process_recurring(node, rsc, start_index, stop_index,
 			  sorted_op_list, data_set);
 	
 	/* no need to free the contents */
 	g_list_free(sorted_op_list);
 	
-	process_rsc_state(rsc, node, on_fail, migrate_op, data_set);
+	process_rsc_state(rsc, node, on_fail, data_set);
 
 	value = g_hash_table_lookup(rsc->meta, XML_RSC_ATTR_TARGET_ROLE);
 	if(value != NULL && safe_str_neq("default", value)) {
@@ -999,11 +965,6 @@ unpack_lrm_resources(node_t *node, crm_data_t * lrm_rsc_list, pe_working_set_t *
 	CRM_CHECK(node != NULL, return FALSE);
 
 	crm_debug_3("Unpacking resources on %s", node->details->uname);
-
-	slist_iter(
-	    rsc, resource_t, data_set->resources, lpc,
-	    common_apply_stickiness(rsc, node, data_set);
-	    );
 	
 	xml_child_iter_filter(
 		lrm_rsc_list, rsc_entry, XML_LRM_TAG_RESOURCE,
@@ -1019,9 +980,8 @@ gboolean
 unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 	      int *max_call_id, enum action_fail_response *on_fail,
 	      pe_working_set_t *data_set) 
-{    
+{
 	const char *id          = NULL;
-	const char *key        = NULL;
 	const char *task        = NULL;
  	const char *task_id     = NULL;
  	const char *actual_rc   = NULL;
@@ -1029,19 +989,15 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 	const char *task_status = NULL;
 	const char *interval_s  = NULL;
 	const char *op_digest   = NULL;
-	const char *op_version  = NULL;
 
 	int interval = 0;
 	int task_id_i = -1;
 	int task_status_i = -2;
 	int actual_rc_i = 0;
-	int target_rc = -1;
 	
 	action_t *action = NULL;
-	node_t *effective_node = NULL;
-	resource_t *failed = NULL;
-
 	gboolean is_probe = FALSE;
+	gboolean is_stop_action = FALSE;
 	
 	CRM_CHECK(rsc    != NULL, return FALSE);
 	CRM_CHECK(node   != NULL, return FALSE);
@@ -1052,8 +1008,6 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
  	task_id     = crm_element_value(xml_op, XML_LRM_ATTR_CALLID);
 	task_status = crm_element_value(xml_op, XML_LRM_ATTR_OPSTATUS);
 	op_digest   = crm_element_value(xml_op, XML_LRM_ATTR_OP_DIGEST);
-	op_version  = crm_element_value(xml_op, XML_ATTR_CRM_VERSION);
-	key	    = crm_element_value(xml_op, XML_ATTR_TRANSITION_KEY);
 
 	CRM_CHECK(id != NULL, return FALSE);
 	CRM_CHECK(task != NULL, return FALSE);
@@ -1073,11 +1027,19 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 		    id, task, task_id, task_status, node->details->uname,
 		    role2text(rsc->role));
 
-	interval_s = crm_element_value(xml_op, XML_LRM_ATTR_INTERVAL);
+	interval_s = get_interval(xml_op);
 	interval = crm_parse_int(interval_s, "0");
 	
 	if(interval == 0 && safe_str_eq(task, CRMD_ACTION_STATUS)) {
 		is_probe = TRUE;
+
+	} else if(interval > 0 && rsc->role < RSC_ROLE_STARTED) {
+		crm_debug_2("Skipping recurring action %s for stopped resource", id);
+		return FALSE;
+	}
+	
+	if(safe_str_eq(task, CRMD_ACTION_STOP)) {
+		is_stop_action = TRUE;
 	}
 	
 	if(task_status_i != LRM_OP_PENDING) {
@@ -1101,141 +1063,47 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 	actual_rc = crm_element_value(xml_op, XML_LRM_ATTR_RC);
 	CRM_CHECK(actual_rc != NULL, return FALSE);	
 	actual_rc_i = crm_parse_int(actual_rc, NULL);
-
-	if(key) {
-	    int dummy = 0;
-	    char *dummy_string = NULL;
-	    decode_transition_key(key, &dummy_string, &dummy, &dummy, &target_rc);
-	    crm_free(dummy_string);
-	}
 	
-	if(task_status_i == LRM_OP_DONE && target_rc >= 0) {
-	    if(target_rc == actual_rc_i) {
-		task_status_i = LRM_OP_DONE;
-		
-	    } else {
-		task_status_i = LRM_OP_ERROR;
-		crm_info("Remapping %s (rc=%d) on %s to an ERROR (expected %d)",
-			 id, actual_rc_i, node->details->uname, target_rc);
-	    }
-
-	} else if(task_status_i == LRM_OP_ERROR) {
-	    /* let us decide that */
-	    crm_debug("Remapping %s (rc=%d, status=%d) on %s to DONE",
-		      id, actual_rc_i, task_status_i, node->details->uname);
- 	    task_status_i = LRM_OP_DONE;
-	}
-	
-	if(task_status_i == LRM_OP_NOTSUPPORTED) {
-	    actual_rc_i = EXECRA_UNIMPLEMENT_FEATURE;
-	}
-
-	/* we could clean this up significantly except for old LRMs and CRMs that
-	 * didnt include target_rc and liked to remap status
-	 */
-	switch(actual_rc_i) {
-	    case EXECRA_NOT_RUNNING:
-		if(is_probe || target_rc == actual_rc_i) {
-		    task_status_i = LRM_OP_DONE;
-		    rsc->role = RSC_ROLE_STOPPED;
-		    
-		    /* clear any previous failure actions */
-		    *on_fail = action_fail_ignore;
-		    rsc->next_role = RSC_ROLE_UNKNOWN;
-		    
-		} else if(safe_str_neq(task, CRMD_ACTION_STOP)) {
-		    task_status_i = LRM_OP_ERROR;
-		}
-		break;
-		
-	    case EXECRA_RUNNING_MASTER:
-		if(is_probe) {
-		    task_status_i = LRM_OP_DONE;
-		    crm_warn("%s found active %s in master mode on %s",
-			     id, rsc->id, node->details->uname);
-
-		} else if(target_rc == actual_rc_i) {
-		    /* nothing to do */
-
-		} else if(target_rc >= 0) {
-		    task_status_i = LRM_OP_ERROR;
-
-		    /* legacy code for pre-0.6.5 operations */
-		} else if(safe_str_neq(task, CRMD_ACTION_STATUS)
-			  || rsc->role != RSC_ROLE_MASTER) {
-		    task_status_i = LRM_OP_ERROR;
-		    if(rsc->role != RSC_ROLE_MASTER) {
-			crm_err("%s reported %s in master mode on %s",
-				id, rsc->id,
-				node->details->uname);
-		    }
-		}
-		rsc->role = RSC_ROLE_MASTER;
-		break;
-		
-	    case EXECRA_FAILED_MASTER:
-		rsc->role = RSC_ROLE_MASTER;
-		task_status_i = LRM_OP_ERROR;
-		break;
-
-	    case EXECRA_UNIMPLEMENT_FEATURE:
-		if(interval > 0) {
-		    task_status_i = LRM_OP_NOTSUPPORTED;
-		    break;
-		}
-		/* else: fall through */
-	    case EXECRA_INSUFFICIENT_PRIV:
-	    case EXECRA_NOT_INSTALLED:
-	    case EXECRA_INVALID_PARAM:
-		effective_node = node;
-		/* fall through */
-	    case EXECRA_NOT_CONFIGURED:
-		failed = rsc;
-		if(is_not_set(rsc->flags, pe_rsc_unique)) {
-		    failed = uber_parent(failed);
-		}
-		
-		crm_err("Hard error: %s failed with rc=%d.", id, actual_rc_i);
-		if(effective_node) {
-		    crm_err("  Preventing %s from re-starting on %s",
-			    failed->id, effective_node->details->uname);
-		} else {
-		    crm_err("  Preventing %s from re-starting anywhere in the cluster",
-			    failed->id);
-		}
-
-		resource_location(failed, effective_node, -INFINITY, "hard-error", data_set);
+	if(EXECRA_NOT_RUNNING == actual_rc_i) {
 		if(is_probe) {
 			/* treat these like stops */
-			task = CRMD_ACTION_STOP;
+			is_stop_action = TRUE;
+		}
+		if(is_stop_action) {
 			task_status_i = LRM_OP_DONE;
-		}
-		break;
-
-	    case EXECRA_OK:
-		if(is_probe && target_rc == 7) {
-		    task_status_i = LRM_OP_DONE;
-		    crm_warn("%s found active %s on %s",
-			     id, rsc->id, node->details->uname);
-
-		    /* legacy code for pre-0.6.5 operations */
-		} else if(target_rc < 0
-		   && interval > 0
-		   && rsc->role == RSC_ROLE_MASTER) {
-		    /* catch status ops that return 0 instead of 8 while they
-		     *   are supposed to be in master mode
-		     */
-		    task_status_i = LRM_OP_ERROR;
+ 		} else {
+			CRM_CHECK(task_status_i == LRM_OP_ERROR,
+				task_status_i = LRM_OP_ERROR);
 		}
 		
-		break;
-		
-	    default:
-		if(task_status_i == LRM_OP_DONE) {
-		    crm_info("Remapping %s (rc=%d) on %s to an ERROR",
-			     id, actual_rc_i, node->details->uname);
-		    task_status_i = LRM_OP_ERROR;
+	} else if(EXECRA_RUNNING_MASTER == actual_rc_i) {
+		if(is_probe
+		   || (rsc->role == RSC_ROLE_MASTER
+		       && safe_str_eq(task, CRMD_ACTION_STATUS))) {
+			task_status_i = LRM_OP_DONE;
+		} else {
+			if(rsc->role != RSC_ROLE_MASTER) {
+				crm_err("%s reported %s in master mode on %s",
+					id, rsc->id,
+					node->details->uname);
+			}
+			
+			CRM_CHECK(task_status_i == LRM_OP_ERROR,
+				task_status_i = LRM_OP_ERROR);
 		}
+		rsc->role = RSC_ROLE_MASTER;
+
+	} else if(EXECRA_FAILED_MASTER == actual_rc_i) {
+		rsc->role = RSC_ROLE_MASTER;
+		task_status_i = LRM_OP_ERROR;
+
+	} else if(EXECRA_OK == actual_rc_i
+		  && interval > 0
+		  && rsc->role == RSC_ROLE_MASTER) {
+		/* catch status ops that return 0 instead of 8 while they
+		 *   are supposed to be in master mode
+		 */
+		task_status_i = LRM_OP_ERROR;
 	}
 
 	if(task_status_i == LRM_OP_ERROR
@@ -1251,7 +1119,7 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 	switch(task_status_i) {
 		case LRM_OP_PENDING:
 			if(safe_str_eq(task, CRMD_ACTION_START)) {
-				set_bit(rsc->flags, pe_rsc_start_pending);
+				rsc->start_pending = TRUE;
 				rsc->role = RSC_ROLE_STARTED;
 				
 			} else if(safe_str_eq(task, CRMD_ACTION_PROMOTE)) {
@@ -1263,16 +1131,13 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 			crm_debug_3("%s/%s completed on %s",
 				    rsc->id, task, node->details->uname);
 
-			if(actual_rc_i == EXECRA_NOT_RUNNING) {
-			    /* nothing to do */
-			    
-			} else if(safe_str_eq(task, CRMD_ACTION_STOP)) {
+			if(is_stop_action) {
 				rsc->role = RSC_ROLE_STOPPED;
-			    
+
 				/* clear any previous failure actions */
 				*on_fail = action_fail_ignore;
 				rsc->next_role = RSC_ROLE_UNKNOWN;
-
+				
 			} else if(safe_str_eq(task, CRMD_ACTION_PROMOTE)) {
 				rsc->role = RSC_ROLE_MASTER;
 
@@ -1289,45 +1154,31 @@ unpack_rsc_op(resource_t *rsc, node_t *node, crm_data_t *xml_op,
 		case LRM_OP_ERROR:
 		case LRM_OP_TIMEOUT:
 		case LRM_OP_NOTSUPPORTED:
-			crm_warn("Processing failed op %s on %s: %s",
-				 id, node->details->uname,
-				 op_status2text(task_status_i));
-			crm_xml_add(xml_op, XML_ATTR_UNAME, node->details->uname);
-			add_node_copy(data_set->failed, xml_op);
+			crm_warn("Processing failed op (%s) for %s on %s",
+				 id, rsc->id, node->details->uname);
 
 			if(*on_fail < action->on_fail) {
 				*on_fail = action->on_fail;
 			}
-
-			if(safe_str_eq(task, CRMD_ACTION_STOP)) {
-			    resource_location(
-				rsc, node, -INFINITY, "__stop_fail__", data_set);
-			    
-			} else if(safe_str_eq(task, CRMD_ACTION_PROMOTE)) {
-			    rsc->role = RSC_ROLE_MASTER;
+			
+			if(task_status_i == LRM_OP_NOTSUPPORTED
+			   || is_stop_action
+			   || safe_str_eq(task, CRMD_ACTION_START) ) {
+				crm_warn("Handling failed %s for %s on %s",
+					 task, rsc->id, node->details->uname);
+				resource_location(rsc, node, -INFINITY,
+					  "__dont_run__failed_stopstart__",
+					  data_set);
+			}
+			
+			if(safe_str_eq(task, CRMD_ACTION_PROMOTE)) {
+				rsc->role = RSC_ROLE_MASTER;
 
 			} else if(safe_str_eq(task, CRMD_ACTION_DEMOTE)) {
-			    /*
-			     * staying in role=master ends up putting the PE/TE into a loop
-			     * setting role=slave is not dangerous because no master will be
-			     * promoted until the failed resource has been fully stopped
-			     */
-			    crm_warn("Forcing %s to stop after a failed demote action", rsc->id);
-			    rsc->next_role = RSC_ROLE_STOPPED;
-			    rsc->role = RSC_ROLE_SLAVE;
+				rsc->role = RSC_ROLE_MASTER;
 				
-			} else if((data_set->start_failure_fatal
-				   || compare_version("2.0", op_version) > 0)
-				  && safe_str_eq(task, CRMD_ACTION_START)) {
-			    crm_warn("Compatability handling for failed op %s on %s",
-				     id, node->details->uname);
-			    resource_location(
-				rsc, node, -INFINITY, "__legacy_start__", data_set);
-
-			}
-
-			if(rsc->role < RSC_ROLE_STARTED) {
-			    rsc->role = RSC_ROLE_STARTED;
+			} else if(rsc->role < RSC_ROLE_STARTED) {
+				rsc->role = RSC_ROLE_STARTED;
 			}
 
 			crm_debug_2("Resource %s: role=%s, unclean=%s, on_fail=%s, fail_role=%s",
