@@ -1,3 +1,4 @@
+/* $Id: utils.c,v 1.8 2006/06/22 16:21:51 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -15,7 +16,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#include <lha_internal.h>
+#include <portability.h>
 #include <crm/crm.h>
 #include <crm/cib.h>
 #include <crm/msg_xml.h>
@@ -43,12 +44,6 @@ pe_free_shallow_adv(GListPtr alist, gboolean with_data)
 {
 	GListPtr item;
 	GListPtr item_next = alist;
-
-	if(with_data == FALSE && alist != NULL) {
-		g_list_free(alist);
-		return;
-	}
-	
 	while(item_next != NULL) {
 		item = item_next;
 		item_next = item_next->next;
@@ -60,10 +55,25 @@ pe_free_shallow_adv(GListPtr alist, gboolean with_data)
 		
 		item->data = NULL;
 		item->next = NULL;
-		g_list_free_1(item);
+		g_list_free(item);
 	}
 }
 
+node_t *
+pe_find_node_id(GListPtr nodes, const char *id)
+{
+	unsigned lpc = 0;
+	node_t *node = NULL;
+  
+	for(lpc = 0; lpc < g_list_length(nodes); lpc++) {
+		node = g_list_nth_data(nodes, lpc);
+		if(safe_str_eq(node->details->id, id)) {
+			return node;
+		}
+	}
+	/* error */
+	return NULL;
+}
 
 node_t *
 node_copy(node_t *this_node) 
@@ -71,13 +81,12 @@ node_copy(node_t *this_node)
 	node_t *new_node  = NULL;
 
 	CRM_CHECK(this_node != NULL, return NULL);
-
 	crm_malloc0(new_node, sizeof(node_t));
-	CRM_ASSERT(new_node != NULL);
+
+	CRM_CHECK(new_node != NULL, return NULL);
 	
 	crm_debug_5("Copying %p (%s) to %p",
 		  this_node, this_node->details->uname, new_node);
-
 	new_node->weight  = this_node->weight; 
 	new_node->fixed   = this_node->fixed;
 	new_node->details = this_node->details;	
@@ -135,43 +144,8 @@ node_list_eq(GListPtr list1, GListPtr list2, gboolean filter)
 	return TRUE;
 }
 
-/* any node in list1 or list2 and not in the other gets a score of -INFINITY */
-GListPtr
-node_list_exclude(GListPtr list1, GListPtr list2)
-{
-    node_t *other_node = NULL;
-    GListPtr result = NULL;
-    
-    result = node_list_dup(list1, FALSE, FALSE);
-    
-    slist_iter(
-	node, node_t, result, lpc,
-	
-	other_node = pe_find_node_id(list2, node->details->id);
-	
-	if(other_node == NULL) {
-	    node->weight = -INFINITY;
-	} else {
-	    node->weight = merge_weights(node->weight, other_node->weight);
-	}
-	);
-    
-    slist_iter(
-	node, node_t, list2, lpc,
-	
-	other_node = pe_find_node_id(result, node->details->id);
-	
-	if(other_node == NULL) {
-	    node_t *new_node = node_copy(node);
-	    new_node->weight = -INFINITY;
-	    result = g_list_append(result, new_node);
-	}
-	);
-
-    return result;
-}
-
-/* the intersection of list1 and list2 */
+/* the intersection of list1 and list2 
+ */
 GListPtr
 node_list_and(GListPtr list1, GListPtr list2, gboolean filter)
 {
@@ -348,51 +322,6 @@ node_list_dup(GListPtr list1, gboolean reset, gboolean filter)
 }
 
 
-void dump_node_scores(int level, resource_t *rsc, const char *comment, GListPtr nodes) 
-{
-    GListPtr list = nodes;
-    if(rsc) {
-	list = rsc->allowed_nodes;
-    }
-    
-    slist_iter(
-	node, node_t, list, lpc,
-	if(rsc) {
-	    do_crm_log(level, "%s: %s allocation score on %s: %d",
-		       comment, rsc->id, node->details->uname, node->weight);
-	} else {
-	    do_crm_log(level, "%s: %s = %d", comment, node->details->uname, node->weight);
-	}
-	);
-
-    if(rsc && rsc->children) {
-	slist_iter(
-	    child, resource_t, rsc->children, lpc,
-	    dump_node_scores(level, child, comment, nodes);
-	    );
-    }
-}
-
-gint sort_rsc_index(gconstpointer a, gconstpointer b)
-{
-	const resource_t *resource1 = (const resource_t*)a;
-	const resource_t *resource2 = (const resource_t*)b;
-
-	if(a == NULL && b == NULL) { return 0; }
-	if(a == NULL) { return 1; }
-	if(b == NULL) { return -1; }
-  
-	if(resource1->sort_index > resource2->sort_index) {
-		return -1;
-	}
-	
-	if(resource1->sort_index < resource2->sort_index) {
-		return 1;
-	}
-
-	return 0;
-}
-
 gint sort_rsc_priority(gconstpointer a, gconstpointer b)
 {
 	const resource_t *resource1 = (const resource_t*)a;
@@ -441,66 +370,66 @@ custom_action(resource_t *rsc, char *key, const char *task,
 		crm_debug_4("Found existing action (%d) %s for %s on %s",
 			  action->id, task, rsc?rsc->id:"<NULL>",
 			  on_node?on_node->details->uname:"<NULL>");
-		g_list_free(possible_matches);
 	}
 
 	if(action == NULL) {
 		if(save_action) {
-			crm_debug_4("Creating%s action %d: %s for %s on %s",
+			crm_debug_2("Creating%s action %d: %s for %s on %s",
 				    optional?"":" manditory", data_set->action_id, key, rsc?rsc->id:"<NULL>",
 				    on_node?on_node->details->uname:"<NULL>");
 		}
 		
 		crm_malloc0(action, sizeof(action_t));
-		if(save_action) {
-			action->id   = data_set->action_id++;
-		} else {
-			action->id = 0;
-		}
-		action->rsc  = rsc;
-		CRM_ASSERT(task != NULL);
-		action->task = crm_strdup(task);
-		action->node = on_node;
-		action->uuid = key;
-		
-		action->actions_before   = NULL;
-		action->actions_after    = NULL;
-		action->failure_is_fatal = TRUE;
-		
-		action->pseudo     = FALSE;
-		action->dumped     = FALSE;
-		action->runnable   = TRUE;
-		action->processed  = FALSE;
-		action->optional   = optional;
-		action->seen_count = 0;
-		
-		action->extra = g_hash_table_new_full(
-			g_str_hash, g_str_equal,
-			g_hash_destroy_str, g_hash_destroy_str);
-		
-		action->meta = g_hash_table_new_full(
-			g_str_hash, g_str_equal,
-			g_hash_destroy_str, g_hash_destroy_str);
-		
-		if(save_action) {
-			data_set->actions = g_list_append(
-				data_set->actions, action);
-		}		
-		
-		if(rsc != NULL) {
-			action->op_entry = find_rsc_op_entry(rsc, key);
-			
-			unpack_operation(
-				action, action->op_entry, data_set);
-			
+		if(action != NULL) {
 			if(save_action) {
-				rsc->actions = g_list_append(
-					rsc->actions, action);
+				action->id   = data_set->action_id++;
+			} else {
+				action->id = 0;
+			}
+			action->rsc  = rsc;
+			action->task = task;
+			action->node = on_node;
+			
+			action->actions_before   = NULL;
+			action->actions_after    = NULL;
+			action->failure_is_fatal = TRUE;
+			
+			action->pseudo     = FALSE;
+			action->dumped     = FALSE;
+			action->runnable   = TRUE;
+			action->processed  = FALSE;
+			action->optional   = optional;
+			action->seen_count = 0;
+			
+			action->extra = g_hash_table_new_full(
+				g_str_hash, g_str_equal,
+				g_hash_destroy_str, g_hash_destroy_str);
+
+			action->meta = g_hash_table_new_full(
+				g_str_hash, g_str_equal,
+				g_hash_destroy_str, g_hash_destroy_str);
+
+			if(save_action) {
+				data_set->actions = g_list_append(
+					data_set->actions, action);
+			}
+			
+			action->uuid = key;
+
+			if(rsc != NULL) {
+				action->op_entry = find_rsc_op_entry(rsc, key);
+				
+				unpack_operation(
+					action, action->op_entry, data_set);
+
+				if(save_action) {
+					rsc->actions = g_list_append(
+						rsc->actions, action);
+				}
+			}
+			if(save_action) {
+				crm_debug_4("Action %d created", action->id);
 			}
-		}
-		
-		if(save_action) {
-			crm_debug_4("Action %d created", action->id);
 		}
 	}
 
@@ -524,27 +453,30 @@ custom_action(resource_t *rsc, char *key, const char *task,
 				action->extra, NULL, data_set->now);
 		}
 
-		if(action->pseudo) {
-			/* leave untouched */
-			
-		} else if(action->node == NULL) {
+		if(action->node == NULL) {
 			action->runnable = FALSE;
-			
-		} else if(g_hash_table_lookup(action->meta, XML_LRM_ATTR_INTERVAL) == NULL
-			  && is_not_set(rsc->flags, pe_rsc_managed)) {
-			do_crm_log(warn_level, "Action %s (unmanaged)",
-				 action->uuid);
+
+		} else if(rsc->is_managed == FALSE) {
+			crm_log_maybe(warn_level, "Action %s %s is for %s (unmanaged)",
+				 action->uuid, task, rsc->id);
 			action->optional = TRUE;
 /*   			action->runnable = FALSE; */
 
+#if 0
+		} else if(action->node->details->unclean) {
+			crm_log_maybe(warn_level, "Action %s on %s is unrunnable (unclean)",
+				 action->uuid, action->node?action->node->details->uname:"<none>");
+
+			action->runnable = FALSE;
+#endif	
 		} else if(action->node->details->online == FALSE) {
 			action->runnable = FALSE;
-			do_crm_log(warn_level, "Action %s on %s is unrunnable (offline)",
+			crm_log_maybe(warn_level, "Action %s on %s is unrunnable (offline)",
 				 action->uuid, action->node->details->uname);
-			if(is_set(action->rsc->flags, pe_rsc_managed)
+			if(action->rsc->is_managed
 			   && save_action
 			   && a_task == stop_rsc) {
-				do_crm_log(warn_level, "Marking node %s unclean",
+				crm_log_maybe(warn_level, "Marking node %s unclean",
 					 action->node->details->uname);
 				action->node->details->unclean = TRUE;
 			}
@@ -565,18 +497,18 @@ custom_action(resource_t *rsc, char *key, const char *task,
 		} else if(data_set->have_quorum == FALSE
 			&& data_set->no_quorum_policy == no_quorum_stop) {
 			action->runnable = FALSE;
-			crm_debug("%s\t%s (cancelled : quorum)",
+			crm_debug("%s\t%s %s (cancelled : quorum)",
 				  action->node->details->uname,
-				  action->uuid);
+				  action->task, rsc->id);
 			
 		} else if(data_set->have_quorum == FALSE
 			&& data_set->no_quorum_policy == no_quorum_freeze) {
 			crm_debug_3("Check resource is already active");
 			if(rsc->fns->active(rsc, TRUE) == FALSE) {
 				action->runnable = FALSE;
-				crm_debug("%s\t%s (cancelled : quorum freeze)",
+				crm_debug("%s\t%s %s (cancelled : quorum freeze)",
 					  action->node->details->uname,
-					  action->uuid);
+					  action->task, rsc->id);
 			}
 
 		} else {
@@ -587,14 +519,14 @@ custom_action(resource_t *rsc, char *key, const char *task,
 		if(save_action) {
 			switch(a_task) {
 				case stop_rsc:
-				    set_bit(rsc->flags, pe_rsc_stopping);
-				    break;
+					rsc->stopping = TRUE;
+					break;
 				case start_rsc:
-				    clear_bit(rsc->flags, pe_rsc_starting);
-				    if(action->runnable) {
-					set_bit(rsc->flags, pe_rsc_starting);
-				    }
-				    break;
+					rsc->starting = FALSE;
+					if(action->runnable) {
+					rsc->starting = TRUE;
+					}
+					break;
 				default:
 					break;
 			}
@@ -607,21 +539,23 @@ void
 unpack_operation(
 	action_t *action, crm_data_t *xml_obj, pe_working_set_t* data_set)
 {
-	int value_i = 0;
-	int start_delay = 0;
-	char *value_ms = NULL;
+	int lpc = 0;
 	const char *class = NULL;
 	const char *value = NULL;
-	const char *field = NULL;
-	
+	const char *fields[] = {
+		XML_LRM_ATTR_INTERVAL,
+		"timeout",
+		"start_delay",
+	};
+
 	CRM_CHECK(action->rsc != NULL, return);
-	class = g_hash_table_lookup(action->rsc->meta, "class");
 	
 	if(xml_obj != NULL) {
 		value = crm_element_value(xml_obj, "prereq");
 	}
 	if(value == NULL && safe_str_eq(action->task, CRMD_ACTION_START)) {
-		value = g_hash_table_lookup(action->rsc->meta, "start_prereq");
+		value = g_hash_table_lookup(
+			action->rsc->meta, "start_prereq");
 	}
 	
 	if(value == NULL && safe_str_neq(action->task, CRMD_ACTION_START)) {
@@ -638,8 +572,7 @@ unpack_operation(
 	} else if(safe_str_eq(value, "fencing")) {
 		action->needs = rsc_req_stonith;
 		
-	} else if(data_set->no_quorum_policy == no_quorum_ignore
-	    || safe_str_eq(class, "stonith")) {
+	} else if(data_set->no_quorum_policy == no_quorum_ignore) {
 		action->needs = rsc_req_nothing;
 		value = "nothing (default)";
 		
@@ -653,9 +586,10 @@ unpack_operation(
 		value = "quorum (default)";
 	}
 
+	class = g_hash_table_lookup(action->rsc->meta, "class");
 	if(safe_str_eq(class, "stonith")) {
 		if(action->needs == rsc_req_stonith) {
-			crm_config_err("Stonith resources (eg. %s) cannot require"
+			pe_config_err("Stonith resources (eg. %s) cannot require"
 				      " fencing to start", action->rsc->id);
 		}
 		action->needs = rsc_req_nothing;
@@ -672,16 +606,16 @@ unpack_operation(
 			action->rsc->meta, "on_stopfail");
 		if(value != NULL) {
 #if CRM_DEPRECATED_SINCE_2_0_2
-			crm_config_err("The \"on_stopfail\" attribute used in"
+			pe_config_err("The \"on_stopfail\" attribute used in"
 				      " %s has been deprecated since 2.0.2",
 				      action->rsc->id);
 #else
-			crm_config_err("The \"on_stopfail\" attribute used in"
+			pe_config_err("The \"on_stopfail\" attribute used in"
 				      " %s has been deprecated since 2.0.2"
 				      " and is now disabled", action->rsc->id);
 			value = NULL;
 #endif
-			crm_config_err("Please use specify the \"on_fail\""
+			pe_config_err("Please use specify the \"on_fail\""
 				      " attribute on the \"stop\" operation"
 				      " instead");
 		}
@@ -694,17 +628,8 @@ unpack_operation(
 	} else if(safe_str_eq(value, "fence")) {
 		action->on_fail = action_fail_fence;
 		value = "node fencing";
-		
-		if(data_set->stonith_enabled == FALSE) {
-		    crm_config_err("Specifying on_fail=fence and"
-				   " stonith-enabled=false makes no sense");
-		    action->on_fail = action_fail_stop;
-		    action->fail_role = RSC_ROLE_STOPPED;
-		    value = "stop resource";
-		}
-		
-	} else if(safe_str_eq(value, "ignore")
-		|| safe_str_eq(value, "nothing")) {
+
+	} else if(safe_str_eq(value, "ignore")) {
 		action->on_fail = action_fail_ignore;
 		value = "ignore";
 
@@ -713,11 +638,11 @@ unpack_operation(
 		value = "force migration";
 		
 	} else if(safe_str_eq(value, "stop")) {
-		action->on_fail = action_fail_stop;
 		action->fail_role = RSC_ROLE_STOPPED;
 		value = "stop resource";
 		
-	} else if(safe_str_eq(value, "restart")) {
+	} else if(safe_str_eq(value, "restart")
+		|| safe_str_eq(value, "nothing")) {
 		action->on_fail = action_fail_recover;
 		value = "restart (and possibly migrate)";
 		
@@ -738,11 +663,6 @@ unpack_operation(
 			value = "resource block (default)";
 		}
 		
-	} else if(value == NULL
-		  && safe_str_eq(action->task, CRMD_ACTION_MIGRATED)) {
-		action->on_fail = action_migrate_failure;		
-		value = "atomic migration recovery (default)";
-		
 	} else if(value == NULL) {
 		action->on_fail = action_fail_recover;		
 		value = "restart (and possibly migrate) (default)";
@@ -782,47 +702,30 @@ unpack_operation(
 		unpack_instance_attributes(xml_obj, XML_TAG_ATTR_SETS,
 					   NULL, action->meta, NULL, data_set->now);
 	}
-
-	field = XML_LRM_ATTR_INTERVAL;
-	value = g_hash_table_lookup(action->meta, field);
-	if(value != NULL) {
-		value_i = crm_get_msec(value);
-		CRM_CHECK(value_i >= 0, value_i = 0);
-		value_ms = crm_itoa(value_i);
-		g_hash_table_replace(action->meta, crm_strdup(field), value_ms);
-	}
-
-	field = "start_delay";
-	value = g_hash_table_lookup(action->meta, field);
-	if(value != NULL) {
-		value_i = crm_get_msec(value);
-		if(value_i < 0) {
-			value_i = 0;
+	
+       if(g_hash_table_lookup(action->meta, "timeout") == NULL) {
+               g_hash_table_insert(action->meta, crm_strdup("timeout"),
+                                   crm_strdup(data_set->transition_idle_timeout));
+       }
+	
+	for(;lpc < DIMOF(fields); lpc++) {
+		value = g_hash_table_lookup(action->meta, fields[lpc]);
+		if(value != NULL) {
+			char *tmp_ms = NULL;
+			int tmp_i = crm_get_msec(value);
+			if(tmp_i < 0) {
+				tmp_i = 0;
+			}
+			tmp_ms = crm_itoa(tmp_i);
+			g_hash_table_replace(
+				action->meta, crm_strdup(fields[lpc]), tmp_ms);
 		}
-		start_delay = value_i;
-		value_ms = crm_itoa(value_i);
-		g_hash_table_replace(action->meta, crm_strdup(field), value_ms);
 	}
-
-	field = "timeout";
-	value = g_hash_table_lookup(action->meta, field);
-	if(value == NULL) {
-		value = pe_pref(
-			data_set->config_hash, "default-action-timeout");
-	}
-	value_i = crm_get_msec(value);
-	if(value_i < 0) {
-		value_i = 0;
-	}
-	value_i += start_delay;
-	value_ms = crm_itoa(value_i);
-	g_hash_table_replace(action->meta, crm_strdup(field), value_ms);
 }
 
 crm_data_t *
 find_rsc_op_entry(resource_t *rsc, const char *key) 
 {
-	int number = 0;
 	const char *name = NULL;
 	const char *value = NULL;
 	const char *interval = NULL;
@@ -839,13 +742,9 @@ find_rsc_op_entry(resource_t *rsc, const char *key)
 			crm_debug_2("%s disabled", ID(operation));
 			continue;
 		}
-
-		number = crm_get_msec(interval);
-		if(number < 0) {
-		    continue;
-		}
 		
-		match_key = generate_op_key(rsc->id, name, number);
+		match_key = generate_op_key(
+			rsc->id, name, crm_get_msec(interval));
 
 		if(safe_str_eq(key, match_key)) {
 			op = operation;
@@ -912,7 +811,7 @@ print_resource(
 	long options = pe_print_log;
 	
 	if(rsc == NULL) {
-		do_crm_log(log_level-1, "%s%s: <NULL>",
+		crm_log_maybe(log_level-1, "%s%s: <NULL>",
 			      pre_text==NULL?"":pre_text,
 			      pre_text==NULL?"":": ");
 		return;
@@ -923,6 +822,89 @@ print_resource(
 	rsc->fns->print(rsc, pre_text, options, &log_level);
 }
 
+void
+log_action(unsigned int log_level, const char *pre_text, action_t *action, gboolean details)
+{
+	const char *node_uname = NULL;
+	const char *node_uuid = NULL;
+	
+	if(action == NULL) {
+
+		crm_log_maybe(log_level, "%s%s: <NULL>",
+			      pre_text==NULL?"":pre_text,
+			      pre_text==NULL?"":": ");
+		return;
+	}
+
+
+	if(action->pseudo) {
+		node_uname = NULL;
+		node_uuid = NULL;
+		
+	} else if(action->node != NULL) {
+		node_uname = action->node->details->uname;
+		node_uuid = action->node->details->id;
+	} else {
+		node_uname = "<none>";
+		node_uuid = NULL;
+	}
+	
+	switch(text2task(action->task)) {
+		case stonith_node:
+		case shutdown_crm:
+			crm_log_maybe(log_level,
+				      "%s%s%sAction %d: %s%s%s%s%s%s",
+				      pre_text==NULL?"":pre_text,
+				      pre_text==NULL?"":": ",
+				      action->pseudo?"Pseduo ":action->optional?"Optional ":action->runnable?action->processed?"":"(Provisional) ":"!!Non-Startable!! ",
+				      action->id, action->uuid,
+				      node_uname?"\ton ":"",
+				      node_uname?node_uname:"",
+				      node_uuid?"\t\t(":"",
+				      node_uuid?node_uuid:"",
+				      node_uuid?")":"");
+			break;
+		default:
+			crm_log_maybe(log_level,
+				      "%s%s%sAction %d: %s %s%s%s%s%s%s",
+				      pre_text==NULL?"":pre_text,
+				      pre_text==NULL?"":": ",
+				      action->optional?"Optional ":action->pseudo?"Pseduo ":action->runnable?action->processed?"":"(Provisional) ":"!!Non-Startable!! ",
+				      action->id, action->uuid,
+				      safe_val3("<none>", action, rsc, id),
+				      node_uname?"\ton ":"",
+				      node_uname?node_uname:"",
+				      node_uuid?"\t\t(":"",
+				      node_uuid?node_uuid:"",
+				      node_uuid?")":"");
+			
+			break;
+	}
+
+	if(details) {
+		crm_log_maybe(log_level+1, "\t\t====== Preceeding Actions");
+		slist_iter(
+			other, action_wrapper_t, action->actions_before, lpc,
+			log_action(log_level+1, "\t\t", other->action, FALSE);
+			);
+#if 1
+		crm_log_maybe(log_level+1, "\t\t====== Subsequent Actions");
+		slist_iter(
+			other, action_wrapper_t, action->actions_after, lpc,
+			log_action(log_level+1, "\t\t", other->action, FALSE);
+			);		
+#endif
+		crm_log_maybe(log_level+1, "\t\t====== End");
+
+	} else {
+		crm_log_maybe(log_level, "\t\t(seen=%d, before=%d, after=%d)",
+			      action->seen_count,
+			      g_list_length(action->actions_before),
+			      g_list_length(action->actions_after));
+	}
+}
+
+
 void
 pe_free_action(action_t *action) 
 {
@@ -933,7 +915,6 @@ pe_free_action(action_t *action)
 	pe_free_shallow(action->actions_after); /* action_warpper_t* */
 	g_hash_table_destroy(action->extra);
 	g_hash_table_destroy(action->meta);
-	crm_free(action->task);
 	crm_free(action->uuid);
 	crm_free(action);
 }
@@ -950,8 +931,6 @@ find_recurring_actions(GListPtr input, node_t *not_on_node)
 		value = g_hash_table_lookup(action->meta, XML_LRM_ATTR_INTERVAL);
 		if(value == NULL) {
 			/* skip */
-		} else if(safe_str_eq(value, "0")) {
-			/* skip */
 		} else if(safe_str_eq(CRMD_ACTION_CANCEL, action->task)) {
 			/* skip */
 		} else if(not_on_node == NULL) {
@@ -1082,16 +1061,8 @@ static void
 resource_node_score(resource_t *rsc, node_t *node, int score, const char *tag) 
 {
 	node_t *match = NULL;
-
-	if(rsc->children) {
-	    slist_iter(
-		child_rsc, resource_t, rsc->children, lpc,
-		resource_node_score(child_rsc, node, score, tag);
-		);
-	}
-	
-	crm_debug_2("Setting %s for %s on %s: %d",
-		    tag, rsc->id, node->details->uname, score);
+	crm_debug("Setting %s for %s on %s: %d",
+		  tag, rsc->id, node->details->uname, score);
 	match = pe_find_node_id(rsc->allowed_nodes, node->details->id);
 	if(match == NULL) {
 		match = node_copy(node);
@@ -1105,6 +1076,8 @@ void
 resource_location(resource_t *rsc, node_t *node, int score, const char *tag,
 		  pe_working_set_t *data_set) 
 {
+	CRM_CHECK(rsc->variant == pe_native, return);
+	
 	if(node != NULL) {
 		resource_node_score(rsc, node, score, tag);
 
@@ -1119,16 +1092,66 @@ resource_location(resource_t *rsc, node_t *node, int score, const char *tag,
 			resource_node_score(rsc, node, score, tag);
 			);
 	}
+}
 
-	if(node == NULL && score == -INFINITY) {
-		if(rsc->allocated_to) {
-			crm_info("Deallocating %s from %s", rsc->id, rsc->allocated_to->details->uname);
-			crm_free(rsc->allocated_to);
-			rsc->allocated_to = NULL;
+void
+order_actions(
+	action_t *lh_action, action_t *rh_action, enum pe_ordering order) 
+{
+	action_wrapper_t *wrapper = NULL;
+	GListPtr list = NULL;
+	
+	crm_debug_2("Ordering Action %s before %s",
+		  lh_action->uuid, rh_action->uuid);
+
+	log_action(LOG_DEBUG_4, "LH (order_actions)", lh_action, FALSE);
+	log_action(LOG_DEBUG_4, "RH (order_actions)", rh_action, FALSE);
+
+	
+	crm_malloc0(wrapper, sizeof(action_wrapper_t));
+	if(wrapper != NULL) {
+		wrapper->action = rh_action;
+		wrapper->type = order;
+		
+		list = lh_action->actions_after;
+		list = g_list_append(list, wrapper);
+		lh_action->actions_after = list;
+		wrapper = NULL;
+	}
+	if(order != pe_ordering_recover) {
+		crm_malloc0(wrapper, sizeof(action_wrapper_t));
+		if(wrapper != NULL) {
+			wrapper->action = lh_action;
+			wrapper->type = order;
+			list = rh_action->actions_before;
+			list = g_list_append(list, wrapper);
+			rh_action->actions_before = list;
 		}
 	}
 }
 
+const char *
+get_interval(crm_data_t *xml_op) 
+{
+	const char *interval_s = NULL;
+        interval_s  = crm_element_value(xml_op, XML_LRM_ATTR_INTERVAL);
+#if CRM_DEPRECATED_SINCE_2_0_4
+	if(interval_s == NULL) {
+		crm_data_t *params = NULL;
+		params = find_xml_node(xml_op, XML_TAG_PARAMS, FALSE);
+		if(params != NULL) {
+			interval_s = crm_element_value(
+				params, XML_LRM_ATTR_INTERVAL);
+		}
+	}
+#endif
+	
+	CRM_CHECK(interval_s != NULL,
+		  crm_err("Invalid rsc op: %s", ID(xml_op)); return "0");
+	
+	return interval_s;
+}
+
 #define sort_return(an_int) crm_free(a_uuid); crm_free(b_uuid); return an_int
 
 gint
@@ -1144,8 +1167,6 @@ sort_op_by_callid(gconstpointer a, gconstpointer b)
 
 	const char *a_xml_id = ID(a);
 	const char *b_xml_id = ID(b);
-
-	int dummy = -1;
 	
 	int a_id = -1;
 	int b_id = -1;
@@ -1196,11 +1217,9 @@ sort_op_by_callid(gconstpointer a, gconstpointer b)
 	/* now process pending ops */
 	CRM_CHECK(a_key != NULL && b_key != NULL, sort_return(0));
 	CRM_CHECK(decode_transition_magic(
-		      a_key, &a_uuid, &a_id, &dummy, &a_status, &a_rc, &dummy),
-		  sort_return(0));
+			       a_key,&a_uuid,&a_id,&a_status, &a_rc), sort_return(0));
 	CRM_CHECK(decode_transition_magic(
-		      b_key, &b_uuid, &b_id, &dummy, &b_status, &b_rc, &dummy),
-		  sort_return(0));
+			       b_key,&b_uuid,&b_id,&b_status, &b_rc), sort_return(0));
 
 	/* try and determin the relative age of the operation...
 	 * some pending operations (ie. a start) may have been supuerceeded
