@@ -1,3 +1,4 @@
+/* $Id: graph.c,v 1.15 2006/05/29 13:22:48 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
@@ -251,7 +252,7 @@ run_graph(crm_graph_t *graph)
 	}
 
 	crm_debug_2("Entering graph %d callback", graph->id);
-
+	
 	slist_iter(
 		synapse, synapse_t, graph->synapses, lpc,
 		if (synapse->confirmed) {
@@ -263,11 +264,9 @@ run_graph(crm_graph_t *graph)
 			if(synapse->priority > graph->abort_priority) {
 				pending_log = LOG_DEBUG_3;
 			} 
-			do_crm_log(pending_log,
+			crm_log_maybe(pending_log,
 				      "Synapse %d: confirmation pending",
 				      synapse->id);
-
-			/* Is this running approximation good enough for batch_limit? */
 			num_pending++;
 			
 		} else if(synapse->priority < graph->abort_priority) {
@@ -276,18 +275,14 @@ run_graph(crm_graph_t *graph)
 			num_skipped++;
 			
 		} else {
+			crm_debug_2("Synapse %d pending", synapse->id);
 			if(should_fire_synapse(synapse)) {
-				crm_debug_2("Synapse %d fired", synapse->id);
 				num_fired++;
 				CRM_CHECK(fire_synapse(graph, synapse),
 					  stat_log_level = LOG_ERR;
 					  graph->abort_priority = INFINITY;
 					  num_incomplete++;
 					  num_fired--);
-
-				if (synapse->confirmed == FALSE) {
-				    num_pending++;
-				}
 				
 			} else {
 				crm_debug_2("Synapse %d cannot fire",
@@ -295,12 +290,6 @@ run_graph(crm_graph_t *graph)
 				num_incomplete++;
 			}
 		}
-		
-		if(graph->batch_limit > 0 && num_pending > graph->batch_limit) {
-		    crm_info("Throttling output: batch limit (%d) reached",
-			     graph->batch_limit);
-		    break;
-		}
 		);
 
 	if(num_pending == 0 && num_fired == 0) {
@@ -321,9 +310,9 @@ run_graph(crm_graph_t *graph)
 	}
 	
 	
-	do_crm_log(stat_log_level+1,
+	crm_log_maybe(stat_log_level+1,
 		      "====================================================");
-	do_crm_log(stat_log_level,
+	crm_log_maybe(stat_log_level,
 		      "Transition %d: (Complete=%d, Pending=%d,"
 		      " Fired=%d, Skipped=%d, Incomplete=%d)",
 		      graph->id, num_complete, num_pending, num_fired,
