@@ -1,3 +1,4 @@
+/* $Id: utils.c,v 1.10 2006/05/08 09:59:42 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
@@ -31,7 +32,7 @@ extern crm_graph_functions_t *graph_fns;
 static gboolean
 pseudo_action_dummy(crm_graph_t *graph, crm_action_t *action) 
 {
-	crm_debug_2("Dummy event handler: action %d executed", action->id);
+	crm_debug("Dummy event handler: action %d executed", action->id);
 	action->confirmed = TRUE;
 	update_graph(graph, action);
 	return TRUE;
@@ -137,20 +138,20 @@ print_elem(int log_level, const char *prefix, gboolean as_input, crm_action_t *a
 	
 	switch(action->type) {
 		case action_type_pseudo:
-			do_crm_log(log_level,
+			crm_log_maybe(log_level,
 				      "%s[%s %d]: %s (id: %s, type: %s, priority: %d)",
 				      prefix, class, action->id, state, key,
 				      actiontype2text(action->type),
 				      priority);
 			break;
 		case action_type_rsc:
-			do_crm_log(log_level,
+			crm_log_maybe(log_level,
 				      "%s[%s %d]: %s (id: %s, loc: %s, priority: %d)",
 				      prefix, class, action->id, state, key, host,
 				      priority);
 			break;
 		case action_type_crm:	
-			do_crm_log(log_level,
+			crm_log_maybe(log_level,
 				      "%s[%s %d]: %s (id: %s, loc: %s, type: %s, priority: %d)",
 				      prefix, class, action->id, state, key, host,
 				      actiontype2text(action->type),
@@ -168,7 +169,7 @@ print_elem(int log_level, const char *prefix, gboolean as_input, crm_action_t *a
 	}
 	
 	if(action->timer) {
-		do_crm_log(log_level, "%s\ttimeout=%d, timer=%d", prefix,
+		crm_log_maybe(log_level, "%s\ttimeout=%d, timer=%d", prefix,
 			      action->timeout, action->timer->source_id);
 	}
 	
@@ -192,16 +193,11 @@ print_graph(unsigned int log_level, crm_graph_t *graph)
 		}
 		return;
 	}
-		
-	do_crm_log(log_level, "Graph %d (%d actions in %d synapses):"
-		   " batch-limit=%d jobs, network-delay=%dms",
-		   graph->id, graph->num_actions, graph->num_synapses,
-		   graph->batch_limit, graph->network_delay);
-	
+
 	slist_iter(
 		synapse, synapse_t, graph->synapses, lpc,
 
-		do_crm_log(log_level, "Synapse %d %s (priority: %d)",
+		crm_log_maybe(log_level, "Synapse %d %s (priority: %d)",
 			      synapse->id,
 			      synapse->confirmed?"was confirmed":
 			        synapse->executed?"was executed":
@@ -237,7 +233,7 @@ update_abort_priority(
 		graph->abort_priority = priority;
 		crm_info("Abort priority upgraded to %d", priority);
 		if(graph->abort_reason != NULL) {
-			crm_info("'%s' abort superceeded",
+			crm_info("'%s'-class abort superceeded",
 				 graph->abort_reason);
 		}
 		graph->abort_reason = abort_reason;
