@@ -1,3 +1,4 @@
+/* $Id: api_test.c,v 1.8 2005/04/06 18:07:53 gshi Exp $ */
 /* 
  * api_test: Test program for testing the heartbeat API
  *
@@ -18,7 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
