@@ -1,3 +1,4 @@
+/* $Id: recmgr.c,v 1.6 2004/11/08 09:07:51 sunjd Exp $ */
 /*
  * recmgr.c: Recovery manager client plug-in implementation
  * 
@@ -40,7 +41,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
