@@ -1,3 +1,4 @@
+/* $Id: ping.c,v 1.45 2005/09/22 16:52:06 alan Exp $ */
 /*
  * ping.c: ICMP-echo-based heartbeat code for heartbeat.
  *
@@ -28,7 +29,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -71,7 +72,6 @@
 #include <arpa/inet.h>
 
 #include <netdb.h>
-#include <clplumbing/uids.h>
 #include <heartbeat.h>
 #include <HBcomm.h>
 
@@ -248,7 +248,6 @@ ping_new(const char * host)
 		FREE(ipi); ipi = NULL;
 		return(NULL);
 	}
-	memset(ret, 0, sizeof(*ret));
 
 	ret->pd = (void*)ipi;
 	name = STRDUP(host);
@@ -280,7 +279,6 @@ ping_close(struct hb_media* mp)
 		if (close(ei->sock) < 0) {
 			rc = HA_FAIL;
 		}
-		ei->sock = -1;
 	}
 	return(rc);
 }
@@ -412,7 +410,6 @@ ping_write(struct hb_media* mp, void *p, int len)
 	const char *		ts;
 	struct ha_msg *		nmsg;
 	struct ha_msg *		msg;
-	static gboolean		needroot = FALSE;
 	
 	
 	msg = wirefmt2msg(p, len, MSG_NEEDAUTH);
@@ -477,7 +474,7 @@ ping_write(struct hb_media* mp, void *p, int len)
 
 	if ((icmp_pkt = MALLOC(pktsize)) == NULL) {
 		PILCallLog(LOG, PIL_CRIT, "out of memory");
-		cl_free(pkt);
+		ha_free(pkt);
 		ha_msg_del(msg);
 		return HA_FAIL;
 	}
@@ -491,36 +488,19 @@ ping_write(struct hb_media* mp, void *p, int len)
 	++ei->iseq;
 
 	memcpy(icp->icmp_data, pkt, size);
-	cl_free(pkt); pkt = NULL;
+	ha_free(pkt); pkt = NULL;
 
 	/* Compute the ICMP checksum */
 	icp->icmp_cksum = in_cksum((u_short *)icp, pktsize);
 
-retry:
-	if (needroot) {
-		return_to_orig_privs();
-	}
-
-	if ((rc=sendto(ei->sock, (void *) icmp_pkt, pktsize, MSG_DONTWAIT
+	if ((rc=sendto(ei->sock, (void *) icmp_pkt, pktsize, 0
 	,	(struct sockaddr *)&ei->addr
 	,	sizeof(struct sockaddr))) != (ssize_t)pktsize) {
-		if (errno == EPERM && !needroot) {
-			needroot=TRUE;
-			goto retry;
-		}
-		if (!mp->suppresserrs) {
-			PILCallLog(LOG, PIL_CRIT, "Error sending packet: %s", strerror(errno));
-			PILCallLog(LOG, PIL_INFO, "euid=%lu egid=%lu"
-			,	(unsigned long) geteuid()
-			,	(unsigned long) getegid());
-		}
+		PILCallLog(LOG, PIL_CRIT, "Error sending packet: %s", strerror(errno));
 		FREE(icmp_pkt);
 		ha_msg_del(msg);
 		return(HA_FAIL);
 	}
-	if (needroot) {
-		return_to_dropped_privs();
-	}
 
 	if (DEBUGPKT) {
 		PILCallLog(LOG, PIL_DEBUG, "sent %d bytes to %s"
