@@ -1,3 +1,4 @@
+/* $Id: ping_group.c,v 1.20 2005/08/15 21:12:16 gshi Exp $ */
 /*
  * ping_group.c: ICMP-echo-based heartbeat code for heartbeat.
  *
@@ -34,7 +35,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -79,7 +80,6 @@
 #include <heartbeat.h>
 #include <HBcomm.h>
 #include <clplumbing/realtime.h>
-#include <clplumbing/uids.h>
 
 #ifdef linux
 #	define	ICMP_HDR_SZ	sizeof(struct icmphdr)	/* 8 */
@@ -287,11 +287,10 @@ ping_group_new(const char *name)
 	priv->ident = getpid() & 0xFFFF;
 
 	media = (struct hb_media *) MALLOC(sizeof(struct hb_media));
-	if (!media) {
+	if(!media) {
 		FREE(priv);
 		return(NULL);
 	}
-	memset(media, 0, sizeof(*media));
 
 	media->pd = (void*)priv;
 	tmp = STRDUP(name);
@@ -366,7 +365,6 @@ ping_group_close(struct hb_media* mp)
 		if (close(ei->sock) < 0) {
 			rc = HA_FAIL;
 		}
-		ei->sock=-1;
 	}
 
 	ping_group_destroy_data(mp);
@@ -524,7 +522,6 @@ ping_group_write(struct hb_media* mp, void *p, int len)
 	struct ha_msg *		nmsg;
 	ping_group_node_t *	node;
 	struct ha_msg*		msg;
-	static gboolean		needroot=FALSE;			
 	
 	PINGGROUPASSERT(mp);
 	
@@ -591,7 +588,7 @@ ping_group_write(struct hb_media* mp, void *p, int len)
 
 	if ((icmp_pkt = MALLOC(pktsize)) == NULL) {
 		PILCallLog(LOG, PIL_CRIT, "out of memory");
-		cl_free(pkt);
+		ha_free(pkt);
 		ha_msg_del(msg);
 		return HA_FAIL;
 	}
@@ -605,32 +602,17 @@ ping_group_write(struct hb_media* mp, void *p, int len)
 	++ei->iseq;
 
 	memcpy(icp->icmp_data, pkt, size);
-	cl_free(pkt); pkt = NULL;
+	ha_free(pkt); pkt = NULL;
 
 	/* Compute the ICMP checksum */
 	icp->icmp_cksum = in_cksum((u_short *)icp, pktsize);
 
-retry:
-	if (needroot) {
-		return_to_orig_privs();
-	}
-
 	for(node = ei->node; node; node = node->next) {
-		if ((rc=sendto(ei->sock, (void *) icmp_pkt, pktsize
-		,	MSG_DONTWAIT
+		if ((rc=sendto(ei->sock, (void *) icmp_pkt, pktsize, 0
 		,	(struct sockaddr *)&node->addr
 		,	sizeof(struct sockaddr))) != (ssize_t)pktsize) {
-			if (errno == EPERM && !needroot) {
-				needroot=TRUE;
-				goto retry;
-			}
-			if (!mp->suppresserrs) {
-				PILCallLog(LOG, PIL_CRIT, "Error sending packet: %s"
-				,	strerror(errno));
-				PILCallLog(LOG, PIL_INFO, "euid=%lu egid=%lu"
-				,	(unsigned long) geteuid()
-				,	(unsigned long) getegid());
-			}
+			PILCallLog(LOG, PIL_CRIT, "Error sending packet: %s"
+			,	strerror(errno));
 			FREE(icmp_pkt);
 			ha_msg_del(msg);
 			return(HA_FAIL);
@@ -643,9 +625,6 @@ retry:
 
 		cl_shortsleep();
 	}
-	if (needroot) {
-		return_to_dropped_privs();
-	}
 
 	if (DEBUGPKTCONT) {
 		PILCallLog(LOG, PIL_DEBUG, "%s"
