@@ -1,3 +1,4 @@
+/* $Id: baytech.c,v 1.28 2005/07/03 22:15:50 alan Exp $ */
 /*
  *	Stonith module for BayTech Remote Power Controllers (RPC-x devices)
  *
@@ -19,7 +20,6 @@
  *
  */
 
-#include <lha_internal.h>
 #define	DEVICE	"BayTech power switch"
 
 #define DOESNT_USE_STONITHKILLCOMM	1
@@ -873,7 +873,7 @@ baytech_destroy(StonithPlugin *s)
 static StonithPlugin *
 baytech_new(const char *subplugin)
 {
-	struct pluginDevice*	bt = ST_MALLOCT(struct pluginDevice);
+	struct pluginDevice*	bt = MALLOCT(struct pluginDevice);
 
 	if (bt == NULL) {
 		LOG(PIL_CRIT, "out of memory");
