@@ -613,7 +613,7 @@ ibmhmc_set_config(StonithPlugin * s, StonithNVpair* list)
 		mansyspats = OurImports->GetValue(list, ST_MANSYSPAT);
 		if (mansyspats != NULL) {
 			if (get_hmc_mansyspats(dev, mansyspats) != S_OK) {
-				FREE(namestocopy[1].s_value);
+				FREE(namestocopy[0].s_value);
 				return S_OOPS;
 			}
 		}
@@ -624,7 +624,7 @@ ibmhmc_set_config(StonithPlugin * s, StonithNVpair* list)
 		/* -p or -F option with args "ipaddr [managedsyspat]..." */
 		char *pch = namestocopy[0].s_value;
 
-		/* skip over password and null-terminate */
+		/* skip over ipaddr and null-terminate */
 		pch += strcspn(pch, WHITESPACE);
 		*pch = EOS;
 
@@ -724,10 +724,18 @@ ibmhmc_getinfo(StonithPlugin* s, int reqtype)
 			ret = "IBM Hardware Management Console (HMC)\n"
 			"Use for IBM i5, p5, pSeries and OpenPower systems "
 			"managed by HMC\n"
-			"Optional parameter name " ST_MANSYSPAT " is "
-			"white-space delimited list of patterns used to match "
-			"managed system names; if last character is '*', all "
-			"names that begin with the pattern are matched\n";
+			"Optional parameter name " ST_MANSYSPAT " is \n"
+			"white-space delimited list of patterns used to match\n"
+			"managed system names; if last character is '*', all\n"
+			"names that begin with the pattern are matched.\n"
+			"Optional parameter name " ST_PASSWD " is password\n"
+			"for " HMCROOT " if passwordless ssh access to HMC\n "
+			"has NOT been setup (to do so, it is necessary to\n "
+			"create a public/private key pair with empty\n "
+			"passphrase - see \"Configure the OpenSSH Client\" in\n"
+			"the redbook at " HMCURL " for more details); note\n"
+			"that passwordless ssh access to HMC must be setup\n "
+			"for R1 clusters.";
 			break;
 
 		case ST_DEVICEURL:
