@@ -1,4 +1,4 @@
-/* $Id: ipmilan_command.c,v 1.10 2005/08/03 14:26:32 horms Exp $ */
+/* $Id: ipmilan_command.c,v 1.12 2006/04/24 10:10:04 lars Exp $ */
 /*
  * This program is largely based on the ipmicmd.c program that's part of OpenIPMI package.
  * 
@@ -97,18 +97,15 @@ dump_msg_data(ipmi_msg_t *msg, ipmi_addr_t *addr, char *type)
 	if (addr->addr_type == IPMI_SYSTEM_INTERFACE_ADDR_TYPE) {
 		smi_addr = (struct ipmi_system_interface_addr *) addr;
 
-	} else if ((addr->addr_type == IPMI_IPMB_ADDR_TYPE) 
-			|| (addr->addr_type == IPMI_IPMB_BROADCAST_ADDR_TYPE)) {
-		ipmb_addr = (struct ipmi_ipmb_addr *) addr;
-	}
-
-	if (smi_addr) {
 		fprintf(stderr, "%2.2x %2.2x %2.2x %2.2x ", 
 			addr->channel,
 			msg->netfn,
 			smi_addr->lun,
 			msg->cmd);
-	} else {
+	} else if ((addr->addr_type == IPMI_IPMB_ADDR_TYPE) 
+			|| (addr->addr_type == IPMI_IPMB_BROADCAST_ADDR_TYPE)) {
+		ipmb_addr = (struct ipmi_ipmb_addr *) addr;
+		
 		fprintf(stderr, "%2.2x %2.2x %2.2x %2.2x ", 
 			addr->channel,
 			msg->netfn,
@@ -355,46 +352,3 @@ do_ipmi_cmd(struct ipmilanHostInfo * host, int request)
 	return setup_ipmi_conn(host, request);
 }
 
-void
-posix_vlog(char *format, enum ipmi_log_type_e log_type, va_list ap)
-{
-    int do_nl = 1; 
-
-    switch(log_type)
-    {
-        case IPMI_LOG_INFO:
-            PILCallLog(PluginImports->log,PIL_INFO, "INFO: ");
-            break;
-                                                                                                                                                             
-        case IPMI_LOG_WARNING:
-            PILCallLog(PluginImports->log,PIL_INFO, "WARN: ");
-            break;
-                                                                                                                                                             
-        case IPMI_LOG_SEVERE:
-            PILCallLog(PluginImports->log,PIL_INFO, "SEVR: ");
-            break;
-                                                                                                                                                             
-        case IPMI_LOG_FATAL:
-            PILCallLog(PluginImports->log,PIL_INFO, "FATL: ");
-            break;
-                                                                                                                                                             
-        case IPMI_LOG_ERR_INFO:
-            PILCallLog(PluginImports->log,PIL_INFO, "EINF: ");
-            break;
-                                                                                                                                                             
-        case IPMI_LOG_DEBUG_START:
-            do_nl = 0;
-            /* FALLTHROUGH */
-        case IPMI_LOG_DEBUG:
-            PILCallLog(PluginImports->log,PIL_INFO, "DEBG: ");
-            break;
-                                                                                                                                                             
-        case IPMI_LOG_DEBUG_CONT:
-            do_nl = 0;
-            /* FALLTHROUGH */
-        case IPMI_LOG_DEBUG_END:
-            break;
-    }
-                                                                                                                                                             
-}
-
