@@ -1,4 +1,4 @@
-/* $Id: nw_rpc100s.c,v 1.10.2.1 2004/04/20 08:50:06 alan Exp $ */
+/* $Id: nw_rpc100s.c,v 1.24 2005/04/22 12:23:05 blaschke Exp $ */
 /*
  *	Stonith module for Night/Ware RPC100S 
  *
@@ -9,6 +9,8 @@
  *	Copyright (c) 2000 Computer Generation Incorporated
  *               Eric Z. Ayers <eric.ayers@compgen.com>
  *
+ *      Mangled by Zhaokai <zhaokai@cn.ibm.com>, IBM, 2005
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -25,91 +27,48 @@
  *
  */
 
-#include <portability.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <errno.h>
-#include <syslog.h>
-#include <libintl.h>
-#include <sys/wait.h>
-#include <termios.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <glib.h>
+#define	DEVICE	"NW RPC100S Power Switch"
+#include "stonith_plugin_common.h"
 
-#include <stonith/stonith.h>
-#define PIL_PLUGINTYPE          STONITH_TYPE
-#define PIL_PLUGINTYPE_S        STONITH_TYPE_S
 #define PIL_PLUGIN              nw_rpc100s
 #define PIL_PLUGIN_S            "nw_rpc100s"
 #define PIL_PLUGINLICENSE 	LICENSE_LGPL
 #define PIL_PLUGINLICENSEURL 	URL_LGPL
+#define MAX_CFGLINE		256
 #include <pils/plugin.h>
 
-/*
- * nw_rpc100sclose is called as part of unloading the nw_rpc100s STONITH plugin.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the plugin, and not a single interface
- * in particular, here's our chance to clean it up.
- */
-
-static void
-nw_rpc100sclosepi(PILPlugin*pi)
-{
-}
-
-
-/*
- * nw_rpc100scloseintf called as part of shutting down the nw_rpc100s STONITH
- * interface.  If there was any global data allocated, or file descriptors
- * opened, etc.  which is associated with the nw_rpc100s implementation,
- * here's our chance to clean it up.
- */
-static PIL_rc
-nw_rpc100scloseintf(PILInterface* pi, void* pd)
-{
-	return PIL_OK;
-}
-
-static void *		nw_rpc100s_new(void);
-static void		nw_rpc100s_destroy(Stonith *);
-static int		nw_rpc100s_set_config_file(Stonith *, const char * cfgname);
-static int		nw_rpc100s_set_config_info(Stonith *, const char * info);
-static const char *	nw_rpc100s_getinfo(Stonith * s, int InfoType);
-static int		nw_rpc100s_status(Stonith * );
-static int		nw_rpc100s_reset_req(Stonith * s, int request, const char * host);
-static char **		nw_rpc100s_hostlist(Stonith  *);
-static void		nw_rpc100s_free_hostlist(char **);
+static StonithPlugin *	nw_rpc100s_new(const char *);
+static void		nw_rpc100s_destroy(StonithPlugin *);
+static int		nw_rpc100s_set_config(StonithPlugin *, StonithNVpair *);
+static const char**	nw_rpc100s_get_confignames(StonithPlugin *);
+static const char *	nw_rpc100s_getinfo(StonithPlugin * s, int InfoType);
+static int		nw_rpc100s_status(StonithPlugin * );
+static int		nw_rpc100s_reset_req(StonithPlugin * s, int request, const char * host);
+static char **		nw_rpc100s_hostlist(StonithPlugin  *);
 
 static struct stonith_ops nw_rpc100sOps ={
-	nw_rpc100s_new,		/* Create new STONITH object	*/
-	nw_rpc100s_destroy,		/* Destroy STONITH object	*/
-	nw_rpc100s_set_config_file,	/* set configuration from file	*/
-	nw_rpc100s_set_config_info,	/* Get configuration from file	*/
-	nw_rpc100s_getinfo,		/* Return STONITH info string	*/
-	nw_rpc100s_status,		/* Return STONITH device status	*/
-	nw_rpc100s_reset_req,		/* Request a reset */
-	nw_rpc100s_hostlist,		/* Return list of supported hosts */
-	nw_rpc100s_free_hostlist	/* free above list */
+	nw_rpc100s_new,		/* Create new STONITH object		*/
+	nw_rpc100s_destroy,	/* Destroy STONITH object		*/
+	nw_rpc100s_getinfo,	/* Return STONITH info string		*/
+	nw_rpc100s_get_confignames,/* Return STONITH info string	*/
+	nw_rpc100s_set_config,	/* Get configuration from NVpairs	*/
+	nw_rpc100s_status,	/* Return STONITH device status		*/
+	nw_rpc100s_reset_req,	/* Request a reset 			*/
+	nw_rpc100s_hostlist,	/* Return list of supported hosts	*/
 };
 
-PIL_PLUGIN_BOILERPLATE("1.0", Debug, nw_rpc100sclosepi);
+PIL_PLUGIN_BOILERPLATE2("1.0", Debug)
 static const PILPluginImports*  PluginImports;
 static PILPlugin*               OurPlugin;
 static PILInterface*		OurInterface;
 static StonithImports*		OurImports;
 static void*			interfprivate;
 
-#define LOG		PluginImports->log
-#define MALLOC		PluginImports->alloc
-#define STRDUP  	PluginImports->mstrdup
-#define FREE		PluginImports->mfree
-#define EXPECT_TOK	OurImports->ExpectToken
-#define STARTPROC	OurImports->StartProcess
+#include "stonith_signal.h"
+
+#define DOESNT_USE_STONITHKILLCOMM
+#define DOESNT_USE_STONITHSCANLINE
+#include "stonith_expect_helpers.h"
 
 PIL_rc
 PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports);
@@ -130,17 +89,12 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
  	return imports->register_interface(us, PIL_PLUGINTYPE_S
 	,	PIL_PLUGIN_S
 	,	&nw_rpc100sOps
-	,	nw_rpc100scloseintf		/*close */
+	,	NULL		/*close */
 	,	&OurInterface
 	,	(void*)&OurImports
 	,	&interfprivate); 
 }
 
-#define	DEVICE	"NW RPC100S Power Switch"
-
-#define N_(text)	(text)
-#define _(text)		dgettext(ST_TEXTDOMAIN, text)
-
 /*
    The Nightware RPS-100S is manufactured by:
    
@@ -192,18 +146,13 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
 
 
 
-struct NW_RPC100S {
-	const char *	RPCid;
-
-	char *	idinfo;  /* ??? What's this for Alan ??? */
-	char *	unitid;  /* ??? What's this for Alan ??? */
+struct pluginDevice {
+	StonithPlugin   sp;
+	const char *	pluginid;
+	const char *	idinfo;
 
 	int	fd;      /* FD open to the serial port */
 
-	int	config;  /* 0 if not configured, 
-				    1 if configured with nw_rpc100s_set_config_file() 
-				    or nw_rpc100s_set_config_info()
-				 */
 	char *	device;  /* Serial device name to use to communicate 
                             to this RPS10
 			 */
@@ -213,35 +162,16 @@ struct NW_RPC100S {
 };
 
 /* This string is used to identify this type of object in the config file */
-static const char * RPCid = "NW_RPC100S";
-static const char * NOTrpcid = "OBJECT DESTROYED: (NW RPC100S)";
+static const char * pluginid = "NW_RPC100S";
+static const char * NOTrpcid = "NW RPC100S device has been destroyed";
 
-#ifndef DEBUG
-#define DEBUG 0
-#endif
-static int gbl_debug = DEBUG;
+#include "stonith_config_xml.h"
 
-#define	ISNWRPC100S(i)	(((i)!= NULL && (i)->pinfo != NULL)	\
-	&& ((struct NW_RPC100S *)(i->pinfo))->RPCid == RPCid)
-
-#define	ISCONFIGED(i)	(ISNWRPC100S(i) && ((struct NW_RPC100S *)(i->pinfo))->config)
-
-#ifndef MALLOCT
-#	define     MALLOCT(t)      ((t *)(MALLOC(sizeof(t)))) 
-#endif
-
-#define WHITESPACE	" \t\n\r\f"
-
-#define	REPLSTR(s,v)	{					\
-			if ((s) != NULL) {			\
-				FREE(s);			\
-				(s)=NULL;			\
-			}					\
-			(s) = STRDUP(v);			\
-			if ((s) == NULL) {			\
-				syslog(LOG_ERR, _("out of memory"));\
-			}					\
-			}
+static const char *nw_rpc100sXML = 
+  XML_PARAMETERS_BEGIN
+    XML_TTYDEV_PARM
+    XML_HOSTLIST_PARM
+  XML_PARAMETERS_END;
 
 /*
  *	Different expect strings that we get from the NW_RPC100S
@@ -253,74 +183,33 @@ static struct Etoken NWtokInvalidEntry[] = { {"Invalid Entry", 0, 0}, {NULL,0,0}
 /* Accept either a CR/NL or an NL/CR */
 static struct Etoken NWtokCRNL[] =	{ {"\n\r",0,0},{"\r\n",0,0},{NULL,0,0}};
 
-static int	RPCConnect(struct NW_RPC100S * ctx);
-static int	RPCDisconnect(struct NW_RPC100S * ctx);
+static int	RPCConnect(struct pluginDevice * ctx);
+static int	RPCDisconnect(struct pluginDevice * ctx);
 
-static int	RPCReset(struct NW_RPC100S*, int unitnum, const char * rebootid);
+static int	RPCReset(struct pluginDevice*, int unitnum, const char * rebootid);
 #if defined(ST_POWERON) 
-static int	RPCOn(struct NW_RPC100S*, int unitnum, const char * rebootid);
+static int	RPCOn(struct pluginDevice*, int unitnum, const char * rebootid);
 #endif
 #if defined(ST_POWEROFF) 
-static int	RPCOff(struct NW_RPC100S*, int unitnum, const char * rebootid);
+static int	RPCOff(struct pluginDevice*, int unitnum, const char * rebootid);
 #endif
-static int	RPCNametoOutlet ( struct NW_RPC100S * ctx, const char * host );
+static int	RPCNametoOutlet ( struct pluginDevice * ctx, const char * host );
 
-static int RPC_parse_config_info(struct NW_RPC100S* ctx, const char * info);
+/*static int RPC_parse_config_info(struct pluginDevice* ctx, const char * info);*/
 
 
-/*
- *	We do these things a lot.  Here are a few shorthand macros.
- */
-
-#define	SEND(cmd, timeout)		{                       \
-	int return_val = RPCSendCommand(ctx, cmd, timeout);     \
-	if (return_val != S_OK)  return return_val;                     \
-}
-
-#define	EXPECT(p,t)	{						\
-			if (RPCLookFor(ctx, p, t) < 0)			\
-				return(errno == ETIMEDOUT		\
-			?	S_TIMEOUT : S_OOPS);			\
-			}
-
-#define	NULLEXPECT(p,t)	{						\
-				if (RPCLookFor(ctx, p, t) < 0)		\
-					return(NULL);			\
-			}
-
-#define	SNARF(s, to)	{						\
-				if (RPCScanLine(ctx,to,(s),sizeof(s))	\
-				!=	S_OK)				\
-					return(S_OOPS);			\
-			}
-
-#define	NULLSNARF(s, to)	{					\
-				if (RPCScanLine(ctx,to,(s),sizeof(s))	\
-				!=	S_OK)				\
-					return(NULL);			\
-				}
-
-/* Look for any of the given patterns.  We don't care which */
-
-static int
-RPCLookFor(struct NW_RPC100S* ctx, struct Etoken * tlist, int timeout)
-{
-	int	rc;
-	if ((rc = EXPECT_TOK(ctx->fd, tlist, timeout, NULL, 0)) < 0) {
-		syslog(LOG_ERR, _("Did not find string: '%s' from" DEVICE ".")
-		,	tlist[0].string);
-		RPCDisconnect(ctx);
-		return(-1);
+#define        SENDCMD(cmd, timeout)              {			\
+		int return_val = RPCSendCommand(ctx, cmd, timeout);     \
+		if (return_val != S_OK) {				\
+			return return_val;				\
+		}							\
 	}
-	return(rc);
-}
-
 
 /*
  * RPCSendCommand - send a command to the specified outlet
  */
 static int
-RPCSendCommand (struct NW_RPC100S *ctx, const char *command, int timeout)
+RPCSendCommand (struct pluginDevice *ctx, const char *command, int timeout)
 {
 	char            writebuf[64]; /* All commands are short.
 					 They should be WAY LESS
@@ -337,7 +226,9 @@ RPCSendCommand (struct NW_RPC100S *ctx, const char *command, int timeout)
 
 	snprintf (writebuf, sizeof(writebuf), "%s\r", command);
 
-	if (gbl_debug) printf ("Sending %s\n", writebuf);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Sending %s", writebuf);
+	}
 
 	/* Make sure the serial port won't block on us. use select()  */
 	FD_SET(ctx->fd, &wfds);
@@ -349,21 +240,21 @@ RPCSendCommand (struct NW_RPC100S *ctx, const char *command, int timeout)
 	return_val = select(ctx->fd+1, NULL, &wfds,&xfds, &tv);
 	if (return_val == 0) {
 		/* timeout waiting on serial port */
-		syslog (LOG_ERR, "%s: Timeout writing to %s",
-			RPCid, ctx->device);
+		LOG(PIL_CRIT, "%s: Timeout writing to %s"
+		,	pluginid, ctx->device);
 		return S_TIMEOUT;
 	} else if ((return_val == -1) || FD_ISSET(ctx->fd, &xfds)) {
 		/* an error occured */
-		syslog (LOG_ERR, "%s: Error before writing to %s: %s",
-			RPCid, ctx->device, strerror(errno));		
+		LOG(PIL_CRIT, "%s: Error before writing to %s: %s"
+		,	pluginid, ctx->device, strerror(errno));		
 		return S_OOPS;
 	}
 
 	/* send the command */
 	if (write(ctx->fd, writebuf, strlen(writebuf)) != 
 			(int)strlen(writebuf)) {
-		syslog (LOG_ERR, "%s: Error writing to  %s : %s",
-			RPCid, ctx->device, strerror(errno));
+		LOG(PIL_CRIT, "%s: Error writing to  %s : %s"
+		,	pluginid, ctx->device, strerror(errno));
 		return S_OOPS;
 	}
 
@@ -379,25 +270,31 @@ RPCSendCommand (struct NW_RPC100S *ctx, const char *command, int timeout)
  *
  */
 static int
-RPCReset(struct NW_RPC100S* ctx, int unitnum, const char * rebootid)
+RPCReset(struct pluginDevice* ctx, int unitnum, const char * rebootid)
 {
 
-	if (gbl_debug) printf ("Calling RPCReset (%s)\n", RPCid);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Calling RPCReset (%s)", pluginid);
+	}
 	
 	if (ctx->fd < 0) {
-		syslog(LOG_ERR, "%s: device %s is not open!", RPCid, 
-		       ctx->device);
+		LOG(PIL_CRIT, "%s: device %s is not open!", pluginid
+		,	ctx->device);
 		return S_OOPS;
 	}
 
 	/* send the "toggle power" command */
-	SEND("//0,0,10;\r\n", 12);
+	SENDCMD("//0,0,10;\r\n", 12);
 
 	/* Expect "OK" */
-	EXPECT(NWtokOK, 5);
-	if (gbl_debug) printf ("Got OK\n");
-	EXPECT(NWtokCRNL, 2);
-	if (gbl_debug) printf ("Got NL\n");
+	EXPECT(ctx->fd, NWtokOK, 5);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got OK");
+	}
+	EXPECT(ctx->fd, NWtokCRNL, 2);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got NL");
+	}
 	
 	return(S_OK);
 
@@ -409,21 +306,21 @@ RPCReset(struct NW_RPC100S* ctx, int unitnum, const char * rebootid)
  * RPCOn - Turn OFF the given outlet number 
  */
 static int
-RPCOn(struct NW_RPC100S* ctx, int unitnum, const char * host)
+RPCOn(struct pluginDevice* ctx, int unitnum, const char * host)
 {
 
 	if (ctx->fd < 0) {
-		syslog(LOG_ERR, "%s: device %s is not open!", RPCid, 
-		       ctx->device);
+		LOG(PIL_CRIT, "%s: device %s is not open!", pluginid
+		,	ctx->device);
 		return S_OOPS;
 	}
 
 	/* send the "On" command */
-	SEND("//0,0,0;\r\n", 10);
+	SENDCMD("//0,0,0;\r\n", 10);
 
 	/* Expect "OK" */
-	EXPECT(NWtokOK, 5);
-	EXPECT(NWtokCRNL, 2);
+	EXPECT(ctx->fd, NWtokOK, 5);
+	EXPECT(ctx->fd, NWtokCRNL, 2);
 
 	return(S_OK);
 
@@ -436,21 +333,21 @@ RPCOn(struct NW_RPC100S* ctx, int unitnum, const char * host)
  * RPCOff - Turn Off the given outlet number 
  */
 static int
-RPCOff(struct NW_RPC100S* ctx, int unitnum, const char * host)
+RPCOff(struct pluginDevice* ctx, int unitnum, const char * host)
 {
 
 	if (ctx->fd < 0) {
-		syslog(LOG_ERR, "%s: device %s is not open!", RPCid, 
-		       ctx->device);
+		LOG(PIL_CRIT, "%s: device %s is not open!", pluginid
+		,	ctx->device);
 		return S_OOPS;
 	}
 
 	/* send the "Off" command */
-	SEND("//0,0,*;\r\n", 10);
+	SENDCMD("//0,0,*;\r\n", 10);
 
 	/* Expect "OK" */
-	EXPECT(NWtokOK, 5);
-	EXPECT(NWtokCRNL, 2);
+	EXPECT(ctx->fd, NWtokOK, 5);
+	EXPECT(ctx->fd, NWtokCRNL, 2);
 
 	return(S_OK);
 
@@ -468,22 +365,17 @@ RPCOff(struct NW_RPC100S* ctx, int unitnum, const char * host)
  *    S_OK   - if the stonith device is reachable and online.
  */
 static int
-nw_rpc100s_status(Stonith  *s)
+nw_rpc100s_status(StonithPlugin  *s)
 {
-	struct NW_RPC100S*	ctx;
+	struct pluginDevice*	ctx;
 	
-	if (gbl_debug) printf ("Calling nw_rpc100s_status (%s)\n", RPCid);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Calling nw_rpc100s_status (%s)", pluginid);
+	}
 	
-	if (!ISNWRPC100S(s)) {
-		syslog(LOG_ERR, "invalid argument to RPC_status");
-		return(S_OOPS);
-	}
-	if (!ISCONFIGED(s)) {
-		syslog(LOG_ERR
-		,	"unconfigured stonith object in RPC_status");
-		return(S_OOPS);
-	}
-	ctx = (struct NW_RPC100S*) s->pinfo;
+	ERRIFNOTCONFIGED(s,S_OOPS);
+
+	ctx = (struct pluginDevice*) s;
 	if (RPCConnect(ctx) != S_OK) {
 		return(S_OOPS);
 	}
@@ -509,62 +401,29 @@ nw_rpc100s_status(Stonith  *s)
  *         of null-terminated malloc'ed strings.
  */
 static char **
-nw_rpc100s_hostlist(Stonith  *s)
+nw_rpc100s_hostlist(StonithPlugin  *s)
 {
 	char **		ret = NULL;	/* list to return */
-	struct NW_RPC100S*	ctx;
+	struct pluginDevice*	ctx;
 
-	if (gbl_debug) printf ("Calling nw_rpc100s_hostlist (%s)\n", RPCid);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Calling nw_rpc100s_hostlist (%s)", pluginid);
+	}
 	
-	if (!ISNWRPC100S(s)) {
-		syslog(LOG_ERR, "invalid argument to RPC_list_hosts");
-		return(NULL);
-	}
-	if (!ISCONFIGED(s)) {
-		syslog(LOG_ERR
-		,	"unconfigured stonith object in RPC_list_hosts");
-		return(NULL);
-	}
+	ERRIFNOTCONFIGED(s,NULL);
 
-	ctx = (struct NW_RPC100S*) s->pinfo;
+	ctx = (struct pluginDevice*) s;
 
-	ret = (char **)MALLOC(2*sizeof(char*));
+	ret = OurImports->StringToHostList(ctx->node);
 	if (ret == NULL) {
-		syslog(LOG_ERR, "out of memory");
+		LOG(PIL_CRIT, "%s: out of memory", __FUNCTION__);
 	} else {
-		ret[1]=NULL;
-		ret[0]=STRDUP(ctx->node);
-		if (ret[0] == NULL) {
-			syslog(LOG_ERR, "out of memory");
-			FREE(ret);
-			ret = NULL;
-		} else {
-			g_strdown(ret[0]);
-		}
+		g_strdown(ret[0]);
 	}
 
 	return(ret);
 } /* end si_hostlist() */
 
-/*
- * nw_rpc100s_free_hostlist - free the result from nw_rpc100s_hostlist() 
- */
-static void
-nw_rpc100s_free_hostlist (char ** hlist)
-{
-	char **	hl = hlist;
-	if (hl == NULL) {
-		return;
-	}
-	while (*hl) {
-		FREE(*hl);
-		*hl = NULL;
-		++hl;
-	}
-	FREE(hlist);
-}
-
-
 /*
  *	Parse the given configuration information, and stash it away...
  *
@@ -589,70 +448,10 @@ nw_rpc100s_free_hostlist (char ** hlist)
  *      stonith nodeb NW_RPC100S /dev/ttyS1 nodec 0
  */
 
-static int
-RPC_parse_config_info(struct NW_RPC100S* ctx, const char * info)
+/*static int
+RPC_parse_config_info(struct pluginDevice* ctx, const char * info)
 {
-	char *copy;
-	char *token;
-
-	if (ctx->config) {
-		/* The module is already configured. */
-		return(S_OOPS);
-	}
-
-	/* strtok() is nice to use to parse a string with 
-	   (other than it isn't threadsafe), but it is destructive, so
-	   we're going to alloc our own private little copy for the
-	   duration of this function.
-	*/
-
-	copy = STRDUP(info);
-	if (!copy) {
-		syslog(LOG_ERR, "out of memory");
-		return S_OOPS;
-	}
-
-	/* Grab the serial device */
-	token = strtok (copy, " \t");
-	if (!token) {
-		syslog(LOG_ERR, "%s: Can't find serial device on config line '%s'",
-		       RPCid, info);
-		goto token_error;		
-	}
-
-	ctx->device = STRDUP(token);
-	if (!ctx->device) {
-		syslog(LOG_ERR, "out of memory");
-		goto token_error;
-	}
-
-
-	/* Grab <nodename>  */
-	token = strtok (NULL, " \t");
-	if (!token) {
-		syslog(LOG_ERR, "%s: Can't find node name on config line '%s'",
-		       RPCid, info);
-		goto token_error;		
-	}
-
-	ctx->node = STRDUP(token);
-	if (!ctx->node) {
-		syslog(LOG_ERR, "out of memory");
-		goto token_error;
-	}
-	
-		
-	/* free our private copy of the string we've been destructively 
-	   parsing with strtok()
-	*/
-	FREE(copy);
-	ctx->config = 1;
-	return S_OK;
-
-token_error:
-	FREE(copy);
-	return(S_BADCONFIG);
-}
+}*/
 
 
 /*
@@ -670,17 +469,22 @@ token_error:
  *
  */
 static int
-RPCConnect(struct NW_RPC100S * ctx)
+RPCConnect(struct pluginDevice * ctx)
 {
   	  
 	/* Open the serial port if it isn't already open */
 	if (ctx->fd < 0) {
 		struct termios tio;
 
+		if (OurImports->TtyLock(ctx->device) < 0) {
+			LOG(PIL_CRIT, "%s: TtyLock failed.", pluginid);
+			return S_OOPS;
+		}
+
 		ctx->fd = open (ctx->device, O_RDWR);
 		if (ctx->fd <0) {
-			syslog (LOG_ERR, "%s: Can't open %s : %s",
-				RPCid, ctx->device, strerror(errno));
+			LOG(PIL_CRIT, "%s: Can't open %s : %s"
+			,	pluginid, ctx->device, strerror(errno));
 			return S_OOPS;
 		}
 
@@ -699,17 +503,19 @@ RPCConnect(struct NW_RPC100S * ctx)
 		tio.c_lflag = ICANON;
 
 		if (tcsetattr (ctx->fd, TCSANOW, &tio) < 0) {
-			syslog (LOG_ERR, "%s: Can't set attributes %s : %s",
-				RPCid, ctx->device, strerror(errno));
+			LOG(PIL_CRIT, "%s: Can't set attributes %s : %s"
+			,	pluginid, ctx->device, strerror(errno));
 			close (ctx->fd);
+			OurImports->TtyUnlock(ctx->device);
 			ctx->fd=-1;
 			return S_OOPS;
 		}
 		/* flush all data to and fro the serial port before we start */
 		if (tcflush (ctx->fd, TCIOFLUSH) < 0) {
-			syslog (LOG_ERR, "%s: Can't flush %s : %s",
-				RPCid, ctx->device, strerror(errno));
+			LOG(PIL_CRIT, "%s: Can't flush %s : %s"
+			,	pluginid, ctx->device, strerror(errno));
 			close (ctx->fd);
+			OurImports->TtyUnlock(ctx->device);
 			ctx->fd=-1;
 			return S_OOPS;		
 		}
@@ -718,20 +524,26 @@ RPCConnect(struct NW_RPC100S * ctx)
 
 
 	/* Send a BOGUS string */
-	SEND("//0,0,BOGUS;\r\n", 10);
+	SENDCMD("//0,0,BOGUS;\r\n", 10);
 	
 	/* Should reply with "Invalid Command" */
-	if (gbl_debug) printf ("Waiting for \"Invalid Entry\"n");
-	EXPECT(NWtokInvalidEntry, 12);
-	if (gbl_debug) printf ("Got Invalid Entry\n");
-	EXPECT(NWtokCRNL, 2);
-	if (gbl_debug) printf ("Got NL\n");
+	if (Debug) {
+		LOG(PIL_DEBUG, "Waiting for \"Invalid Entry\"");
+	}
+	EXPECT(ctx->fd, NWtokInvalidEntry, 12);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got Invalid Entry");
+	}
+	EXPECT(ctx->fd, NWtokCRNL, 2);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Got NL");
+	}
 	   
   return(S_OK);
 }
 
 static int
-RPCDisconnect(struct NW_RPC100S * ctx)
+RPCDisconnect(struct pluginDevice * ctx)
 {
 
   if (ctx->fd >= 0) {
@@ -740,6 +552,9 @@ RPCDisconnect(struct NW_RPC100S * ctx)
     */
     tcflush(ctx->fd, TCIOFLUSH);
     close (ctx->fd);
+    if (ctx->device != NULL) {
+      OurImports->TtyUnlock(ctx->device);
+    }
   }
   ctx->fd = -1;
 
@@ -755,19 +570,14 @@ RPCDisconnect(struct NW_RPC100S * ctx)
  * 
  */
 static int
-RPCNametoOutlet ( struct NW_RPC100S * ctx, const char * host )
+RPCNametoOutlet ( struct pluginDevice * ctx, const char * host )
 {
-	char *shost;
 	int rc = -1;
 	
-	if ( (shost = strdup(host)) == NULL) {
-		syslog(LOG_ERR, "strdup failed in RPCNametoOutlet");
-		return -1;
-	}
-	if (!strcmp(ctx->node, host))
+	if (!strcasecmp(ctx->node, host)) {
 		rc = 0;
+	}
 
-	free(shost);
 	return rc;
 }
 
@@ -779,36 +589,31 @@ RPCNametoOutlet ( struct NW_RPC100S * ctx, const char * host )
  *          on the stonith device.
  */
 static int
-nw_rpc100s_reset_req(Stonith * s, int request, const char * host)
+nw_rpc100s_reset_req(StonithPlugin * s, int request, const char * host)
 {
 	int	rc = S_OK;
 	int	lorc = S_OK;
 	int outletnum = -1;
-	struct NW_RPC100S*	ctx;
+	struct pluginDevice*	ctx;
 	
-	if (gbl_debug) printf ("Calling nw_rpc100s_reset (%s)\n", RPCid);
+	if (Debug) {
+		LOG(PIL_DEBUG, "Calling nw_rpc100s_reset (%s)", pluginid);
+	}
 	
-	if (!ISNWRPC100S(s)) {
-		syslog(LOG_ERR, "invalid argument to RPC_reset_host");
-		return(S_OOPS);
-	}
-	if (!ISCONFIGED(s)) {
-		syslog(LOG_ERR
-		,	"unconfigured stonith object in RPC_reset_host");
-		return(S_OOPS);
-	}
-	ctx = (struct NW_RPC100S*) s->pinfo;
+	ERRIFNOTCONFIGED(s,S_OOPS);
+
+	ctx = (struct pluginDevice*) s;
 
 	if ((rc = RPCConnect(ctx)) != S_OK) {
 		return(rc);
 	}
 
 	outletnum = RPCNametoOutlet(ctx, host);
+	LOG(PIL_DEBUG, "zk:outletname=%d", outletnum);
 
 	if (outletnum < 0) {
-		syslog(LOG_WARNING, _("%s %s "
-				      "doesn't control host [%s]."), 
-		       ctx->idinfo, ctx->unitid, host);
+		LOG(PIL_WARN, "%s doesn't control host [%s]"
+		,	ctx->device, host);
 		RPCDisconnect(ctx);
 		return(S_BADHOST);
 	}
@@ -839,99 +644,81 @@ nw_rpc100s_reset_req(Stonith * s, int request, const char * host)
 }
 
 /*
- *	Parse the information in the given configuration file,
+ *	Parse the information in the given string 
  *	and stash it away...
  */
 static int
-nw_rpc100s_set_config_file(Stonith* s, const char * configname)
+nw_rpc100s_set_config(StonithPlugin* s, StonithNVpair *list)
 {
-	FILE *	cfgfile;
+	struct pluginDevice*	ctx;
+	StonithNamesToGet	namestocopy [] =
+	{	{ST_TTYDEV,	NULL}
+	,	{ST_HOSTLIST,	NULL}
+	,	{NULL,		NULL}
+	};
+	int rc;
 
-	char	cfgline[256];
 
-	struct NW_RPC100S*	ctx;
-
-	if (!ISNWRPC100S(s)) {
-		syslog(LOG_ERR, "invalid argument to RPC_set_configfile");
-		return(S_OOPS);
-	}
-	ctx = (struct NW_RPC100S*) s->pinfo;
-
-	if ((cfgfile = fopen(configname, "r")) == NULL)  {
-		syslog(LOG_ERR, _("Cannot open %s"), configname);
-		return(S_BADCONFIG);
+	ERRIFWRONGDEV(s,S_OOPS);
+	if (s->isconfigured) {
+		return S_OOPS;
 	}
 
-	while (fgets(cfgline, sizeof(cfgline), cfgfile) != NULL){
-
-		RPC_parse_config_info(ctx, cfgline);
+	ctx = (struct pluginDevice*) s;
+	
+	if ((rc = OurImports->CopyAllValues(namestocopy, list)) != S_OK) {
+		return rc;
 	}
-	return(S_BADCONFIG);
+	ctx->device = namestocopy[0].s_value;
+	ctx->node = namestocopy[1].s_value;
+
+	return S_OK;
 }
 
 /*
- *	nw_rpc100s_set_config_info - API entry point to process one line of config info 
- *       for this particular device.
- *
- *      Parse the config information in the given string, and stash it away...
- *
+ * Return STONITH config vars
  */
-static int
-nw_rpc100s_set_config_info(Stonith* s, const char * info)
+static const char **
+nw_rpc100s_get_confignames(StonithPlugin* p)
 {
-	struct NW_RPC100S* ctx;
-
-	if (!ISNWRPC100S(s)) {
-		syslog(LOG_ERR, "RPC_provide_config_info: invalid argument");
-		return(S_OOPS);
-	}
-	ctx = (struct NW_RPC100S *)s->pinfo;
-
-	return(RPC_parse_config_info(ctx, info));
+	static const char *	RpcParams[] = {ST_TTYDEV , ST_HOSTLIST, NULL };
+	return RpcParams;
 }
 
+
+
 /*
  * nw_rpc100s_getinfo - API entry point to retrieve something from the handle
  */
 static const char *
-nw_rpc100s_getinfo(Stonith * s, int reqtype)
+nw_rpc100s_getinfo(StonithPlugin * s, int reqtype)
 {
-	struct NW_RPC100S* ctx;
+	struct pluginDevice* ctx;
 	const char *		ret;
 
-	if (!ISNWRPC100S(s)) {
-		syslog(LOG_ERR, "RPC_idinfo: invalid argument");
-		return NULL;
-	}
+	ERRIFWRONGDEV(s,NULL);
+
 	/*
 	 *	We look in the ST_TEXTDOMAIN catalog for our messages
 	 */
-	ctx = (struct NW_RPC100S *)s->pinfo;
+	ctx = (struct pluginDevice *)s;
 
 	switch (reqtype) {
 		case ST_DEVICEID:
 			ret = ctx->idinfo;
 			break;
-
-		case ST_CONF_INFO_SYNTAX:
-			ret = _("<serial_device> <node>\n"
-			"All tokens are white-space delimited.\n");
+		case ST_DEVICENAME:
+			ret = ctx->device;
 			break;
-
-		case ST_CONF_FILE_SYNTAX:
-			ret = _("<serial_device> <node>\n"
-			"All tokens are white-space delimited.\n"
-			"Blank lines and lines beginning with # are ignored");
-			break;
-
 		case ST_DEVICEDESCR:
-			ret = _("Micro Energetics Night/Ware RPC100S");
+			ret = "Micro Energetics Night/Ware RPC100S";
 			break;
-
 		case ST_DEVICEURL:
-			ret = "http://microenergeticscorp.com/";
+			ret = "http://www.microenergeticscorp.com/";
+			break;
+		case ST_CONF_XML:		/* XML metadata */
+			ret = nw_rpc100sXML;
 			break;
-
 		default:
 			ret = NULL;
 			break;
@@ -943,17 +730,15 @@ nw_rpc100s_getinfo(Stonith * s, int reqtype)
  * nw_rpc100s_destroy - API entry point to destroy a NW_RPC100S Stonith object.
  */
 static void
-nw_rpc100s_destroy(Stonith *s)
+nw_rpc100s_destroy(StonithPlugin *s)
 {
-	struct NW_RPC100S* ctx;
+	struct pluginDevice* ctx;
 
-	if (!ISNWRPC100S(s)) {
-		syslog(LOG_ERR, "nw_rpc100s_del: invalid argument");
-		return;
-	}
-	ctx = (struct NW_RPC100S *)s->pinfo;
+	VOIDERRIFWRONGDEV(s);
 
-	ctx->RPCid = NOTrpcid;
+	ctx = (struct pluginDevice *)s;
+
+	ctx->pluginid = NOTrpcid;
 
 	/*  close the fd if open and set ctx->fd to invalid */
 	RPCDisconnect(ctx);
@@ -962,39 +747,33 @@ nw_rpc100s_destroy(Stonith *s)
 		FREE(ctx->device);
 		ctx->device = NULL;
 	}
-	if (ctx->idinfo != NULL) {
-		FREE(ctx->idinfo);
-		ctx->idinfo = NULL;
-	}
-	if (ctx->unitid != NULL) {
-		FREE(ctx->unitid);
-		ctx->unitid = NULL;
+	if (ctx->node != NULL) {
+		FREE(ctx->node);
+		ctx->node = NULL;
 	}
+	FREE(ctx);
 }
 
 /* 
- * nw_rpc100s_new - API entry point called to create a new NW_RPC100S Stonith device
- *          object. 
+ * nw_rpc100s_new - API entry point called to create a new NW_RPC100S Stonith
+ * device object. 
  */
-static void *
-nw_rpc100s_new(void)
+static StonithPlugin *
+nw_rpc100s_new(const char *subplugin)
 {
-	struct NW_RPC100S*	ctx = MALLOCT(struct NW_RPC100S);
+	struct pluginDevice*	ctx = MALLOCT(struct pluginDevice);
 
 	if (ctx == NULL) {
-		syslog(LOG_ERR, "out of memory");
+		LOG(PIL_CRIT, "out of memory");
 		return(NULL);
 	}
 	memset(ctx, 0, sizeof(*ctx));
-	ctx->RPCid = RPCid;
+	ctx->pluginid = pluginid;
 	ctx->fd = -1;
-	ctx->config = 0;
 	ctx->device = NULL;
 	ctx->node = NULL;
-	ctx->idinfo = NULL;
-	ctx->unitid = NULL;
-	REPLSTR(ctx->idinfo, DEVICE);
-	REPLSTR(ctx->unitid, "unknown");
+	ctx->idinfo = DEVICE;
+	ctx->sp.s_ops = &nw_rpc100sOps;
 
-	return((void *)ctx);
+	return &(ctx->sp);
 }
