@@ -1,3 +1,4 @@
+/* $Id: meatclient.c,v 1.9 2005/09/05 02:59:42 msoffen Exp $ */
 /*
  * Stonith client for Human Operator Stonith device
  *
@@ -23,7 +24,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -102,8 +103,7 @@ main(int argc, char** argv)
 		fd = open(meatpipe, O_WRONLY | O_NONBLOCK);
 
 		if (fd < 0) {
-			snprintf(line, sizeof(line)
-			,	"Meatware_IPC failed: %s", meatpipe);
+			sprintf(line, "Meatware_IPC failed: %s", meatpipe);
 			perror(line);
 			exit(S_BADHOST);
 		}
