@@ -18,7 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
-#include <lha_internal.h>
 #include <glib.h>
 #include <clplumbing/cl_log.h>
 #include <clplumbing/ipc.h>
@@ -27,6 +26,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <ha_config.h>
 #include <clplumbing/loggingdaemon.h>
 #include <syslog.h>
 #include <string.h>
@@ -73,7 +73,7 @@ send_log_msg(gpointer data)
 	priority = LOG_INFO;
 	msgstring[0]=0;
 	
-	snprintf(msgstring,  sizeof(msgstring),"Message %d", count++);
+	sprintf(msgstring, "Message %d", count++);
 	fprintf(stderr, "sending %s\n", msgstring);
 	if (LogToDaemon(priority, msgstring,MAXMSGSIZE, FALSE) != HA_OK){			
 		printf("sending out messge %d failed\n", count);
@@ -107,7 +107,7 @@ main(int argc, char** argv)
 	
 	maxcount = atoi(argv[1]);
 	
-	cl_log_set_facility(HA_LOG_FACILITY);
+	cl_log_set_facility(LOG_LOCAL7);
 	cl_log_set_uselogd(TRUE);
 	
 	if(!cl_log_test_logd()){
