@@ -1,4 +1,4 @@
-/* $Id: lrmadmin.c,v 1.35 2005/06/22 06:53:41 sunjd Exp $ */
+/* $Id: lrmadmin.c,v 1.41 2006/06/26 01:48:20 sunjd Exp $ */
 /* File: lrmadmin.c
  * Description: A adminstration tool for Local Resource Manager
  *
@@ -46,7 +46,7 @@
 #include <clplumbing/GSource.h>
 #include <clplumbing/Gmain_timeout.h>
 
-const char * optstring = "AD:dEF:d:sg:M:P:c:S:LI:CT:h";
+const char * optstring = "AD:dEF:d:sg:M:P:c:S:LI:CT:n:h";
 
 #ifdef HAVE_GETOPT_H
 static struct option long_options[] = {
@@ -70,6 +70,7 @@ static struct option long_options[] = {
 
 GMainLoop *mainloop = NULL;
 const char * lrmadmin_name = "lrmadmin";
+const char * fake_name = NULL;
 /* 20 is the length limit for a argv[x] */
 const int ARGVI_MAX_LEN = 48;
 
@@ -91,14 +92,38 @@ typedef enum {
 	HELP
 } lrmadmin_cmd_t;
 
-static const char * status_msg[5] = {
+#define nullcheck(p)       ((p) ? (p) : "<null>")
+static const char * status_msg[6] = {
+	"pending",		  /* LRM_OP_PENDING	 */
 	"succeed", 		  /* LRM_OP_DONE         */
         "cancelled", 		  /* LRM_OP_CANCELLED    */
         "timeout",		  /* LRM_OP_TIMEOUT 	 */
         "not Supported",	  /* LRM_OP_NOTSUPPORTED */
-        "failed due to an error"   /* LRM_OP_ERROR	 */
+        "failed due to an error"  /* LRM_OP_ERROR	 */
 };
 
+static const char * rc_msg[] = {
+        "unknown error",
+        "no ra",
+        "ok",
+        "unknown error",
+        "invalid parameter",
+        "unimplement feature",
+        "insufficient priority",
+        "not installed",
+        "not configured",
+        "not running",
+        "running master",
+        "failed master",
+	"invalid rc",
+        /* For status command only */
+        "daemon dead1",
+        "daemon dead2",
+        "daemon stopped",
+        "status unknow"
+};
+
+
 static gboolean QUIT_GETOPT = FALSE;
 static lrmadmin_cmd_t lrmadmin_cmd = NULL_OP;
 static gboolean ASYN_OPS = FALSE; 
@@ -111,13 +136,13 @@ const char * simple_help_screen =
 "         {-D|--delete} <rscid>\n"
 "         {-F|--flush} <rscid>\n"
 "         {-E|--execute} <rscid> <operator> <timeout> <interval> <target_rc|EVERYTIME|CHANGED> [<operator_parameters_list>]\n"
-"         {-S|--state} <rscid>\n"
+"         {-S|--state} <rscid> [-n <fake_name>]\n"
 "         {-L|--listall}\n"
 "         {-I|--information} <rsc_id>\n"
 "         {-C|--raclass_supported}\n"
-"         {-T|--ratype_supported} <raclss>\n"
-"         {-M|--metadata} <raclss> <ratype> <provider|NULL>\n"
-"         {-P|--provider} <raclss> <ratype>\n"
+"         {-T|--ratype_supported} <raclass>\n"
+"         {-M|--metadata} <raclass> <ratype> <provider|NULL>\n"
+"         {-P|--provider} <raclass> <ratype>\n"
 "         {-h|--help}\n";
 
 #define OPTION_OBSCURE_CHECK \
@@ -132,8 +157,8 @@ static int resource_operation(ll_lrm_t * lrmd, int argc, int optind,
 static int add_resource(ll_lrm_t * lrmd, int argc, int optind, char * argv[]);
 static int transfer_cmd_params(int amount, int start, char * argv[], 
 			   const char * class, GHashTable ** params_ht);
-static void g_print_stringitem(gpointer data, gpointer user_data);
-static void g_print_rainfo_item(gpointer data, gpointer user_data);
+static void g_print_stringitem_and_free(gpointer data, gpointer user_data);
+static void g_print_rainfo_item_and_free(gpointer data, gpointer user_data);
 static void g_print_ops(gpointer data, gpointer user_data);
 static void g_get_rsc_description(gpointer data, gpointer user_data);
 static void print_rsc_inf(lrm_rsc_t * lrmrsc);
@@ -166,6 +191,7 @@ int main(int argc, char **argv)
 		*ratype_list = 0,
 		*rscid_list;
 	char raclass[20];
+	const char * login_name = lrmadmin_name;
 
 	/* Prevent getopt_long to print error message on stderr isself */
 	/*opterr = 0; */  
@@ -270,9 +296,14 @@ int main(int argc, char **argv)
 				lrmadmin_cmd = INF_RSC;
 				break;
 
+			case 'n':
+				if (optarg) {
+					fake_name = optarg;
+				}
+				break;
+
 			case 'h':
 				OPTION_OBSCURE_CHECK 
-				/* print detailed help screen? */
 				printf("%s",simple_help_screen);
 				return 0;
 
@@ -299,7 +330,10 @@ int main(int argc, char **argv)
 
 	lrmd->lrm_ops->set_lrm_callback(lrmd, lrm_op_done_callback);
 
-        if (lrmd->lrm_ops->signon(lrmd, lrmadmin_name) != 1) { /* != HA_OK */
+	if (fake_name != NULL) {
+		login_name = fake_name;
+	}
+        if (lrmd->lrm_ops->signon(lrmd, login_name) != 1) { /* != HA_OK */
 		printf("lrmd daemon is not running.\n");
 		if (lrmadmin_cmd == DAEMON_OP) { 
 			return LSB_STATUS_STOPPED;
@@ -392,6 +426,7 @@ int main(int argc, char **argv)
 					printf("Failed to flush.\n");
 					ret_value = -3;
 				}
+				lrm_free_rsc(lrm_rsc);
 			}
 
 			ASYN_OPS = FALSE;
@@ -403,7 +438,7 @@ int main(int argc, char **argv)
 			printf("Support %d RA classes\n", 
 					g_list_length(raclass_list));
 			if (raclass_list) {
-				g_list_foreach(raclass_list, g_print_stringitem,
+				g_list_foreach(raclass_list, g_print_stringitem_and_free,
 						NULL);
 				g_list_free(raclass_list);
 				ret_value = LSB_EXIT_OK;
@@ -418,14 +453,11 @@ int main(int argc, char **argv)
 		case RATYPE_SUPPORTED:
 		     	ratype_list = lrmd->lrm_ops->
 				get_rsc_type_supported(lrmd, raclass);
-			printf("List size: %d\n", g_list_length(ratype_list));
+			printf("The are %d RAs:\n", g_list_length(ratype_list));
 			if (ratype_list) {
-				g_list_foreach(ratype_list, g_print_rainfo_item,
+				g_list_foreach(ratype_list, g_print_rainfo_item_and_free,
 						NULL);
-				/* g_list_free(ratype_list); */
-			} else {
-				printf("For this RA class, no any RA type is "
-					"supported\n");
+				g_list_free(ratype_list);
 			}
 
 			ASYN_OPS = FALSE;
@@ -450,7 +482,7 @@ int main(int argc, char **argv)
 				ret_value = -3;
 			} else {
 				print_rsc_inf(lrm_rsc);
-				g_free(lrm_rsc);
+				lrm_free_rsc(lrm_rsc);
 			}
 
 			ASYN_OPS = FALSE;
@@ -469,11 +501,16 @@ int main(int argc, char **argv)
 					 cur_state==LRM_RSC_IDLE?
 					 "LRM_RSC_IDLE":"LRM_RSC_BUSY");
 								
+				printf("The resource %d operations' "
+					"information:\n"
+					, g_list_length(ops_queue));
 				if (ops_queue) {
-					g_list_foreach(ops_queue, g_print_ops, 
-							NULL);
-					g_list_free(ops_queue);
+					g_list_foreach(ops_queue,
+						       g_print_ops, 
+						       NULL);
+					lrm_free_op_list(ops_queue);
 				}
+				lrm_free_rsc(lrm_rsc);
 			}
 
 			ASYN_OPS = FALSE;
@@ -541,11 +578,11 @@ lrm_op_done_callback(lrm_op_t* op)
 		printf("operation status:%s\n", status_msg[LRM_OP_DONE]);
 	} else {
 		printf("operation status:%s\n", status_msg[(op->op_status 
-			- LRM_OP_DONE) % DIMOF(status_msg)]);
+			- LRM_OP_PENDING) % DIMOF(status_msg)]);
 	}
 	printf("op_status: %d\n", op->op_status);
 	printf("return code: %d\n", op->rc);
-	printf("output data: \n%s\n", op->output);
+	printf("output data: \n%s\n", (op->output ? op->output : "[null]"));
 	printf("---------------------------------------\n\n");
 	ret_value = op->op_status;	
 }
@@ -603,7 +640,7 @@ resource_operation(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	op.params = params_ht;
 
 	call_id = lrm_rsc->ops->perform_op(lrm_rsc, &op);
-	/* g_free(lrm_rsc);   don't need to free it ? */
+	lrm_free_rsc(lrm_rsc);
 	if (params_ht) {
 		g_hash_table_foreach(params_ht, free_stritem_of_hashtable, NULL);
 		g_hash_table_destroy(params_ht);
@@ -629,7 +666,7 @@ ra_metadata(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 
 	metadata = lrmd->lrm_ops->get_rsc_type_metadata(lrmd, class, type, provider);
 	if (NULL!=metadata) {
-		printf ("metadata of %s(%s) is: %s\n",type,class,metadata);
+		printf ("%s\n", metadata);
 		g_free (metadata);
 	}
 	return 0;
@@ -811,29 +848,47 @@ params_hashtable_to_str(const char * class, GHashTable * ht)
 }
 
 static void
-g_print_stringitem(gpointer data, gpointer user_data)
+g_print_stringitem_and_free(gpointer data, gpointer user_data)
 {
 	printf("%s\n", (char*)data);
-	g_free(data);  /*  ?  */
+	g_free(data);
 }
 
 static void
-g_print_rainfo_item(gpointer data, gpointer user_data)
+g_print_rainfo_item_and_free(gpointer data, gpointer user_data)
 {
-/*	rsc_info_t * rsc_info = (rsc_info_t *) data; */
-	printf("RA type name: %s\n", (char *)data);
-/*
-	printf("RA type name: %s  Version: %s\n", 
-		rsc_info->rsc_type, rsc_info->version);
-*/
-	g_free(data); /*  ?  */
+	printf("%s\n", (char *)data);
+	g_free(data);
 }
 
+
 static void
 g_print_ops(gpointer data, gpointer user_data)
 {
-	printf("%s  ", (char*)data);
-	g_free(data);  /*  ?  */
+	lrm_op_t* op = (lrm_op_t*)data;
+	GString * param_gstr;
+
+	if (NULL == op) {
+		cl_log(LOG_ERR, "%s:%d: op==NULL"
+			, __FUNCTION__, __LINE__);
+		return;
+	}
+
+	param_gstr = g_string_new("");
+	g_hash_table_foreach(op->params, ocf_params_hash_to_str, &param_gstr);
+
+	printf("   operation '%s' [call_id=%d]:\n"
+	       "      start_delay=%d, interval=%d, timeout=%d, app_name=%s\n"
+	       "      rc=%d (%s), op_status=%d (%s)\n"
+	       "      parameters: %s\n"
+		, nullcheck(op->op_type), op->call_id
+		, op->start_delay, op->interval, op->timeout
+		, nullcheck(op->app_name), op->rc
+		, rc_msg[(op->rc-EXECRA_EXEC_UNKNOWN_ERROR) % DIMOF(rc_msg)]
+		, op->op_status
+		, status_msg[(op->op_status-LRM_OP_PENDING) % DIMOF(status_msg)]
+		, param_gstr->str);
+	g_string_free(param_gstr, TRUE);
 }
 
 static void
@@ -853,12 +908,12 @@ g_get_rsc_description(gpointer data, gpointer user_data)
 	lrm_rsc = lrmd->lrm_ops->get_rsc(lrmd, rsc_id_tmp);
 	if (lrm_rsc) {
 		print_rsc_inf(lrm_rsc);
-		g_free(lrm_rsc);   /* ? */
+		lrm_free_rsc(lrm_rsc); 
 	} else
 		cl_log(LOG_ERR, "There is a invalid resource id %s.", 
 			rsc_id_tmp);
 	
-	g_free(data); /* ? */
+	g_free(data);
 }
 
 static void
@@ -939,6 +994,24 @@ get_lrm_rsc(ll_lrm_t * lrmd, char * rscid)
 
 /*
  * $Log: lrmadmin.c,v $
+ * Revision 1.41  2006/06/26 01:48:20  sunjd
+ * (bug#1301) Support fake login name; adjust memory freeing
+ *
+ * Revision 1.40  2006/06/22 21:00:57  davidlee
+ * Beware null pointer in printf(%s)
+ *
+ * Revision 1.39  2006/06/22 10:33:22  sunjd
+ * (bug1301): make a better output
+ *
+ * Revision 1.38  2006/06/22 05:56:31  sunjd
+ * (bug1301): add the capacity of get_cur_state; polish on memory free
+ *
+ * Revision 1.37  2006/05/16 09:40:37  sunjd
+ * fix a minor typo; remove the hint for metadata output
+ *
+ * Revision 1.36  2006/05/15 03:35:43  sunjd
+ * minor changes: typoes and formats
+ *
  * Revision 1.35  2005/06/22 06:53:41  sunjd
  * handling for no parameter
  *
