@@ -1,4 +1,4 @@
-/* $Id: ccmlib_memapi.c,v 1.38 2005/07/30 02:33:08 alan Exp $ */
+/* $Id: ccmlib_memapi.c,v 1.44 2006/04/14 02:41:30 zhenh Exp $ */
 /* 
  * ccmlib_memapi.c: Consensus Cluster Membership API
  *
@@ -118,49 +118,18 @@ static const char *llm_get_Id_from_Uuid(ccm_llm_t *stuff, uint uuid);
 
 
 static void
-init_llm(mbr_private_t *mem, struct IPC_MESSAGE *msg)
+on_llm_msg(mbr_private_t *mem, struct IPC_MESSAGE *msg)
 {
 	unsigned long len = msg->msg_len;
-	int	numnodes;
-
+	
+	if (mem->llm != NULL) {
+		g_free(mem->llm);
+	}
 	mem->llm = (ccm_llm_t *)g_malloc(len);
 	memcpy(mem->llm, msg->msg_body, len);
-
-	mem->bornon = g_hash_table_new(g_direct_hash, 
-				g_direct_equal);
-
-	numnodes = CLLM_GET_NODECOUNT(mem->llm);
-	mem->cookie = NULL;
-
-
 	return;
 }
 
-static void
-init_bornon(mbr_private_t *private, 
-		struct IPC_MESSAGE *msg)
-{
-	ccm_born_t *born;
-	int numnodes, i, n;
-	struct born_s *bornon;
-
-	numnodes = CLLM_GET_NODECOUNT(private->llm);
-
-	born = (ccm_born_t *)msg->msg_body;
-
-	n = born->n;
-	assert(msg->msg_len == sizeof(ccm_born_t)
-			+n*sizeof(struct born_s));
-	bornon = born->born;
-	for (i = 0 ; i < n; i++) {
-		assert(bornon[i].index <= numnodes);
-		g_hash_table_insert(private->bornon, 
-			GINT_TO_POINTER(CLLM_GET_UUID(private->llm, 
-					bornon[i].index)),
-			GINT_TO_POINTER(bornon[i].bornon+1));
-	}
-	return;
-}
 
 
 static void
@@ -178,10 +147,10 @@ reset_llm(mbr_private_t *private)
 }
 
 static int
-init_llmborn(mbr_private_t *private)
+init_llm(mbr_private_t *private)
 {
 	struct IPC_CHANNEL *ch;
-	int 	sockfd, i=0, ret;
+	int 	sockfd, ret;
 	struct IPC_MESSAGE *msg;
 
 	if(private->llm) {
@@ -190,14 +159,7 @@ init_llmborn(mbr_private_t *private)
 
 	ch 	   = private->channel;
 	sockfd = ch->ops->get_recv_select_fd(ch);
-
-	/* receive the initiale low level membership 
-	*  information in the first iteration, and 
-	*  recieve the bornon information in the 
-	*  second iteration 
-	*/
-	while( i < 2) {
-		
+	while(1) {
 		if(ch->ops->waitin(ch) != IPC_OK){
 			ch->ops->destroy(ch);
 			return -1;
@@ -207,22 +169,21 @@ init_llmborn(mbr_private_t *private)
 			fprintf(stderr, "connection denied\n");
 			return -1;
 		}
-	 	if(ret == IPC_FAIL){
+ 		if(ret == IPC_FAIL){
 			fprintf(stderr, ".");
 			cl_shortsleep();
 			continue;
 		}
-
-		switch(i) {
-		case 0: init_llm(private, msg);
-			break;
-		case 1: init_bornon(private, msg);
-			private->client_report = TRUE;
-			break;
-		}
-		i++;
-		msg->msg_done(msg);
+		break;
 	}
+	on_llm_msg(private, msg);
+	
+	private->bornon = g_hash_table_new(g_direct_hash, 
+				g_direct_equal);
+	private->cookie = NULL;
+	
+	private->client_report = TRUE;
+	msg->msg_done(msg);
 	return 0;
 }
 
@@ -252,8 +213,7 @@ already_present(oc_node_t *arr, uint size, oc_node_t node)
 {
 	uint i;
 	for ( i = 0 ; i < size ; i ++ ) {
-		if(arr[i].node_id == node.node_id &&
-			arr[i].node_born_on >= node.node_born_on) {
+		if(arr[i].node_id == node.node_id) {
 			return TRUE;
 		}
 	}
@@ -307,7 +267,7 @@ get_new_membership(mbr_private_t *private,
 {
 	mbr_track_t *newmbr, *oldmbr;
 	int trans, i, j, in_index, out_index, born;
-	int n_members,uuid;
+	int n_members;
 	
 	int n_nodes = CLLM_GET_NODECOUNT(private->llm);
 	
@@ -325,19 +285,18 @@ get_new_membership(mbr_private_t *private,
 
 	for ( i = 0 ; i < n_members; i++ ) {
 		const char *uname = NULL;
-		gpointer	gborn; /* Help make gcc warning go away */
-		uuid =  CLLM_GET_UUID(private->llm, mbrinfo->member[i]);
+		int	index;
 
-		uname = llm_get_Id_from_Uuid(private->llm, uuid);
+		index = mbrinfo->member[i].index;
+
+		uname = llm_get_Id_from_Uuid(private->llm, index);
 
 		newmbr->m_mem.m_array[j].node_uname = strdup(uname); 
+		
+		OC_EV_SET_NODEID(newmbr,j,index);
 
-		OC_EV_SET_NODEID(newmbr,j,uuid);
-
-		gborn = g_hash_table_lookup(private->bornon, 
-				GINT_TO_POINTER(mbrinfo->member[i]));
 		/* gborn was an int to begin with - so this is safe */
-		born = POINTER_TO_SIZE_T(gborn);/*pointer cast as int*/
+		born = mbrinfo->member[i].bornon;
 
 		/* if there is already a born entry for the
 		 * node, use it. Otherwise create a born entry
@@ -346,7 +305,7 @@ get_new_membership(mbr_private_t *private,
 		 * NOTE: born==0 implies the entry has not been
 		 * 	initialized.
 		 */
-		OC_EV_SET_BORN(newmbr,j, born==0?trans:(born-1));
+		OC_EV_SET_BORN(newmbr,j, born);
 		j++;
 	}
 	/* sort the m_arry */
@@ -494,7 +453,7 @@ mem_handle_event(class_t *class)
 	private = (mbr_private_t *)class->private;
 	ch 	   = private->channel;
 
-	if(init_llmborn(private)){
+	if(init_llm(private)){
 		return FALSE;
 	}
 	
@@ -637,6 +596,8 @@ mem_handle_event(class_t *class)
 						NULL,NULL);
 			}
 			break;
+		case CCM_LLM:
+			on_llm_msg(private, msg);
 		}
 
 		
@@ -836,7 +797,8 @@ oc_ev_memb_class(oc_ev_callback_t  *fn)
 	private->magiccookie = 0xabcdef;
 	private->client_report = FALSE;
 	private->special = 0; /* no special behaviour */
-
+	private->llm = NULL; 
+	
 	attrs = g_hash_table_new(g_str_hash,g_str_equal);
 	g_hash_table_insert(attrs, path, ccmfifo);
 	ch = ipc_channel_constructor(IPC_DOMAIN_SOCKET, attrs);
