@@ -19,7 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <unistd.h>
 #include <glib.h>
@@ -27,7 +27,6 @@
 #include <heartbeat.h>
 #include <clplumbing/cl_log.h>
 #include <clplumbing/cl_syslog.h>
-#include <clplumbing/cl_malloc.h>
 
 #include "mgmt_internal.h"
 #include <lrm/lrm_api.h>
@@ -106,11 +105,10 @@ on_rsc_type(char* argv[], int argc)
 	
 	GList* types;
 	GList* cur;
-	char* ret;
+	char* ret = cl_strdup(MSG_OK);
 
 	ARGC_CHECK(2)
-	
-	ret = cl_strdup(MSG_OK);
+
 	types = lrm->lrm_ops->get_rsc_type_supported(lrm, argv[1]);
 	cur = types;
 	while (cur != NULL) {
