@@ -1,3 +1,4 @@
+/* $Id: recoverymgrd.c,v 1.15 2005/07/27 09:03:24 panjiam Exp $ */
 /*
  * Generic Recovery manager implementation
  * 
@@ -28,7 +29,7 @@
  * 
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <sys/time.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -63,7 +64,7 @@
 #define HBINTERVAL_MSEC		2000
 
 #define CONFIG_FILE	"./recoverymgrd.conf"
-#define PIDFILE		HA_VARRUNDIR "/recoverymgrd.pid"
+#define PIDFILE		VAR_RUN_D "/recoverymgrd.pid"
 #define DEBUG 
 #define         DBGMIN          1
 #define         DBGDETAIL       3
