@@ -1,3 +1,4 @@
+/* $Id: attrd.c,v 1.7 2006/05/16 14:46:25 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 #include <stdio.h>
@@ -35,7 +36,7 @@
 #include <clplumbing/coredumps.h>
 #include <clplumbing/Gmain_timeout.h>
 
-/* #include <lha_internal.h> */
+/* #include <portability.h> */
 #include <ocf/oc_event.h>
 /* #include <ocf/oc_membership.h> */
 
@@ -52,7 +53,7 @@
 
 GMainLoop*  mainloop = NULL;
 const char *attrd_uname = NULL;
-char *attrd_uuid = NULL;
+const char *attrd_uuid = NULL;
 ll_cluster_t	*attrd_cluster_conn;
 gboolean need_shutdown = FALSE;
 
@@ -84,7 +85,6 @@ free_hash_entry(gpointer data)
 	}	
 	crm_free(entry->id);
 	crm_free(entry->set);
-	crm_free(entry->dampen);
 	crm_free(entry->section);
 	if(entry->value != entry->last_value) {
 		crm_free(entry->value);
@@ -98,8 +98,6 @@ free_hash_entry(gpointer data)
 void attrd_ha_callback(HA_Message * msg, void* private_data);
 void attrd_local_callback(HA_Message * msg);
 gboolean attrd_timer_callback(void *user_data);
-gboolean attrd_trigger_update(attr_hash_entry_t *hash_entry);
-void attrd_perform_update(attr_hash_entry_t *hash_entry);
 
 static gboolean
 attrd_shutdown(int nsig, gpointer unused)
@@ -298,18 +296,12 @@ attrd_ha_connection_destroy(gpointer user_data)
 	}
 
 	crm_crit("Lost connection to heartbeat service!");
-	if (mainloop != NULL && g_main_is_running(mainloop)) {
-		g_main_quit(mainloop);
-		return;
-	}
-	exit(LSB_EXIT_OK);	
 }
 
 
 static gboolean
 register_with_ha(void) 
 {
-	const char *const_attrd_uuid = NULL;
 	if(attrd_cluster_conn == NULL) {
 		attrd_cluster_conn = ll_cluster_new("heartbeat");
 	}
@@ -353,42 +345,26 @@ register_with_ha(void)
 	crm_info("Hostname: %s", attrd_uname);
 
 	crm_debug_3("Finding our node uuid");
-	const_attrd_uuid = get_uuid(attrd_cluster_conn, attrd_uname);
-	if(const_attrd_uuid == NULL) {
+	attrd_uuid = get_uuid(attrd_cluster_conn, attrd_uname);
+	if(attrd_uuid == NULL) {
 		crm_err("get_uuid_by_name() failed");
 		return FALSE;
 	}
 	/* copy it so that unget_uuid() doesn't trash the value on us */
-	attrd_uuid = crm_strdup(const_attrd_uuid);
+	attrd_uuid = crm_strdup(attrd_uuid);
 	crm_info("UUID: %s", attrd_uuid);
 
 	return TRUE;
 }
 
-static void
-attrd_cib_connection_destroy(gpointer user_data)
-{
-	if(need_shutdown) {
-	    crm_info("Connection to the CIB terminated...");
-
-	} else {
-	    /* eventually this will trigger a reconnect, not a shutdown */ 
-	    crm_err("Connection to the CIB terminated...");
-	    exit(1);
-	}
-	
-	return;
-}
-
 int
 main(int argc, char ** argv)
 {
 	int flag;
 	int argerr = 0;
 	gboolean was_err = FALSE;
-	char *channel_name = crm_strdup(attrd_channel);
 	
-	crm_log_init(T_ATTRD, LOG_INFO, TRUE, FALSE, 0, NULL);
+	crm_log_init(T_ATTRD);
 	G_main_add_SignalHandler(
 		G_PRIORITY_HIGH, SIGTERM, attrd_shutdown, NULL, NULL);
 	
@@ -437,19 +413,10 @@ main(int argc, char ** argv)
 			was_err = TRUE;
 		}
 	}
-
-	if(was_err == FALSE) {
-	    enum cib_errors rc = cib_conn->cmds->set_connection_dnotify(
-		cib_conn, attrd_cib_connection_destroy);
-	    if(rc != cib_ok) {
-		crm_err("Could not set dnotify callback");
-		was_err = TRUE;
-	    }
-	}
 	
 	if(was_err == FALSE) {
 		int rc = init_server_ipc_comms(
-			channel_name, attrd_connect,
+			crm_strdup(attrd_channel), attrd_connect,
 			default_ipc_connection_destroy);
 		
 		if(rc != 0) {
@@ -470,18 +437,7 @@ main(int argc, char ** argv)
 	mainloop = g_main_new(FALSE);
 	g_main_run(mainloop);
 	crm_info("Exiting...");
-
-	attrd_cluster_conn->llc_ops->signoff(attrd_cluster_conn, TRUE);
-	attrd_cluster_conn->llc_ops->delete(attrd_cluster_conn);
 	
-	cib_conn->cmds->signoff(cib_conn);
-	cib_delete(cib_conn);
-
-	g_hash_table_destroy(attr_hash);
-	crm_free(channel_name);
-	crm_free(attrd_uuid);
-	empty_uuid_cache();
-
 	return 0;
 }
 
@@ -501,29 +457,12 @@ attrd_cib_callback(const HA_Message *msg, int call_id, int rc,
 	crm_free(attr);
 }
 
-static void
-log_hash_entry(int level, attr_hash_entry_t *entry, const char *text) 
-{
-	do_crm_log(level, "%s", text);
-	do_crm_log(level, "Set:     %s", entry->section);
-	do_crm_log(level, "Name:    %s", entry->id);
-	do_crm_log(level, "Value:   %s", entry->value);
-	do_crm_log(level, "Timeout: %s", entry->dampen);
-}
-
 static attr_hash_entry_t *
 find_hash_entry(HA_Message * msg) 
 {
 	const char *value = NULL;
 	const char *attr  = ha_msg_value(msg, F_ATTRD_ATTRIBUTE);
-	attr_hash_entry_t *hash_entry = NULL;
-
-	if(attr == NULL) {
-		crm_info("Ignoring message with no attribute name");
-		return NULL;
-	}
-	
-	hash_entry = g_hash_table_lookup(attr_hash, attr);
+	attr_hash_entry_t *hash_entry = g_hash_table_lookup(attr_hash, attr);
 
 	if(hash_entry == NULL) {	
 		/* create one and add it */
@@ -538,72 +477,54 @@ find_hash_entry(HA_Message * msg)
 
 	value = ha_msg_value(msg, F_ATTRD_SET);
 	if(value != NULL) {
-		crm_free(hash_entry->set);
 		hash_entry->set = crm_strdup(value);
 		crm_debug("\t%s->set: %s", attr, value);
 	}
 	
 	value = ha_msg_value(msg, F_ATTRD_SECTION);
-	if(value == NULL) {
-		value = XML_CIB_TAG_STATUS;
+	if(value != NULL) {
+		hash_entry->section = crm_strdup(value);
+		crm_debug("\t%s->section: %s", attr, value);
 	}
-	crm_free(hash_entry->section);
-	hash_entry->section = crm_strdup(value);
-	crm_debug("\t%s->section: %s", attr, value);
 	
 	value = ha_msg_value(msg, F_ATTRD_DAMPEN);
 	if(value != NULL) {
-		crm_free(hash_entry->dampen);
 		hash_entry->dampen = crm_strdup(value);
-
 		hash_entry->timeout = crm_get_msec(value);
 		crm_debug("\t%s->timeout: %s", attr, value);
 	}
 
-	log_hash_entry(LOG_DEBUG_2, hash_entry, "Found (and updated) entry:");
 	return hash_entry;
 }
 
 void
 attrd_ha_callback(HA_Message * msg, void* private_data)
-{
-	attr_hash_entry_t *hash_entry = NULL;
-	const char *from   = ha_msg_value(msg, F_ORIG);
-	const char *op     = ha_msg_value(msg, F_ATTRD_TASK);
-	const char *ignore = ha_msg_value(msg, F_ATTRD_IGNORE_LOCALLY);
-
-	if(ignore == NULL || safe_str_neq(from, attrd_uname)) {
-		crm_info("%s message from %s", op, from);
-		hash_entry = find_hash_entry(msg);
-		stop_attrd_timer(hash_entry);
-		attrd_perform_update(hash_entry);
-	}
-}
-
-void
-attrd_perform_update(attr_hash_entry_t *hash_entry)
 {
 	int rc = cib_ok;
+	attr_hash_entry_t *hash_entry = NULL;
+	const char *from = ha_msg_value(msg, F_ORIG);
+	const char *op   = ha_msg_value(msg, F_ATTRD_TASK);
+	const char *attr = ha_msg_value(msg, F_ATTRD_ATTRIBUTE);
 
-	if(hash_entry == NULL) {
-	    return;
-	    
-	} else if(hash_entry->value == NULL) {
+	crm_info("%s message from %s", op, from);
+	hash_entry = find_hash_entry(msg);
+	stop_attrd_timer(hash_entry);
+	if(hash_entry->value == NULL) {
 		/* delete the attr */
 		rc = delete_attr(cib_conn, cib_none, hash_entry->section, attrd_uuid,
-				 hash_entry->set, NULL, hash_entry->id, NULL, FALSE);
+				 hash_entry->set, NULL, attr, NULL);
 		crm_info("Sent delete %d: %s %s %s",
-			 rc, hash_entry->id, hash_entry->set, hash_entry->section);
+			 rc, attr, hash_entry->set, hash_entry->section);
 		
 	} else {
 		/* send update */
 		rc = update_attr(cib_conn, cib_none, hash_entry->section,
  				 attrd_uuid, hash_entry->set, NULL,
- 				 hash_entry->id, hash_entry->value, FALSE);
-		crm_info("Sent update %d: %s=%s", rc, hash_entry->id, hash_entry->value);
+ 				 hash_entry->id, hash_entry->value);
+		crm_info("Sent update %d: %s=%s", rc, hash_entry->id,hash_entry->value);
 	}
 
-	add_cib_op_callback(rc, FALSE, crm_strdup(hash_entry->id), attrd_cib_callback);
+	add_cib_op_callback(rc, FALSE, crm_strdup(attr), attrd_cib_callback);
 	
 	return;
 }
@@ -630,15 +551,13 @@ attrd_local_callback(HA_Message * msg)
 		return;
 	}
 
-	crm_debug("%s message from %s: %s=%s", op, from, attr, crm_str(value));
+	crm_debug("%s message from %s: %s=%s", op, from, attr, value);
 	hash_entry = find_hash_entry(msg);
-	if(hash_entry == NULL) {
-	    return;
-	}
 	
 	crm_free(hash_entry->last_value);
 	hash_entry->last_value = hash_entry->value;
 
+	value = ha_msg_value(msg, F_ATTRD_VALUE);
 	if(value != NULL) {
 		hash_entry->value = crm_strdup(value);
 
@@ -657,7 +576,7 @@ attrd_local_callback(HA_Message * msg)
 		hash_entry->timer_id = Gmain_timeout_add(
 			hash_entry->timeout, attrd_timer_callback, hash_entry);
 	} else {
-		attrd_trigger_update(hash_entry);
+		attrd_timer_callback(hash_entry);
 	}
 	
 	return;
@@ -665,22 +584,14 @@ attrd_local_callback(HA_Message * msg)
 
 gboolean
 attrd_timer_callback(void *user_data)
-{
- 	stop_attrd_timer(user_data);
-	attrd_trigger_update(user_data);
-	return TRUE;
-}
-
-gboolean
-attrd_trigger_update(attr_hash_entry_t *hash_entry)
 {
 	HA_Message *msg = NULL;
+	attr_hash_entry_t *hash_entry = user_data;
+	stop_attrd_timer(hash_entry);
 
 	/* send HA message to everyone */
 	crm_info("Sending flush op to all hosts for: %s", hash_entry->id);
- 	log_hash_entry(LOG_DEBUG_2, hash_entry, "Sending flush op to all hosts for:");
-
-	msg = ha_msg_new(8);
+	msg = ha_msg_new(4);
 	ha_msg_add(msg, F_TYPE, T_ATTRD);
 	ha_msg_add(msg, F_ORIG, attrd_uname);
 	ha_msg_add(msg, F_ATTRD_TASK, "flush");
@@ -688,15 +599,7 @@ attrd_trigger_update(attr_hash_entry_t *hash_entry)
 	ha_msg_add(msg, F_ATTRD_SET, hash_entry->set);
 	ha_msg_add(msg, F_ATTRD_SECTION, hash_entry->section);
 	ha_msg_add(msg, F_ATTRD_DAMPEN, hash_entry->dampen);
-	ha_msg_add(msg, F_ATTRD_VALUE, hash_entry->value);
-
-	if(hash_entry->timeout <= 0) {
-		ha_msg_add(msg, F_ATTRD_IGNORE_LOCALLY, hash_entry->value);
-		attrd_perform_update(hash_entry);
-	}
-
 	send_ha_message(attrd_cluster_conn, msg, NULL, FALSE);
-	crm_msg_del(msg);
 	
 	return TRUE;
 }
